/*<std-header incl-file-exclusion=ERRORMANAGER_H>

           Copyright 1998, NCR Corporation.  All Rights Reserved.

                               NCR Confidential.


 $Id: ErrorManager.h,v 1.11 1998/05/19 16:58:29 nhall Exp $


*/

#ifndef ERRORMANAGER_H
#define ERRORMANAGER_H


/*  -- do not edit anything above this line --   </std-header>*/

#include <stdlib.h>
#include <iostream.h>
#include <stringType.h>
#include <w_strstream.h>

//
// Each thread has a single instance of this class.
// It is designed to:
//
// 	- accumulate error and stack info when called from INT_PARA_CALL
//	- create a KR_Data_t response based on its current state and
//	  depending on the includeInternalErrors flag, optionally include
//	  internal error messages.
//	- automatically generate some kind of user error if none was specified
//	  the idea being that this is an internal error-- we should always
//	  explicitly specify a user error.
//	- Make use of ParaError objects for raising errors.  These error objects
//	  will allow the inclusion of extra information available at the error's
//	  origin like file name, etc.
//

class Error;
class UserError;
class KR_Data_t;
class ClientErrorManager;
class ServerErrorManager;

class BaseErrorManager {
	
protected:
	ostrstream internalError;
	int internalErrorCode;

	ostrstream userError;
	int userErrorCode;

	ostrstream explanation;

	//
	// SQL Class/Subclass as defined by the SQL standard.
	//
	String SqlClass;
	String SqlSubclass;

	friend class Error;
	friend class UserError;
	friend class ExplainError;

	void RaiseError(Error& );	
			// RaiseError adds error and stack info to the internal error string.

	void AddExplanation(ExplainError &);
			// ExplainError adds explanations to internal error, and to ExplainError stream

	void SetUserError(UserError &);

public:
	static BaseErrorManager *MyErrorManager();
			// Return the error manager associated with the current thread.

	static void FatalError(const char *);
			// Signal a fatal error.

	BaseErrorManager();

	virtual ~BaseErrorManager();

	const char *GetInternalErrorString();
	const char *GetUserErrorString();
	const char *GetExplanation();

	const char *GetSqlClass();
	const char *GetSqlSubclass();

	int GetUserErrorCode();
	int GetInternalErrorCode();

	void HandleError(); 
			// clears all errors.

	virtual void Reset();		
			// clears all errors.  Call this at the beginning of an operation.

	virtual void SetServerResponse(KR_Data_t *) = 0;
	virtual KR_Data_t *CreateResponse() = 0;

	// 
	// Functions to do down-casts safely.
	//

	virtual ClientErrorManager *AsClientErrorManager() {
		return 0;
	};
	virtual ServerErrorManager *AsServerErrorManager() {
		return 0;
	}
	
};

class ClientErrorManager : public BaseErrorManager {

	int serverErrorCode;
public:
	ClientErrorManager() {serverErrorCode=0;}
	void SetServerResponse(KR_Data_t *);

	KR_Data_t *CreateResponse() {
		cerr << "ERROR, called CreateResponse on client!!" << endl;
		exit(-1);
		return 0;
	};
	void Reset();

	int GetServerErrorCode() {
		return serverErrorCode;
	};

	ClientErrorManager* AsClientErrorManager() {
		return this;
	};

};
//
// Class for the server side, with capability for creating a response 
//
class ServerErrorManager : public BaseErrorManager {
public:
	KR_Data_t *CreateResponse();
			// Create a response message to be sent to the client.
			// Normally, operations can just do a return CreateResponse();

	void SetServerResponse(KR_Data_t *) {
		cerr << "ERROR, called SetServerResponse on server!!" << endl;
	};

	ServerErrorManager* AsServerErrorManager() {
		return this;
	};
};

/*<std-footer incl-file-exclusion=ERRORMANAGER_H>  -- do not edit anything below this line -- */

#endif          /*</std-footer>*/
