/*<std-header incl-file-exclusion=PDCOORD_H>

           Copyright 1998, NCR Corporation.  All Rights Reserved.

                               NCR Confidential.


 $Id: PDcoord.h,v 1.1 1998/06/08 16:33:50 kristint Exp $


*/

#ifndef PDCOORD_H
#define PDCOORD_H


/*  -- do not edit anything above this line --   </std-header>*/

//
// PDcoord.h:
//
// Coordinates conversion between long/lat and laea, and also
//	to screen coordinates.
//

class ParaRectangle;

//
// this number was multiplied to convert original lon/lat to integers
// Before converting them to screen coord, we have to take out this
// factor.
//
const float COORD_SCALE = 10000.0;

class coord_cl 
{
    // bounding box in long/lat
    float bx1; 	// low x 
    float bx2;  // high x
    float by1;  // low y
    float by2;	// high y

    // bounding box in laea
    int ibx1;   // low x
    int ibx2;   // high x
    int iby1;   // low y
    int iby2;	// high y
  
    // height and wide of screen view
    int view_w,
	view_h;		
    
    // scale factor to speed up conversion
    float x_scale_fac, 
          y_scale_fac; 

  public:

    //
    // constructors
    //
    coord_cl()
    { 
        bx1 = bx2 = by1 = by2 = x_scale_fac = y_scale_fac = 0.0;
        ibx1 = ibx2 = iby1 = iby2 = 0;
        view_w = view_h = 0;
    };

    coord_cl(float x1, float y1, float x2, float y2, int w, int h)
    { 
        bx1 = x1, bx2 = x2, by1 = y1, by2 = y2, view_w = w, view_h =h;
        x_scale_fac = w / (x2 - x1), y_scale_fac = h / (y2 - y1);
    };
  
    coord_cl(int x1, int y1, int x2, int y2, int w, int h)
    { 
	ibx1 = x1, ibx2 = x2, iby1 = y1, iby2 = y2, view_w = w, view_h =h;
        x_scale_fac = w / (float)(x2 - x1), y_scale_fac = h / (float)(y2 - y1);
    };
  
    coord_cl(const coord_cl &c)
    { 
	bx1 = c.bx1, bx2 = c.bx2, by1 = c.by1, by2 = c.by2;
        ibx1 = c.ibx1, ibx2 = c.ibx2, iby1 = c.iby1, iby2 = c.iby2;
        view_w = c.view_w, view_h = c.view_h;
        x_scale_fac = c.x_scale_fac; y_scale_fac = c.y_scale_fac;
    };
  
    // destructor
    ~coord_cl() {};

    //
    // initializers
    //
    void init(float x1, float y1, float x2, float y2, int w, int h)
    { 
	bx1 = x1, bx2 = x2, by1 = y1, by2 = y2, view_w = w, view_h =h;
        x_scale_fac = w / (x2 - x1), y_scale_fac = h / (y2 - y1);
    };   
  
    void init(int x1, int y1, int x2, int y2, int w, int h)
    { 
	ibx1 = x1, ibx2 = x2, iby1 = y1, iby2 = y2, view_w = w, view_h =h;
        x_scale_fac = w / (float)(x2 - x1), y_scale_fac = h / (float)(y2 - y1);
    };   

    //
    // Conversion routines
    //
    int longLatToScrX(float l) const
    { 
	return (int)((l - bx1) * x_scale_fac); 
    };
  
    int longLatToScrY(float lat) const
    { 
	return (int)((lat - by1) * y_scale_fac);
    };

    float toLong(int x) const
    { 
	return (x / x_scale_fac + bx1); 
    };

    float toLat(int y) const
    { 
	return (y / y_scale_fac + by1); 
    };

    int laeaxToScrX(int l) const
    { 
	return (int)((l - ibx1) * x_scale_fac); 
    };

    int laeayToScrY(int lat) const
    { 
	return (int)((lat - iby1) * y_scale_fac);
    };
  
    int toLaeax(int x) const
    { 
	return (int)(x / x_scale_fac + ibx1); 
    };

    int toLaeay(int y) const
    { 
	return (int)(y / y_scale_fac + iby1); 
    };

    void getbbox(ParaRectangle &rect) const 
    {
        rect.set(ibx1, iby2, ibx2, iby1);
    }
};

/*<std-footer incl-file-exclusion=PDCOORD_H>  -- do not edit anything below this line -- */

#endif          /*</std-footer>*/
