/*<std-header incl-file-exclusion=STRINGARRAY_H>

           Copyright 1998, NCR Corporation.  All Rights Reserved.

                               NCR Confidential.


 $Id: StringArray.h,v 1.17 1998/05/19 03:34:30 kupsch Exp $


*/

#ifndef STRINGARRAY_H
#define STRINGARRAY_H


/*  -- do not edit anything above this line --   </std-header>*/

//
// This file defines a String array object which is designed to 
// hold and manage an array of character strings, and provide the ability to
// serialize and de-serialize the entire array.
//
// -curt.
//

#include <w_strstream.h>
#include <stdio.h>
#include <assert.h>
#include <paraAssert.h>

class StringArray {
private:

	//
	// Define a private copy-in, share-out string
	//
	class CisoString {
		char *s;
	public:
		CisoString() : s(0) {
		};
		~CisoString() {
			if (s) delete [] s;
		};
		void Give(char *str) {
			if (s) delete [] s;
			s = str;
		};

		CisoString &operator=(const char *str) {
			if (s) delete [] s;
			if (str == 0) {
				s = 0;
			} else {
				s = new char [strlen(str) + 1];
				strcpy(s, str);
			}
			return *this;
		};
		CisoString &operator=(int i) {
			char buf[64];
			sprintf(buf, "%d", i);
			return operator=(buf);
		};
		CisoString &operator=(const CisoString& other) {
			return operator=(other.s);
        };

		bool operator==(const char* str) const {
			if (s==str) 
				return true;

			if (!s || !str) 
				return false;

			if (strcmp(s, str) == 0)
				return true;
			else
				return false;
		};
			
		operator const char *() {
			return s;
		};
		const char *String() {
			if (s == 0)
				return "";
			else
				return s;
		};
	};

	inline void UnpackString(const char *);
	inline void UnpackString(istream& );
			
	int argc;
	int allocSize;
	int allocIncrement;
	CisoString *argv;
	char *packedString;


	void Clear() {
	    delete [] argv;
		delete [] packedString;
		packedString = 0;
		argv = 0;
		allocSize = 0;
		argc = 0;
	};
	//
	// Only call Init from a constructor.
	//
	void Init() {
		packedString = 0;
		argv = 0;
		allocSize = 0;
		allocIncrement = 1;
		argc = 0;
	};

public:


	StringArray(const char *packedArray) {
		Init();
		UnpackString(packedArray);
	};

	StringArray(int c=0, int _allocIncrement = 5) {
		Init();
		allocIncrement = _allocIncrement;
		if (allocIncrement <= 0)
			allocIncrement = 1;
		SetSize(c);
	};

	~StringArray() {
		Clear();
	};

	StringArray(const StringArray &X) {
		Init();
		UnpackString(X.Pack());
	};

	StringArray &operator=(const StringArray &X) {
		if (this != &X) {
			Clear();
			UnpackString(X.Pack());
		}
		return *this;
	};

	inline char *Pack() const;
	void Unpack( const char * s) {
		Clear();
		UnpackString(s);
	};

	void Unpack( istream& i) {
		Clear();
		UnpackString(i);
	};

	//
	// Use operator[] for reading.  
	// Only use this for writing if you know that the size is large enough.
	//
	CisoString &operator[](int idx) const {
		assert(idx < argc);
		return argv[idx];
	};
	int operator==(const StringArray&X) const {
		  return (strcmp(this->Pack(), X.Pack()) == 0);
	};
	int operator!=(const StringArray &X) const {
		return !operator==(X);
	};
	//
	// Use operator<<() for adding elements to the end.
	// Note that the list count is increased by 1.
	//
	StringArray &operator<<(const char *s) {
		SetSize(argc+1);
		operator[](argc-1) = s;
		return *this;
	};
	StringArray &operator<<(int i) {
		SetSize(argc+1);
		operator[](argc-1) = i;
		return *this;
	};

	void SetSize(int count) {
		int i;
		if (count > allocSize) {
			allocSize = count + allocIncrement;
			CisoString * a = new CisoString[allocSize];
			for (i=0; i < argc && i < count; i ++) {
				a[i] = argv[i];
			}
			delete [] argv; 
			argv = a;
		}
		argc = count;
	};
	int GetCount() const {
		return argc;
	};

};

inline char *StringArray::Pack() const {
	ostrstream os;
	char **p = (char **)&packedString;

	os << "[" << argc << "]";
	for (int i = 0; i < argc; i ++) {
		if (argv[i] == 0) {
			os << "[0]";
		} else {
			os << "[" << strlen(argv[i]) << "]" << argv[i];
		}
	}
	os << ends;

	if (packedString)
	    delete [] packedString;
	*p =  os.str();
       
	return packedString;
};

inline void StringArray::UnpackString(const char *s) 
{
	istrstream is((char*)s, strlen(s));
	UnpackString(is);
}

inline void StringArray::UnpackString(istream& is) 
{
	int len;
	char *a;

	char p;
	int c=0;

	is >> p;
	if (p != '[')
		return;
	is >> argc;
	is >> p;
	if (p != ']')
		return;

	if (argc == 0) 
		return; 

	// Currently this function is only called after Clear()
	// or Init(), so we don't have to worry about a memory leak
	argv = new CisoString[argc];
	
	while(c < argc && !is.eof() ) {

		is >> p;
		if (p != '[')
			return;
		is >> len;
		is >> p;
		if (p != ']')
			return;
		if (len > 0) {
			a = new char[len + 1];
			is.get(a, len+1, '\0');
		} else {
			a = 0;
		}
		argv[c++].Give(a);
	}
};

inline ostream &operator<<(ostream &os, const StringArray &X) {

	int i;
	for (i=0; i < X.GetCount(); i++) {
		os << "argv[" << i <<"] = \"" << X[i] << "\"" << endl;
	}
	return os;
};

/*<std-footer incl-file-exclusion=STRINGARRAY_H>  -- do not edit anything below this line -- */

#endif          /*</std-footer>*/
