/*<std-header incl-file-exclusion=BASEADTINFO_H>

           Copyright 1998, NCR Corporation.  All Rights Reserved.

                               NCR Confidential.


 $Id: baseAdtInfo.h,v 1.4 1998/10/30 16:39:10 kupsch Exp $


*/

#ifndef BASEADTINFO_H
#define BASEADTINFO_H


/*  -- do not edit anything above this line --   </std-header>*/

/////////////////////////////////////////////////////////////////////////
//
// BaseAdtInfo.h : contains information about adts common to server and client
//
//////////////////////////////////////////////////////////////////////////

//#include <w_stream.h>
#include<iostream.h>

// Typedefs for easier raster and array template declarations
typedef unsigned char       u8bit;
typedef unsigned short int  u16bit;
typedef short int           s16bit;
typedef unsigned int        u32bit;
typedef int                 s32bit;
typedef float               f32bit;
typedef double              f64bit;

class StringArray;

//
// Basic Dictionary Information Class for ADTs
//
class BaseAdtInfo
{ 
public:
      // Possible attribute types. User defined ADT's.
      // Static for built-in types, external user defined
      // types will use a dynamic table.
      enum _AttrType{_aggrFunc = -2, // used for global functions
				    // which actually stand for aggregates.
		    _dummyAdt = -1, // for global functions.
				// The numbers after each ADT indicate
				// the integral value that each enum gets.
				// Unfortunately, this value is used by
				// the paradise front-end. If anything here
				// changes, you have to go and change the
				// ptk/misc.tcl file...
		    _Int4Adt,	// 0	
		    _RealAdt,	// 1
		    _BoolAdt,	// 2	
		    _StringAdt,	// 3	
		    _PointAdt,	// 4	
		    _RectangleAdt,   // 5	
	            _ClosedPolygonAdt, // 6	
		    _PolyLineAdt, // 7
		    _CircleAdt,	// 8	
		    _Raster8bAdt,  // 9 // do not insert any attributes
		    _Raster16bAdt, // 10 // in between Raster8b and UrlAdt
		    _Raster24bAdt, // 11 // unless they are array_based
		    _UInt1ArrayAdt,// 12 // Thanks, Kristin
                    _UInt2ArrayAdt, // 13
		    _Int4ArrayAdt, // 14
                    _Int4OneDArray, // 15
		    _CharOneDArray, // 16
		    _GifAdt,	// 17
		    _TextAdt,	// 18
		    _VideoAdt,	// 19
		    _AudioAdt,	// 20
		    _BlobAdt,	// 21
		    _Dicom3Adt, // 22 // Added by TXI 
		    _Video1Adt, // 23 // Added by TXI 
		    _Video2Adt, // 24 // Added by TXI 
		    _ClobAdt,   // 25
		    _UrlAdt,	// 26
		    _OidAdt,	// 27
		    _DateAdt,   // 28
		    _FidAdt,    // 29
		    _LargeStoresAdt,	// 30
		    _TimeAdt, // 31
                    _HistAdt, // 32
		    _CountApproxAdt, // 33
		     _SCPolygonAdt, // 34
		     _Int2Adt,      // 35
		     _Int1Adt,      // 36
		     _Int8Adt,      // 37
		     _DecAdt,       // 38
		     _TimestampAdt, // 39
		     NUM_ADTS,      // 40
				// total number of ADTs in the system
				// the next two are dummy ADTs
		     _AnyAdt,       // 41
				// used to indicate to the typechecker
				// that the function is capable of
				// accepting ANY adt as a given
				// argument. For example the count
				// aggregate 
		     _SameAdt,      // 42
				// used to indicate to the typechecker
				// that a function is capable of
				// accepting any adt as give arguments
				// as long as they are the same. For
				// example lessThan
      };

 	// the basic element types for arrays.
       enum DataType { _u8bit, 
                       _s8bit, 
                       _u16bit, 
                       _s16bit, 
                       _u32bit, 
                       _s32bit,
                       _f32bit, 
                       _f64bit};
 

  //
  // Basic Attribute Type Class
  //
  class BaseAttrType {
  protected:
    _AttrType _attrType;	// type id
    StringArray* _params;	// parameter array for parameterized types:
				// e.g. size, or element type for arrays.
    StringArray* _allocParams;	// buffer space for parameter array.

  public:

    // expose member variables
    _AttrType baseType (void) const {return _attrType;}
    StringArray* params() const {return _params;}

    // constructors
    BaseAttrType (_AttrType a = _dummyAdt) : _allocParams(NULL)
      { init(a, NULL); }

    BaseAttrType (const BaseAttrType& a) : _allocParams(NULL)
      { 
	init(a._attrType, a._params);
      }

    // destructors
    virtual ~BaseAttrType(); 

    // initilization
    int init(_AttrType a, const StringArray* const p);

    // assignment oiperators
    BaseAttrType& operator= (const BaseAttrType& a)
      {
	if(this != &a)
          {
	    init(a._attrType, a._params);
          }
	return *this;
      }

    // equality operator: type match only
    int operator == (const BaseAttrType& a) const {return _attrType == a._attrType;}
    int operator != (const BaseAttrType& a) const {return _attrType != a._attrType;}

    // equality with native attribute class
    int operator == (const _AttrType& a) const {return _attrType == a;}
    int operator != (const _AttrType& a) const {return _attrType != a;}

    // equality on both type and parameters
    int deep_eq (const BaseAttrType& a) const;

    // output function
    void write (ostream &os) const;

    // validity check
    int isValid () const ;

    // whether type is parameterized
    bool isParameterized() const;

  };

};

/*<std-footer incl-file-exclusion=BASEADTINFO_H>  -- do not edit anything below this line -- */

#endif          /*</std-footer>*/
