/*<std-header incl-file-exclusion=CLERROR_H>

           Copyright 1998, NCR Corporation.  All Rights Reserved.

                               NCR Confidential.


 $Id: clError.h,v 1.12 1998/12/07 15:22:07 jiebing Exp $


*/

#ifndef CLERROR_H
#define CLERROR_H


/*  -- do not edit anything above this line --   </std-header>*/

#include "ClientErrorMap_gen.h"
#include "paraError.h"

extern int  clSaveError(eClientErrors code, const char* file, int line);
extern void clCheckError(int code, const char* file, int line, const char* errorStr=NULL);


// This is for instantiating errors on the client side.
//
class ClientError : public Error {
	static ClientErrorMap ce;
public:
    ClientError(eClientErrors err) : Error(err) {
        *this << ce[err].code << ": TOR Client Error \"" << ce[err].message << "\" occurred ";
    };
};

class ServerError : public Error {

public:
    ServerError() : Error(clSERVER_ERR) {
    };

    ~ServerError() {
    };
};

inline int  clSaveError(int code, const char* file, int line) {
	return clSaveError((eClientErrors) code, file, line);
};
//
// CurrentErrorManager
// 
// Instances of this class arrange themselves in a linked list.  When
// one goes away, whether from the middle of the list or from the end,
// the list is patched back together.
//
// Objects of this class are typically instantiated on the stack at the 
// beginning of a cursor function to set the error manager to that of 
// the cursor.
//
// The current field of this class is a static pointer to the current
// manager.  When a new error manager is created, it becomes the current
// manager.
//
class CurrentErrorManager {
	static CurrentErrorManager *current;

	CurrentErrorManager *prev;
	CurrentErrorManager *next;
	BaseErrorManager *myEm;

public:
	static BaseErrorManager *GetCurrentErrorManager() {
		return current->myEm;
	};

	CurrentErrorManager(BaseErrorManager*em): myEm(em) {
		prev = current;
		current = this;
		if (prev)
			prev->next = this;
		next = 0;
	} ;
	~CurrentErrorManager() {
		if (this == current) {
			current = prev;
		}
		if (prev) {
			prev->next = next;
		}
		if (next) {
			next->prev = prev;
		}
	};

};

/*<std-footer incl-file-exclusion=CLERROR_H>  -- do not edit anything below this line -- */

#endif          /*</std-footer>*/
