/*<std-header incl-file-exclusion=CLIENTBASEADT_H>

           Copyright 1998, NCR Corporation.  All Rights Reserved.

                               NCR Confidential.


 $Id: clientBaseAdt.h,v 1.12 1998/12/11 16:04:08 kupsch Exp $


*/

#ifndef CLIENTBASEADT_H
#define CLIENTBASEADT_H


/*  -- do not edit anything above this line --   </std-header>*/

#include <baseAdtInfo.h>
#include <paraRectangle.h>
#include <iostream.h>
#include <fstream.h>
#ifdef _WINDOWS
    #include <strstrea.h>
#else
    #include <strstream.h>
#endif
#include <PDcoord.h>

////////////////////////////////////////////////////////////////////////
//
// ClientBaseAdt.h : Declaration of the clientBaseAdt class.
//
/////////////////////////////////////////////////////////////////////////


class baseAdt;  // forward declaration of the adt class as seen by the server


// ClientBaseAdt is the base adt that all Paradise clients see. 
// 	It is a wrapper class for the baseAdt class to hide the detail
//	declaration and implementation of ADTs.
//	It only exposes user-accessable methods/functions to the application.
//
class ClientBaseAdt
{
   friend class ParaCursor ;

   public :

      ClientBaseAdt() ;  // Constructor.
      virtual ~ClientBaseAdt();            // Virtual destructor.
   
      ////////////////////////////////////////////////////////////////////
      /////////////// These functions common to all the ADT's. ///////////
      ////////////////////////////////////////////////////////////////////

      bool isNull() const;

      // The type of the ADT.
      virtual BaseAdtInfo::BaseAttrType myType() const;

      virtual baseAdt* adtPtr() const;

      // Converts from the external (ascii) format to the in-memory format.
      virtual int asciiToMem(char data[]);
     

      // See comment in clientBaseAdt.cpp for explaination of why there
      // are three different memToAscii functions

      // Converts from the in-memory format to external (ascii) format.
      virtual int memToAscii(ofstream& outf, char* filename);
      virtual int memToAscii(ostrstream& outf, char* filename);
      virtual int memToAscii(ostream& outf, char* filename);

      // convert from memory to the ascii representation given
      // in queries (this is the inverse of asciiToMem(char data[])
      virtual int memToAscii(ofstream& os) const;
      virtual int memToAscii(ostrstream& os) const;
      virtual int memToAscii(ostream& os) const;

      // is a synonym for memToAscii
      void output(ofstream &os);
      void output(ostrstream &os);
      void output(ostream &os);
   
      // Convert the in-memory representation to database (compact) rep. 
      virtual int memToDb();  

//       // Converts from the external (ascii) format to the in-memory rep.
//       virtual int asciiToDb(const char* fileName,  
// 			    file_cache_t& inputFile, 
// 			    bool& eof);
   
      // Convert the database representation to the in-memory representation.
      virtual int dbToMem(const char* dbData, 
			  int&        dbSize,
			  bool      isPinned = false);
   
      // Estimate the size of the adt data.
      virtual int estSize(const char* dbData, 
			  int&  dbSize); 

      // For spatial types, returns the most bounding rectangle. (if any)
      virtual int mbr(ParaRectangle& boundingBox) const;

      // Convert to Postgres string type
      virtual int memToPGStr(char* pgStr, int maxLen, 
			     bool convert=true);

      // Convert to tk front end string type
      virtual int memToPDStr(char *pgStr, int maxLen,
			     const coord_cl *coord_converter,
			     bool convert=true);
         
      ///////////////////////// The operators. ////////////////////////
      
      // Linear/Spatial Operators.
      virtual bool equal(const ClientBaseAdt* other) const;
      virtual bool notEqual(const ClientBaseAdt* other) const;

      virtual bool lessThan(const ClientBaseAdt* other) const;
      virtual bool lessOrEqual(const ClientBaseAdt* other) const;
      virtual bool greaterThan(const ClientBaseAdt* other) const;
      virtual bool greaterOrEqual(const ClientBaseAdt* other) const;

      // Spatial Operators.
      virtual bool overlap(const ClientBaseAdt* other) const;
      virtual bool notoverlap(const ClientBaseAdt* other) const;

      ///////////////////////// ADT Assignment. ///////////////////////
      virtual int copy (ClientBaseAdt *other);


   protected :
      baseAdt* serverInst; // point to the instance of real ADT implementation.
			   // (as seen by the server).
      int      memStatus;  // memory status

     // bind to a particular adt instance
      virtual void setAdtPtr(baseAdt* newInst);

   private :
};

ostream &operator<<(ostream &os, const ClientBaseAdt &cadt);

/*<std-footer incl-file-exclusion=CLIENTBASEADT_H>  -- do not edit anything below this line -- */

#endif          /*</std-footer>*/
