/*<std-header incl-file-exclusion=CLIENTNARRAYADT_H>

           Copyright 1998, NCR Corporation.  All Rights Reserved.

                               NCR Confidential.


 $Id: clientNArrayAdt.h,v 1.8 1998/05/26 17:31:56 jignesh Exp $


*/

#ifndef CLIENTNARRAYADT_H
#define CLIENTNARRAYADT_H


/*  -- do not edit anything above this line --   </std-header>*/

///////////////////////////////////////////////////////////////////////////
//
// ClientNArrayAdt.h : Wrapper class for a multi-dimensional array data type
//                  
// General Description:  The multi-dimensional array data type is
// built based on three classes NChunk, ChunkMgr, and NArrayAdt
// NArrayAdt is the main class NChunk and ChunkMgr are supporting
// classes.  Unless the data is very small, a n-dimensional array is 
// stored on disk in n-dimensional chunks.  The dimensions of the 
// chunks are proportional to the dimensions of the array.  A shore 
// file is created for each array - this file stores a 'master data'
// record which keeps track of the oids and states of each chunk and
// then stores one record for each chunk.  The chunk manager class
// handles caching and related issues for the array - right now this
// isn't so useful as most functions need only cache one chunk at
// a time...  The main interface to the array is read and write hyperslab.
// That is you can read or write an arbitrary hyperslab into or from
// an array.  An arbitrary hyperslab is specified by corner, edges, 
// and strides.  corner is the array dimensions of the corner of the
// hyperslabs, edges is the number of elements to read in each dimension,
// and strides is the number of elements to skip in each dimension
// when reading and writing (a stride of 1 reads consecutively).
// There are two read/write hyperslab interfaces. One for the Paradise
// SQL interface and one for the HDF library interface.  The interface
// for the Paradise SQL takes as input the corner, edges, and strides as 1-D
// arrayAdts and produces a result n-D array that is disk resident.  This
// interface is working but doesn't currently support strides.
// The interface for the HDF library takes as input int*s for corner,
// strides, and edges and fills in (or reads from) a previously-allocated
// in-memory array of data.  This interface supports strides, and is
// basically written but needs some custom rpc functions to work with
// client and server. (I'm working on these.)
//
///////////////////////////////////////////////////////////////////////////
#include <clientBaseArrayAdt.h>

template <class T>
class ClientNArrayAdt : public ClientBaseArrayAdt
{
public:   

  ClientNArrayAdt();

  // ///////////////////// The Functions. ////////////////////////////
  int getVal(unsigned int indices[], T& data) const;
  int setVal(unsigned int indices[], T data);

  // //////////////// The Aggregate Functions. ///////////////////////
  int average(double& value) const;  // returns average of values in array
  
  int readHyperslab(unsigned int corner[], 
		    unsigned int edges[], 
		    void* data, 
		    unsigned int strides[]);     

  int writeHyperslab(unsigned int corner[], 
		     unsigned int edges[], 
		     void* data, 
		     unsigned int strides[]);

  virtual unsigned int     rank();
  virtual unsigned int     dimSz(unsigned int dim);
  virtual int     dumpToFile(char* filename);
};

typedef ClientNArrayAdt<u8bit>        ClientUInt1ArrayAdt;
typedef ClientNArrayAdt<u16bit>       ClientUInt2ArrayAdt;
// typedef ClientNArrayAdt<u32bit>       ClientUInt4ArrayAdt;
typedef ClientNArrayAdt<s32bit>       ClientInt4ArrayAdt;

/*<std-footer incl-file-exclusion=CLIENTNARRAYADT_H>  -- do not edit anything below this line -- */

#endif          /*</std-footer>*/
