/*<std-header incl-file-exclusion=CURSOR_H>

           Copyright 1998, NCR Corporation.  All Rights Reserved.

                               NCR Confidential.


 $Id: cursor.h,v 1.41 1998/10/30 16:39:45 kupsch Exp $


*/

#ifndef CURSOR_H
#define CURSOR_H


/*  -- do not edit anything above this line --   </std-header>*/

///////////////////////////////////////////////////////////////////////////
//
// cursor.h : The ParaCursor class.  Creates a cursor over the result
//            of a query.  
//
///////////////////////////////////////////////////////////////////////////

#include <adtClient.h>
#include <paraErrorStatus.h>

class InternalParaConnect ; 
class ClientCursor ;

//
// ParaConnect class:
//	Connection object needed in client application to
//	establish connection with databse server.
//
class ParaConnect {
	
    friend class ParaCursor ; 
public:
	
    // constructors
    ParaConnect() ;
    ParaConnect(int port) ;
    ParaConnect( const char* name ) ;	// with host name
    ParaConnect( const char* name, int port) ; // with both host name and port id
	
    // authentication with database server to setup right privileges
    int authenticate( const char *username, 
		const char *password );      
    int unAuthenticate (void);
	
    ~ParaConnect() ;
	
    int open() ;
    int close() ;
    int is_open() ;
	
private:
	
    InternalParaConnect*           paraConnectImpl ;
} ;

//
// ParaCursor class:
//	This class provides a simple mechanism for a client C++
//	application to execute  SQL statements, fetch and process
//	result tuples.
//
class ParaCursor {
	
public:
	
    // Contrsuctor
    ParaCursor( ParaConnect& aConnection, // connection object
		ParaErrorStatus::ErrorMode // error mode
		= ParaErrorStatus::eDetailed ); 
	
    ~ParaCursor();
	
    // initialization functions
    static void ParaAppInit( bool StartParaFtp = true ) ;
    static void ParaAppClose() ;
	
    // report error status.
    ParaErrorStatus& ErrorStatus() ;
	
    //
    // basic cursor operations 
    //
	
    // declare a query in cursor
    int  declare( const char* 			queryStr, 	  
		bool 				update = false );
	
    // execute the query and start the scan
    int  open();   
	
    // executes query in cursor
    int  exec(); 			  
	
    // reopen the cursor (no re-execution)
    int  reopen();	  
	
    // fetch the next row
    int  fetch();			
	
    // close the cursor
    int  close();			
	
    // authenticate with server
    int authenticate( const char*		username, 
		const char*		password );	

    int unAuthenticate(void);
	
    //
    // manipulation of current tuple in cursor 
    //
	
    // get current tuple in adt format
    int  getTuple( ClientBaseAdt**& 		result);  
	
    // get attribute at offset (adt format)
    int  getObj( ClientBaseAdt*& 		obj, 	
		BaseAdtInfo::BaseAttrType& 	type,
		int 				attrIdx );  
	
    // get attribute at offset (binary format )
    int  getField( char*& 			field,		
		int& 			size, 		
		BaseAdtInfo::BaseAttrType& type, 
		int 				attrIdx, 
		bool 			convert = false);	
	
	
    // get attribute by name (adt format )
    int  getObj( ClientBaseAdt*& 		obj,		
		BaseAdtInfo::BaseAttrType& 	type,
		char* 				attrName );  
	
    // get attribute by name (binary format)
    int  getField( char*&			field, 	
		int&				size, 
		BaseAdtInfo::BaseAttrType& 	type, 
		char* 			attrName);	
	
    // update attributes of current row
    int  update( ClientBaseAdt** 		tuple, 	
		int 				modAttrs = 0, 
		int* 				modIdx = NULL);
	
    // delete current row from table
    int  remove();			
	
    //
    // utility functions for updates 
    //
	
    // base table name (for update)
    int getBaseName(const char *&) const ;
	
    // is the cursor updateable
    int Updateable(bool&) ; 
	
    //
    // misc utility functions (user visible)
    //
	
    // query yield result
    int HasResult(bool &) ;
	
    // total number of tuples affected by the query
    int numTupleAffected(int &) ;
	
    // number of attributes
    int numAttrs(int &) ;			
	
    // number of tuples in current batch (use numTupleAffected() for total
    // tuples
    int numTuples(int &) ;
	
    // attribute length
    int attrLen(int attrIdx, int &) ; 		   
	
    // attribute type
    int attrType(int attrIdx, BaseAdtInfo::BaseAttrType &) ;
	
    // attribute name
    int attrName(int attrIdx, const char *&) ;
	
    // extent name
    int extName(const char *&) ;
	
    // export result extent name and disable auto result dropping
    int getResultName(char *&) ;
	
    // for transaction management
    int beginTransaction();
    int commitTransaction();
    int abortTransaction();
	
  private:
	  
	  ClientCursor* paraCursorImpl;
	  baseAdt** internalResult;//ashutosh
	  //InternalParaCursor*           paraCursorImpl ; // internal implementation
	  // of cursor class
	  ClientBaseAdt**		  adtImplPtr ; 	   // internal adt pointer array
	  void                          reset() ; 	   
	  void                          allocate() ; 
} ;

/*<std-footer incl-file-exclusion=CURSOR_H>  -- do not edit anything below this line -- */

#endif          /*</std-footer>*/
