/*<std-header incl-file-exclusion=TORTYPES_H>

           Copyright 1998, NCR Corporation.  All Rights Reserved.

                               NCR Confidential.


 $Id: tortypes.h,v 1.2 1998/11/13 19:23:56 bolo Exp $


*/

#ifndef TORTYPES_H
#define TORTYPES_H


/*  -- do not edit anything above this line --   </std-header>*/


#if defined(I386) || defined(_WIN32)
#  define TOR_BIG_ENDIAN 0
#else
#  define TOR_BIG_ENDIAN 1
#endif
#if TOR_SOLARIS
//#  define _REENTRANT
//#  define _POSIX_C_SOURCE 199506L
#endif

/*
   Since the ANSI C++ standard doesn't nail down the size of various integer
   types, a selection of size specific types are defined  here, based on the
   specific  compiler  that is  being  used,  and  the  assumption  that all
   architectures of interest  support 8 bit   bytes, and multiples  thereof.

   The "_fast" versions are provided for use where the range of values would
   fit in the base type (8 bits for uint8_fast, for  example), but for which
   the semantics  of overflow and  carry are irrelevant,  and speed  is more
   important.   In  modern machines,  types  larger than  a  single byte are
   typically faster even   though larger, because    of the underlying   CPU
   circuitry.  These "_fast" versions are chosen

   All numeric datatypes   that must be   portable across compilers   and/or
   architectures (stored on disk or passed on a network interface) should be
   defined using  an appropriately sized  typedef to define  the  type.  For
   example,

      typedef uint8 MyData;

   Where useful,  a corresponding "_fast"  version of the  type may  also be
   defined.  To continue the above example,

      typedef uint8_fast MyData_fast;

   Notes:

      Currently the choice   of compiler is   implied by architecture,   but
      additional checks could be  made if disparate  compilers might be used
      on the same architecture.

      Making the size known doesn't solve the  whole portability problem, as
      different   architectures  order  the   bytes  within  numeric  values
      differently  (big  endian vs  little  endian).   But  if  the  size if
      standardized, software on either  platform can be  written to fix that
      problem.  Most network  interfaces define a  particular byte order for
      multibyte data values, requiring hosts that use a different byte order
      to compensate.
*/

#if defined(WATCOM)
enum { false, true };
#define __int64 int;
#endif

typedef signed char int8;
typedef unsigned char uint8;
typedef short int16;
typedef unsigned short uint16;
typedef long int32;
typedef unsigned long uint32;
#ifdef __GNUG__
typedef long long int64;
typedef unsigned long long uint64;
#endif
#ifdef _MSC_VER
typedef signed __int64 int64;
typedef unsigned __int64 uint64;
#endif
typedef float float32;
typedef double float64;

#if 1
typedef int int8_fast;
typedef int int16_fast;
typedef long int32_fast;
typedef int64 int64_fast;
typedef unsigned int uint8_fast;
typedef unsigned int uint16_fast;
typedef unsigned long uint32_fast;
typedef uint64 uint64_fast;
typedef double float32_fast;
typedef double float64_fast;
#endif

// Establish macros for numeric limits
#include <limits.h>
#include <float.h>

#define TOR_INT8_MIN           SCHAR_MIN
#define TOR_INT8_MAX           SCHAR_MAX
#define TOR_UINT8_MAX          UCHAR_MAX

#define TOR_INT16_MIN          SHRT_MIN
#define TOR_INT16_MAX          SHRT_MAX
#define TOR_UINT16_MAX         USHRT_MIN

#define TOR_INT32_MIN          LONG_MIN
#define TOR_INT32_MAX          LONG_MAX
#define TOR_UINT32_MAX         ULONG_MAX

#define TOR_FLOAT64_DIG        DBL_DIG          /* # of decimal digits of precision */
#define TOR_FLOAT64_EPSILON    DBL_EPSILON      /* smallest such that 1.0+DBL_EPSILON != 1.0 */
#define TOR_FLOAT64_MANT_DIG   DBL_MANT_DIG     /* # of bits in mantissa */
#define TOR_FLOAT64_MAX        DBL_MAX          /* max value */
#define TOR_FLOAT64_MAX_10_EXP DBL_MAX_10_EXP   /* max decimal exponent */
#define TOR_FLOAT64_MAX_EXP    DBL_MAX_EXP      /* max binary exponent */
#define TOR_FLOAT64_MIN        DBL_MIN          /* min positive value */
#define TOR_FLOAT64_MIN_10_EXP DBL_MIN_10_EXP   /* min decimal exponent */
#define TOR_FLOAT64_MIN_EXP    DBL_MIN_EXP      /* min binary exponent */

#if defined(_MSC_VER)
#define TOR_INT64_MIN          _I64_MIN
#define TOR_INT64_MAX          _I64_MAX
#define TOR_UINT64_MAX         _UI64_MAX
#elif defined(__GNUG__)
#define TOR_INT64_MIN          LLONG_MIN
#define TOR_INT64_MAX          LLONG_MAX
#define TOR_UINT64_MAX         ULLONG_MAX
#else
#error "Unknown compiler type - can't determine numeric ranges"
#endif

/*
   Since the ANSI  C++ specification doesn't   nail down sizeof(bool),  when
   using booleans in data structures that must  be portable across compilers
   and/or  architectures, they should  be converted to  types of known size.
   Three possible storage containers of known size  are supplied here, along
   with  the  template for  making  others.  Use  plain  "bool"  for "_fast"
   versions of these.

   When assigning to  bool or wrapped   bool types, it   is most obvious  to
   assign the  result of a  comparison operator  (==,  <, etc.)  rather than
   convert  pointers  or integers  implicitly,  although  the  semantics are
   identical.  For example:

      MyData foo = 43;
      bool onStack = (foo != 0);
      Bool8 onDisk = (foo <= 77);

   Not:

      bool = foo;

   Note  that if storing  a large number  of  bool values  in  the same data
   structure, it might be appealing to use bit  fields.  However, bit fields
   are  not  portable across  compilers,  again, because  the  order  is not
   specified.  Bit masking and shifting is the most  portable way to achieve
   this, if the space saved is really critical.
*/

template <class T> class BoolWrapper
{
public:
	BoolWrapper() : value(0) {}
	BoolWrapper(bool b) : value(b) {}
	operator bool() const {return value != 0;}
private:
        T value;
};

typedef BoolWrapper<int8>       Bool8;
typedef BoolWrapper<int16>      Bool16;
typedef BoolWrapper<int32>      Bool32;

/*
   Since  the  ANSI  C++  specification  doesn't  nail   down  the  size  of
   enumerations, when using  enumerations in data   structures that must  be
   portable across compilers and/or architectures,  they should be converted
   to types of  known size.  A  template is provided  below for making known
   size storage containers for each enumeration for which this is needed.

   The SizedInt  class should be  chosen from int8,  int16, int32, or int64,
   and should be  sized large enough  to hold the  two extreme values  (most
   negative and most  positive) used  within  the enumeration.   Engineering
   judgement should  be used  to  determine how  much  extra space  might be
   needed in the future if the extreme values are close  to a size boundary.
   Note that picking too small a type here can cause  problems like the Year
   2000 problem, which is by no means the first of its genre.

   You should  name the wrapper  class after the  enumeration that it wraps.
   For example:

      enum MyEnum
      { apple = 63,
        pear = 125,
      };

      typedef EnumWrapper<int8, MyEnum> MyEnumWrap;

*/

template <class SizedInt, class Enumeration> class EnumWrapper
{
public:
  EnumWrapper() : value(0) {}
  EnumWrapper(Enumeration e) : value(e) {}
  operator Enumeration() const {return Enumeration(value);}
  EnumWrapper & operator = (Enumeration e) {value = e; return *this;}
private:
  SizedInt value;
};

//  98aug14, cam, added this

    # if        WATCOM || __GNUC__
    # define    NcrTorPi            /**/
    # define    namespace           extern "C++"
    # define    using               /**/
    # define    using_namespace     /**/
    # else
    # define    using_namespace     using namespace
    # endif


/*<std-footer incl-file-exclusion=TORTYPES_H>  -- do not edit anything below this line -- */

#endif          /*</std-footer>*/
