/*<std-header incl-file-exclusion=PARARECTANGLE_H>

           Copyright 1998, NCR Corporation.  All Rights Reserved.

                               NCR Confidential.


 $Id: paraRectangle.h,v 1.7 1998/09/25 22:32:39 bolo Exp $


*/

#ifndef PARARECTANGLE_H
#define PARARECTANGLE_H


/*  -- do not edit anything above this line --   </std-header>*/

////////////////////////////////////////////////////////////////////////
//
// ParaRectangle.h : Definitions of the baseAdt class.
//
/////////////////////////////////////////////////////////////////////////

#include <iostream.h>
#include <nbox.h>
#include <paraAssert.h>

class ParaRectangle : public nbox_t
{
    public:
       ParaRectangle() : nbox_t(2) { };
       ParaRectangle(const nbox_t& box) : nbox_t(box) {};
       ParaRectangle(int dim) : nbox_t(dim){};
   
       inline void setNull()
       {
	  para_assert(dim == 2);
	  nullify();
       }

       inline void set(int4_t low_x, int4_t low_y, int4_t high_x, int4_t high_y)
       {
	 para_assert(dim == 2);
	 array[0] = low_x;  array[1] = low_y; 
	 array[2] = high_x; array[3] = high_y; 
       }; 

       // set functions for higher dimensions
       inline void set(int _dim, int* low, int* high)
	 {
	   dim = _dim;
	   for(int i=0; i<dim; i++)
	     {
	       array[i] = low[i];
	       array[i+dim] = high[i];
	     }
	 }

       inline void set(int* low, int* high)
	 {
	   for(int i=0; i<dim; i++)
	     {
	       array[i] = low[i];
	       array[i+dim] = high[i];
	     }
	 }

       inline void setVal(int n, int val)
	 {
	   array[n] = val;
	 }

       inline void set(const ParaRectangle& universe)
       {
	  nbox_t::operator=(universe);
       };
   

       // functions to get high and low bounds for a particular
       // dimension
       inline int lbound(int n) {return bound(n);}
       inline int hbound(int n) {return bound(n+dim);}
       inline int4_t* low() {return array;}
       inline int4_t* high() {return array+dim;}

       void write (ostream &os) const
       {
	  os << "Points (x,y): Lower (";
	  int i;
	  for(i=0; i<dim; i++)
	    {
	      os.width(8);
	      os << array[i] << " ";
	    }
	  os << ")  Upper (";
	  for(i=0; i<dim; i++)
	    {
	      os.width(8);
	      os << array[i+dim] << " ";
	    }
	  os << ")" <<endl;
       }

       inline void print() const {write(cout);}

       // Align the box.
       void alignBox(const ParaRectangle& box, float xCellLen, float yCellLen);

       // Find the center. 
       void center(int& x, int& y) 
       {
	 para_assert(dim == 2);
	 x = (array[0] + array[2]) / 2;
	 y = (array[1] + array[3]) / 2;
       };
};

inline ostream& operator<<(ostream& os, const ParaRectangle& rect)
{
   rect.write (os); 
   return os;
}

/*<std-footer incl-file-exclusion=PARARECTANGLE_H>  -- do not edit anything below this line -- */

#endif          /*</std-footer>*/
