/*<std-header incl-file-exclusion=PARATIMER_H>

           Copyright 1998, NCR Corporation.  All Rights Reserved.

                               NCR Confidential.


 $Id: paraTimer.h,v 1.8 1998/05/19 03:34:34 kupsch Exp $


*/

#ifndef PARATIMER_H
#define PARATIMER_H


/*  -- do not edit anything above this line --   </std-header>*/

///////////////////////////////////////////////////////////////////////////
//
// paraTimer.h : Paradise timer class for measuring performance
//
///////////////////////////////////////////////////////////////////////////

#include <errno.h>
#include <iostream.h>
#include "w_gettimeofday.h"

#ifdef SUNOS41
extern "C" {
	int gettimeofday (struct timeval *, struct timezone *);
}
#endif /* SUNOS41 */

static const double _Para_Timer_Million = 1000000.0;

class Timer
{
 private:
   double microsecs;   // The time in microseconds.
   bool inUse;
   struct timeval first, second;
   
   void calc_time()
   {
      microsecs = _Para_Timer_Million * (second.tv_sec - first.tv_sec) + 
	                                (second.tv_usec - first.tv_usec);
   }
   
 public:
   Timer (void) {inUse = false; init ();}

   void init (void)
   {
      if (inUse)
      {
	 cerr << "Warning! Timer already in Use" << endl;
	 return;
      }
      inUse = true;
      gettimeofday(&first, 0);
   }

   void reset (void)
   {
      gettimeofday(&first, 0);
   }

   void freeze (void)
   {
      gettimeofday (&second, 0);
      calc_time();
      inUse = false;
   }

   void write (ostream &os) const
   {
      long oldbase = os.setf(ios::fixed, ios::floatfield);
      os << microsecs/_Para_Timer_Million << " secs";
      os.setf(oldbase, ios::floatfield);
   }

   // get time in seconds.
   inline double gettime() { return microsecs/_Para_Timer_Million; }
};

inline ostream& operator<<(ostream& os, const Timer& t)
{
   t.write(os);
   return os;
}

/*<std-footer incl-file-exclusion=PARATIMER_H>  -- do not edit anything below this line -- */

#endif          /*</std-footer>*/
