# <std-header style=TCL orig-src=PTK>
#
#            Copyright 1998, NCR Corporation.  All Rights Reserved.
#
#                                NCR Confidential.
#
#
#  $Id: EditLayer.tcl,v 1.7 1998/07/23 04:21:55 kristint Exp $
#
#  This file was originally part of ptk, a front end based on TCL/TK to the
#  Paradise database system developed by the University of Wisconsin-Madison.
#  The above copyright applies to changes made to this file by NCR Corporation.
#  The original copyright which may appear in the source below applies to the
#  original file as obtained by NCR Corporation and unchanged parts of this file.
#
#
#   -- do not edit anything above this line --   </std-header>

#This file contains functions used to edit the top layer.  
#Move objects, Draw and Insert new objects
proc layer_edit {pw sid canv {wi .e}} {
    post_msg $sid "Editing layers...";
    global session toplayer LAYER_UPDATE
    upvar #0 session s
    global layerCnt hLayerCnt showSelect hideSelect
    upvar #0 localLayers layers
    upvar #0 localHiddenLayers hLayers

    set layers [getLayers $sid]
    set hLayers [getHiddenLayers $sid]
    set layerCnt [llength $layers]
    set hLayerCnt [llength $hLayers]

    set btns [list $pw.b.b1.new $pw.b.b2.ok]
    setBtnDisable "$btns"   
    set toplayer [lindex $layers [expr $layerCnt - 1]]
    set updatable [lindex $toplayer $LAYER_UPDATE]
    if { $updatable != "update" } {
	pd_dialog .error {Not Updatable Layer} "This is not an updatable \
		layer" {} error 0 {Ok}
	return	
    }
    if { $toplayer == {} } {
	pd_dialog .error {Empty Layer} "You must working on an non-empty \
		layer" {} error 0 {Ok}
	return
    }
    set w .el
    catch "destroy $w"
    toplevel $w;
    wm title $w "Layer Editor";
    wm group $w $pw;
    set wmposx [expr 400 + ($sid * 10) % 250]
    set wmposy [expr 70 + ($sid * 10) % 250]
    wm geometry $w "+$wmposx+$wmposy"

    ptkPDWait ON;

    bind $canv <Any-Motion> "setnewcrd %x %y $sid"
    $canv addtag selectable all
    #right frame contains the drawing type
    frame $w.lm -bd 1m -relief groove
    button $w.lm.point -text "Point" -command "DrawPoint $canv {$toplayer} $sid 4" \
	-state disabled
    button $w.lm.line -text "Line" -command "DrawPoly $canv {$toplayer} $sid 8" \
	-state disabled
    button $w.lm.poly -text "Polygon" -command "DrawPoly $canv {$toplayer} $sid 6" \
	-state disabled
    pack $w.lm.point $w.lm.line $w.lm.poly -side top \
	-padx 1m -pady 1m -fill x

    #left frame contains the commands
    frame $w.b -bd 1m -relief groove
    frame $w.b.b1
    pack $w.b.b1 -side top -fill x

    set btnlist [list $w.lm.point $w.lm.line $w.lm.poly]

    button $w.b.b1.draw -text "Draw" \
	-command "setBtnActive {$btnlist};"

    button $w.b.b1.move -text "Move" \
	-command "setBtnDisable {$btnlist}; \
		  MoveObj $canv $sid;"
    button $w.b.b1.shift -text "Shift" \
	-command "setBtnDisable {$btnlist}; \
		  ShiftLayer $canv $sid;"

    button $w.b.b1.ok -text "Ok" -command "resetBinding {$btns} $canv $sid; \
					   ptkPDWait OFF; \
					   destroy $w";

    pack $w.b.b1.draw $w.b.b1.move $w.b.b1.shift $w.b.b1.ok \
	-side top -expand 1 -fill x -padx 1m -pady 1m;

    pack $w.b $w.lm -side left -expand 1 -fill x -padx 1m -pady 1m
    ptkPDWait OFF
    tkwait window $w
#    set edl_active(editl) 0
}

# reset bindings when exit Move
proc resetBinding { btns canv sid } {
    post_msg $sid "Managing layers..."
    bind $canv <Any-Motion> "setnewcrd %x %y $sid"
    # random selection using button 1
    bind $canv <ButtonPress-1> "
    $canv addtag cur_select withtag current
    itemSelect $canv $sid \
	    \[$canv itemconfigure current -tags\] %x %y"
    bind $canv <ButtonRelease-1> "
    itemSelect2 $canv $sid  \
	    \[$canv itemconfigure current -tags\] %x %y
    $canv dtag cur_select"
    
    # new center by B3
    bind $canv <Button-3> "newCenter \$sid $canv %x %y"
    bind $canv <B3-Motion> {}
    bind $canv <ButtonRelease-3> {}
    setBtnActive "$btns"
}

proc setBtnActive { activebtns } {
    foreach btn $activebtns {
	$btn config -state normal
    }
}

proc setBtnDisable { disablebtns } {
    foreach btn $disablebtns {
	$btn config -state disabled
    }
}

# Draw Polygon or Polyling Objects
proc DrawPoly { canv layer sid type} {
    global session LAYER_DESC stroke

    post_msg $sid "Please use B3 to draw on the map.  Click once to start, twice to finish.";
    set desc [lindex $layer $LAYER_DESC]
    set attrcnt [llength $desc]
    set attrnames [getAttrNamesFromDesc $desc]
    set attrtypestrs [getAttrTypesFromDesc $desc]
    set valid 0
    set stroke(N) -1

    # find out if this is a polygon or polyling layer, refuse to edit if not
    foreach tstr $attrtypestrs {
	set id [ptkTypename2id $tstr];
	if {$id == $type} {
	    set valid 1
	    set attrIndex [lsearch $attrtypestrs $tstr]
	}
	
    }
	
    if { $valid == 0 } {
	pd_dialog .error {Wrong Attrtype} "Wrong Attrtype layer" \
		{} error 0 {Ok}
	return
    }

    set color [lindex [getLayerColor $layer] $attrIndex]
    set width [lindex [getLayerWidth $layer] $attrIndex]
    #use the third button to draw
    bind $canv <Button-3> "Stroke %W %x %y $color $width"
    bind $canv <Any-Motion> "Motion %W %x %y $color $width; \
			     setnewcrd %x %y $sid"
    bind $canv <Double-3> "StrokeEnd %W %x %y $color $width $sid $type"
}

#Draw point objects
proc DrawPoint { canv layer sid type} {
    global session LAYER_DESC

    bind $canv <Any-Motion> "setnewcrd %x %y $sid"
    post_msg $sid "Please click Button-3 to position the point."

    set desc [lindex $layer $LAYER_DESC]

    set attrcnt [llength $desc]
    set attrnames [getAttrNamesFromDesc $desc]
    set attrtypestrs [getAttrTypesFromDesc $desc]
    set valid 0
    foreach tstr $attrtypestrs {
	set id [ptkTypename2id $tstr];
	if {$id == $type} {
	    set valid 1
	    set attrIndex [lsearch $attrtypestrs $tstr]
	}
	
    }
	
    if { $valid == 0 } {
	pd_dialog .error {Wrong Attrtype} "Wrong Attrtype layer" \
		{} error 0 {Ok}
	return
    }

    set color [lindex [getLayerColor $layer] $attrIndex]
    set bmap [lindex [getLayerBitmap $layer] $attrIndex]

    bind $canv <Button-3> "ShowPoint %W %x %y $color $bmap $sid"
}

#show a new point on the map
proc ShowPoint { w x y color bmap sid} {
    global points
    	
    set points {}
    append points "$x $y"

    set rc [catch { eval {$w create bitmap} $points {-bitmap $bmap \
	-foreground $color -tag new} } err]    
    if $rc {
	set points {}
    }

    set curobj $err

    CreateTuple $curobj $sid $w   
}

# begin to draw 
proc Stroke { w x y color width} {
    global stroke last

    if { $stroke(N) == -1 } {
	set stroke(0) [list $x $y]
	incr stroke(N)
    } else {
    	set last $stroke($stroke(N))
    	incr stroke(N)
	set stroke($stroke(N)) [list $x $y]
	$w delete previous
    	eval {$w create line} $last {$x $y -tag segments -fill $color \
 	    -width $width}
    }
}

# during the draw process
proc Motion { w x y color width } {
    global stroke 

    if { $stroke(N) == -1 } { return }
    if { $stroke(N) == 0 } {
    $w delete previous
    eval {$w create line} $stroke(0) {$x $y -tag previous -fill $color \
	-width $width}
    } else {
    $w delete previous
    eval {$w create line} $stroke($stroke(N)) {$x $y -tag previous -fill $color \
	-width $width}
    }
}
    
# end of drawing one segment
proc StrokeEnd { w x y color width sid typeid} {
    global stroke points
    set points {}
    for {set i 0} {$i <= $stroke(N)} { incr i } {
	append points $stroke($i) " "
    }
    $w delete segments
    if { $typeid == 8 } {
    	set rc [catch { eval {$w create line} $points {-tag new -fill \
		$color -width $width} } err]
    } else {
    	set rc [catch { eval {$w create poly} $points {-tag new -fill \
		$color} } err]
    }	
    if $rc {
	set points {}
    }
    set curobj $err

    CreateTuple $curobj $sid $w
    set stroke(N) -1
}

# Move an object 
proc MoveObj { canv sid } {
    global toplayer session

    bind $canv <Any-Motion> "setnewcrd %x %y $sid"
    bind $canv <ButtonPress-1> { MarkObj %x %y %W }
    bind $canv <B1-Motion> { DragObj %x %y %W }
    bind $canv <ButtonRelease-1> "UpdateCoord $canv $sid %x %y %W;"


}

# Mark the initial position
proc MarkObj { x y w } {
    global state
    upvar #0 localLayers layers

    set state($w,obj) [$w find closest $x $y]
    if { $state($w,obj) == "" } {
	pd_dialog .error {Invalid Object} "No Object Selected!" \
	  {} error 0 {Ok}
	return	
    }
    set tags [$w gettags $state($w,obj)]
    set select 0
    foreach tag $tags {
	if {$tag == {}} {continue}
	if [string match {toplayer} $tag] {
	    set select 1
	}
    }
    if { $select == 0 } {
	pd_dialog .error {Invalid Object} "This is not an valid object in \
	  current layer!"  {} error 0 {Ok}
	return
    }	
    set state($w,xi) $x
    set state($w,yi) $y
    set state($w,x) $x
    set state($w,y) $y
} 

# after Mark, drag the selected object to new position
proc DragObj { x y w } {
    global state
    set dx [expr $x - $state($w,x)]
    set dy [expr $y - $state($w,y)]
    $w move $state($w,obj) $dx $dy
    set state($w,x) $x
    set state($w,y) $y

}

# calculate the dx dy
proc UpdateCoord { canv sid x y w } {
    global session state view_w view_h 
    global toplayer LAYER_BASETABLE

    set layers [getLayers $sid]
    set layerCnt [llength $layers]

    set x_fac [expr (double($session($sid,bx2)-$session($sid,bx1)))/$view_w]
    set y_fac [expr (double($session($sid,by2)-$session($sid,by1)))/$view_h]

    set tags [$w gettags $state($w,obj)]

    set dx [expr int(($x-$state($w,xi))*$x_fac)]
    set dy [expr int(($y-$state($w,yi))*$y_fac)]
    
    $w addtag cur_select withtag current
    itemSelect $w $sid [$w itemconfigure current -tags] %x %y
    changeCoord $w $sid [$w itemconfigure current -tags] $dx $dy
    $w dtag cur_select"
    set extName [lindex $toplayer $LAYER_BASETABLE]
    foreach layer $layers {    
        set lid [getLayerId $layer]
        set lidtag "l$lid"
	if { [lindex $layer $LAYER_BASETABLE] == $extName } {
	    $canv delete $lidtag
	    set session($sid,bbox_changed) 1
	    showOneLayer $canv $layer $sid "false"
	    set session($sid,bbox_changed) 0
	}
    }	

#    foreach layer $layers {    
#	if { [lindex $layer $LAYER_BASETABLE] == $extName } {
#	    set session($sid,bbox_changed) 1
#	    showOneLayer $canv $layer $sid
#	}
#    }
}

# modify database to save the changes. 
proc changeCoord {c sid tags dx dy} {
    global mapview_tcl debugMode LAYER_BASEQUERY
    global mapview_item_sel session

    set layer_id "";
    set oid "";

    foreach tag $tags {
	if {$tag == {}} {continue}
	if { [string index [lindex $tag 1] 0] == "o"} {
	    # this is the one that contains the oid
	    set oid [string trimleft [lindex $tag 1] o]
	}
	if [string match {l[0-9]*} [lindex $tag 0]] {
	    set layer_id [string range [lindex $tag 0] 1 end]
	    set tmpfn [getLayerTmpfnById $sid $layer_id]
	}
    }
    if {$layer_id == "" || $oid == ""} {
	post_msg $sid "No object found!"
	return
    }

    set l [getLayers $sid]
    set Layer [getVisibleLayerById $sid $layer_id]
    set baseQuery [lindex $Layer $LAYER_BASEQUERY]

    # now we are sure an object has been selected
    global ParserOidAttributeName 
    set q "select * from $tmpfn where $ParserOidAttributeName = \"$oid\""
    if $debugMode {puts "\tRun query: $q"}
    ptkPDWait ON
    set rc [PDsql_cursoropen $q]
    if { $rc == "NO_TUPLE" } { return }

    set tuple [PDsql_cursorfetch]

    set desc [PDsql_cursorschema]
    set Tmpfn [getTmpfnFromDesc $desc]

    set attrns [getAttrNamesFromDesc $desc]
    set type_strs [getAttrTypesFromDesc $desc]
    set modIdx {}
    set index 0

    for {set i 0} {$i < [llength $attrns]} {incr i} {
	set attr_name [lindex $attrns $i]
	set attr_type [lindex $type_strs $i]
	set attr_val  [lindex $tuple $i]
	if [isAttrDispType [ptkTypename2id $attr_type]] {
	    set newptlist {}
 	    set attr_val [string trim $attr_val ()]
	    set pointlist [split $attr_val ,]
	    for {set j 0} {$j < [expr [llength $pointlist]/2]} {incr j} {
		set point($j,x) [expr [lindex $pointlist [expr $j*2]]+$dx]
		set point($j,y) [expr [lindex $pointlist [expr $j*2+1]]+$dy]

		append newpt "$point($j,x) "
		append newpt "$point($j,y),"
	    }
	    set newpt [string trimright $newpt ,]
	    lappend modIdx $i
	    set index $i
	    set newTuple [lreplace $tuple $index $index $newpt]
    	} 
    }    

    UpdateTuple $q $newTuple $modIdx $type_strs $baseQuery

    if [regexp "__PTmpF_" $Tmpfn] {
	PDsql_cursorrun "drop table $Tmpfn"
    }

    ptkPDWait OFF
}

# create a new tuple to insert after user type in the needed information
proc CreateTuple { curobj sid canv} {
    global session debugMode bitmap_dir ptk_tupleDlg_tcl ptk_typewd
    global LAYER_DESC LAYER_BASETABLE toplayer attr_vals 
    global points view_w view_h
    upvar #0 localLayers layers

    set layerCnt [llength $layers]
    set x_fac [expr (double($session($sid,bx2)-$session($sid,bx1)))/$view_w]
    set y_fac [expr (double($session($sid,by2)-$session($sid,by1)))/$view_h]

    if $debugMode {
	puts "Enter CreateTuple"
    }

    set desc [lindex $toplayer $LAYER_DESC]
    set extName [lindex $toplayer $LAYER_BASETABLE]
    set attrns [getAttrNamesFromDesc $desc]
    set type_strs [getAttrTypesFromDesc $desc]
    set pointlist [split $points " "]
    set newPoint ""
    set newptlist {}
    for {set i 0} {$i<[expr [llength $pointlist]/2]} {incr i} {
	lappend newptlist [expr int([lindex $pointlist [expr $i*2]]*$x_fac\
		+$session($sid,bx1))]
	lappend newptlist [expr int([lindex $pointlist [expr $i*2+1]]*$y_fac\
		+$session($sid,by1))]
    }
    set newPoint [join $newptlist ,]
     
    set attrwd {}
    foreach i $type_strs {
	lappend attrwd $ptk_typewd($i)
    }
	
    set nattrs [llength $attrns]

    # pop up a configuration window
    set w ".new_obj_config"
    catch {destroy $w}
    toplevel $w
    wm title $w "Create New Tuple"
    incr ptk_tupleDlg_tcl(wcnt)

    set tpframe [frame $w.tpframe$sid -bd 0.8m -relief groove];
    set msgFrame [frame $w.msgframe$sid -bd 0.5m -relief raised];
    set msgwin [message $msgFrame.msg -bg gray80 -fg black -width 360];
    pack $msgwin -expand 1 -fill x;
    
    pack $tpframe -side top -padx 3 -pady 3
    set tablist {}
    set newtuple ""
    for {set i 0} {$i < [llength $attrns]} {incr i} {
	set attr_name [lindex $attrns $i]
	set attr_type [lindex $type_strs $i]
	set attr_val  {}
	set attr_vals($i) $attr_val
	set valwd $ptk_typewd($attr_type)

	set attrframe [frame $tpframe.attr$i]
	label $attrframe.an$i -width $ptk_tupleDlg_tcl(attrn_wd) \
		-anchor w -text "$attr_name" -bd 1 -relief raised
	if {$attr_type == "Video"} {
	    button $attrframe.at$i -width $ptk_tupleDlg_tcl(attrt_wd) \
		    -anchor w -text "play me" \
		    -bd 0.5m -relief raised
	    entry $attrframe.av$i -width $valwd \
		    -textvariable attr_vals($i) -bd 3 -relief sunken \
		    -fg blue -bg white 
 	    lappend tablist $attrframe.av$i  

	} else {
	    if [regexp {oid} $attr_name] {
		button $attrframe.at$i -width $ptk_tupleDlg_tcl(attrt_wd) \
			-anchor w -text "($attr_type)" \
			-bd 0.5m -relief raised -state disabled
	    	entry  $attrframe.av$i -width $valwd \
		    	-textvariable attr_vals($i) -bd 3 -relief sunken \
		    	-fg blue -bg white -state disabled
	    } else {	
	    	if [isAttrDispType [ptkTypename2id $attr_type]] {
		button $attrframe.at$i -width $ptk_tupleDlg_tcl(attrt_wd) \
			-anchor w -text "($attr_type)" \
			-bd 0.5m -relief raised
		if {$newPoint != "" } {
		    set attr_vals($i) "($newPoint)"
		}
	    	} else {
		button $attrframe.at$i -width $ptk_tupleDlg_tcl(attrt_wd) \
			-anchor w -text "($attr_type)" \
			-bd 0.5m -relief raised -state disabled
	    	}
	    	if {[string length $attr_val] > $valwd} {
		set attr_val [string range $attr_val 0 \
			[expr $valwd - 6]]
		append attr_val " ... "
	    	}
	    	entry  $attrframe.av$i -width $valwd \
		    -textvariable attr_vals($i) -bd 3 -relief sunken \
		    -fg blue -bg white 
		lappend tablist $attrframe.av$i
	    }
	}

	label $attrframe.sep$i -width 3 -text ":"
	pack $attrframe.an$i $attrframe.at$i $attrframe.sep$i \
		-side left 
	pack $attrframe.av$i -side left -expand 1 -fill x;
	pack $attrframe -side top -expand 1 -fill x;
	    if { [isAttrDispType [ptkTypename2id $attr_type]] || 
  		  ($attr_type == "String") } {		
	    	append newtuple "\"$attr_vals($i)\""
	    } else {	    	
		append newtuple "$attr_vals($i)"
	    }		
    }

    foreach entry $tablist {
	bind $entry <Tab> " tab {$tablist} "
    } 
    
    set btframe [frame $w.btframe$sid]

    set numMod 0
    set newTuple {}
    set modIdx {}

    set layertag "l[expr $layerCnt-1]"
    set instbtn [button $btframe.instbtn -width 9 \
	   -text "Insert" -command " \
				 ptkPDWait ON; \
				 $canv addtag $layertag withtag new; \
				 InsertNew {$extName} {$type_strs} $sid $canv;\
				 $canv delete new; \
				 $canv dtag new; \
				 destroy $w; \
				 ptkPDWait OFF;"]

    set okbtn [button $btframe.okbtn -width 9 -text "Cancel"]

    pack $instbtn $okbtn -side left -padx 3 -pady 0.5m -expand 1 -fill x
    pack $btframe -side top -pady 1m;
    pack $msgFrame -side top -pady .3m -expand 1 -fill x;

    bind $okbtn <Enter> \
	    "+$msgwin configure -text \"Click to dismiss\"";
    bind $okbtn <Leave> \
	    "+$msgwin configure -text \"    \"";
    bind $okbtn <1> \
	    "$canv delete new; \
	     destroy $w;"
}

# use tab to interate through the configuration box
proc tab { list } {
    set i [lsearch -exact $list [focus]]
    incr i
    if {$i >= [llength $list]} {
	set i 0
    }
    focus [lindex $list $i]
}

# insert the newly create tuple in to database
proc InsertNew { extName type_strs sid canv} {
    global attr_vals session LAYER_BASETABLE

    set layers [getLayers $sid]
    set layerCnt [llength $layers]

    set query "insert into $extName values "  
    set attrlist ""
    for { set i 0 } { $i < [llength $type_strs] } { incr i } {
	set attr_type [lindex $type_strs $i]
	if { $attr_type == "String" } {
	    append tuple "\"$attr_vals($i)\","
	} else {
     	if { $attr_vals($i) == "" && $attr_type != "String" \
		&& $attr_type != "Oid" } {
	    pd_dialog .error {Empty Fields} "Not enough value to insert. \
		You must fill in the required fields" {} error 0 {Ok}    
	    $canv delete new
	    return
	}
	if { $attr_type == "Point" } {
	    append tuple "$attr_type$attr_vals($i),"
	} else {
	    if [isAttrDispType [ptkTypename2id $attr_type]] {
		set ptlist [split [string trim $attr_vals($i) ()] ,]
		for { set j 0 } {$j < [expr [llength $ptlist]/2]} { incr j } {
		    append ptstring [lindex $ptlist [expr $j*2]]
		    append ptstring " [lindex $ptlist [expr $j*2+1]],"
   		}
 	    	set ptstring [string trimright $ptstring ,]
		set ptstring "$attr_type\(\"$ptstring\"\)"
		append tuple "$ptstring,"
	    } else {
	   	append tuple "$attr_vals($i),"
	    }
	}
    }
    }
    set tuple [string trimright $tuple ,]
    append query $tuple
    PDsql_cursorrun $query
    foreach layer $layers {    
        set lid [getLayerId $layer]
        set lidtag "l$lid"
	if { [lindex $layer $LAYER_BASETABLE] == $extName } {
	    $canv delete $lidtag
	    set session($sid,bbox_changed) 1
	    showOneLayer $canv $layer $sid "false"
	    set session($sid,bbox_changed) 0
	}
    }	
}

proc ShiftLayer { canv sid } {
    global toplayer session

    post_msg $sid "Shift the entire layer."
    bind $canv <ButtonPress-1> { MarkLayer %x %y %W }
    bind $canv <B1-Motion> { DragLayer %x %y %W }
    bind $canv <ButtonRelease-1> {}
    bind $canv <2><3> {post_msg $sid "Please press Button-1, hold and drag to new position."}
}

proc MarkLayer { x y w } {
    global toplayer objs coord

    set objs [$w find withtag toplayer]
    if { $objs == "" } {
	pd_dialog .error {No Objects} "No Objects displayed for this layer" \
	  {} error 0 {Ok}
	return	
    }

    set coord(xi) $x
    set coord(yi) $y
    set coord(x) $x
    set coord(y) $y
} 

proc DragLayer { x y w } {
    global objs coord
    set dx [expr $x - $coord(x)]
    set dy [expr $y - $coord(y)]
    foreach obj $objs {
    	$w move $obj $dx $dy
	set coord(x) $x
	set coord(y) $y
    }

}

