# <std-header style=TCL orig-src=PTK>
#
#            Copyright 1998, NCR Corporation.  All Rights Reserved.
#
#                                NCR Confidential.
#
#
#  $Id: attrConfig.tcl,v 1.14 1998/05/19 03:22:21 kupsch Exp $
#
#  This file was originally part of ptk, a front end based on TCL/TK to the
#  Paradise database system developed by the University of Wisconsin-Madison.
#  The above copyright applies to changes made to this file by NCR Corporation.
#  The original copyright which may appear in the source below applies to the
#  original file as obtained by NCR Corporation and unchanged parts of this file.
#
#
#   -- do not edit anything above this line --   </std-header>

#
#   This file contains the procs to manage the configurations
#   of each spatical attriubte of each layer.
#
#   Author: Jun Luo
#   Date  : June 15, 1994
#	

set show_attrs(null) {};
set attrConfigVar(title_width) 32;
set attrConfigVar(title_height) 5;
set attrConfigVar(title_font)  "-Adobe-Times-Medium-R-Normal-*-180-*";
set attrConfigVar(title_fg) blue;
set attrConfigVar(title_bg) bisque1;
set attrConfigVar(button_width) 16;

# configure the presentation of an attribute
# this is just a swith routine, each type of
# attribute gets its own configuration proc.
#
# Parameters are:
#	sid:		session id
#	attr_indx:	attribute index
#	tags:		tags of the object on screen  
#	attr_types:	attribute types in string format
#	attr_names:	attribute names of all attributes of this extent 
# 	btn:		option button name
proc attrConfig {sid attr_indx tags attr_types attr_names btn canvwin} {
    global session debugMode;

    if $debugMode {
	puts "attrConfig called";
	puts "\tsid = $sid";
	puts "\tattr_indx = $attr_indx";
	puts "\ttags = $tags";
	puts "\tattr_types = $attr_types";
	puts "\tattr_names = $attr_names";
	puts "\tbutton name = $btn";
	puts "\tcanvas path = $canvwin";
    }

    set attr_type [lindex $attr_types $attr_indx];
    
    if {$attr_type == "Polygon" || $attr_type == "Rectangle"} {
	closedPolygonConfig $sid $attr_indx $attr_types $attr_names $tags $canvwin;
	return;
    }
    if {$attr_type == "Point"} {
	pointConfig $sid $attr_indx $attr_types $attr_names $tags $canvwin;
	return;
    }
    if {$attr_type == "PolyLine"} {
	polylineConfig $sid $attr_indx $attr_types $attr_names $tags $canvwin;
	return;
    }
    if {$attr_type == "Raster" || $attr_type == "Raster8H" || $attr_type == "Raster8" || $attr_type == "Raster16" || $attr_type == "Raster24"} {
	rasterConfig $sid $attr_indx $attr_types $attr_names $tags $canvwin;
	return;
    }
    if {$attr_type == "Video"} {
	videoConfig $sid $attr_indx $attr_types $attr_names $tags $canvwin;
	return;
    }
    if {$attr_type == "Text"} {
	textConfig $sid $attr_indx $attr_types $attr_names $tags $canvwin;
	return;
    }

    if {$attr_type == "Gif"} {
	textConfig $sid $attr_indx $attr_types $attr_names $tags $canvwin;
	return;
    }
    
    

    # Other types are not configurable at this time yet
    # disable the option button
    puts "Warning: $attr_type is not a configurable type";
    $btn configure -state disabled
    if $debugMode {puts "exit attrConfig"}
}

proc videoConfig {sid attr_indx attr_types attr_names tags canvwin} {
    global debugMode
    if $debugMode {
	puts "tmpfn = [getLayerTmpfnById $sid [tags::get_lid $tags]]"
	puts "oid = [tags::get_oid $tags]"
    }
    ptk_showvid [getLayerTmpfnById $sid [tags::get_lid $tags]] \
    	    [tags::get_oid $tags] $attr_indx
}

proc textConfig {sid attr_indx attr_types attr_names tags canvwin} {
    global debugMode
    if $debugMode {
	puts "tmpfn = [getLayerTmpfnById $sid [tags::get_lid $tags]]"
	puts "oid = [tags::get_oid $tags]"
    }
    ptk_showtext [getLayerTmpfnById $sid [tags::get_lid $tags]] \
    	    [tags::get_oid $tags] $attr_indx
}

proc gifConfig {sid attr_indx attr_types attr_names tags canvwin} {
    puts "GIF CONFIG CALLED"
    global debugMode
    if $debugMode {
	puts "tmpfn = [getLayerTmpfnById $sid [tags::get_lid $tags]]"
	puts "oid = [tags::get_oid $tags]"
    }
    ptk_showgif [getLayerTmpfnById $sid [tags::get_lid $tags]] \
    	    [tags::get_oid $tags] $attr_indx
}


proc polylineConfig {sid attr_indx attr_types attr_names tags canvwin} {
    global session debugMode;
    global show_attrs attrConfigVar;
    
    if $debugMode {
	puts "polylineConfig called";
	puts "\tsid = $sid";
	puts "\tattr index = $attr_indx";
	puts "\ttags = $tags";
    }

    set attr_name [lindex $attr_names $attr_indx];
    
    # pop up a mono dialog to configure the color, icon and
    # transparency information of point data

    # first, we need the current presentation information
    # from layer data structure
    set lid [tags::get_lid $tags];
    set layer [getVisibleLayerById $sid $lid];
    set tag_name [format "l%s_%s" $lid $attr_indx];
    set label_name [format "l%slb%s" $lid $attr_indx];
    
    # extract the color, icon and transparency information
    # and put them in an associative array
    getShowAttr $layer show_attrs $attr_indx;
    
    set intAttrsIndxs {};
    set i 0;
    foreach at $attr_types {
	if {$at == "Integer"} {
	    lappend intAttrsIndxs $i;
	}
	incr i;
    }

    if [useColorPallete $show_attrs(color)] {
	set color_sample "                           ";
	set bgcolr $show_attrs(color);
    } else {
	set color_sample \
		"colored by [lindex $attr_names $show_attrs(color)]";
	set bgcolr "white";
    }

    # finally, we are ready to show our dialog
    set w ".plineConfig$sid";
    toplevel $w;
    wm title $w "Configure Appearence";

    set len [string length "Configure $attr_name"]
    set title [label $w.msg -text "Configure $attr_name" \
	    -bd 2 -relief raised \
	    -fg $attrConfigVar(title_fg) \
	    -anchor c \
	    -width $attrConfigVar(title_width) \
	    -height $attrConfigVar(title_height) \
	    -font $attrConfigVar(title_font)];
    set cf [frame $w.cf -bd 1 -relief raised];
    set wf [frame $w.wf -bd 1 -relief raised];
    set lf [frame $w.lf -bd 1 -relief raised];
    set rf [frame $w.rf -bd 1 -relief raised];
    set okf [frame $w.okf -bd 1 -relief raised];
    pack $title $cf $wf $lf $rf $okf -side top -expand 1 -fill x;

    # set up the color frame
    set cbtn [menubutton $cf.mb -text "Choose Line Color" -menu $cf.mb.m\
	    -bd 2 -relief raised -width $attrConfigVar(button_width)];
    set csample [label $cf.show -text $color_sample -bg $bgcolr\
	    -bd 2 -relief sunken -anchor w];
    pack $cbtn $csample -side left -expand 1 -fill x -padx 0.5m \
	    -pady 5

    menu $cbtn.m;
    $cbtn.m add cascade -label "Color Palette" -menu $cbtn.m.c;
    if {$intAttrsIndxs != {}} {
	$cbtn.m add cascade -label "Integer Attributes" \
		-menu $cbtn.m.i;
	menu $cbtn.m.i;
	foreach i $intAttrsIndxs {
	    set attrn [lindex $attr_names $i];
	    set msg "Colored by <$attrn>"; 
	    set len [string length $msg]; 
	    $cbtn.m.i add command -label "$attrn" -command \
		    "set show_attrs(color) $i; \
		    $csample configure -text \"$msg\" -width $len \
		    -background white";
	}
    }

    global col_palete;
    set msg "                           ";
    set len [string length $msg]; 
    menu $cbtn.m.c;
    foreach color_cell $col_palete {
	$cbtn.m.c add command -label "$color_cell" \
		-background $color_cell -command \
		"set show_attrs(color) $color_cell; \
		$csample configure -text \"$msg\" -width $len \
		-background $color_cell";
    }

    # set up label frame
    if {$show_attrs(label) != -1} {
	set lshow_txt [lindex $attr_names $show_attrs(label)];
	set lshow_txt "Labeled by <$lshow_txt>";
    } else {
	set lshow_txt "None"
    }
    set lbtn [menubutton $lf.mb -text "Choose Label" -menu $lf.mb.m \
	    -bd 2 -relief raised -width $attrConfigVar(button_width)];
    set lshow [label $lf.show -text "$lshow_txt" -bg white -anchor w];
    pack $lbtn $lshow -side left -expand 1 -fill x -padx 0.5m \
	    -pady 5
    menu $lbtn.m;
    set msg "No Label";
    set len [string length $msg];
    $lbtn.m add command -label "None" -command \
	    "set show_attrs(label) -1; \
	    $lshow configure -text \"$msg\" -width $len";
    set i 0;
    foreach attrn $attr_names {
	set msg "Labeled by <$attrn>";
	set len [string length $msg];
	$lbtn.m add command -label "$attrn" -command \
		"set show_attrs(label) $i; \
		$lshow configure -text \"$msg\" -width $len";
	incr i;
    }
    
    # set up the width frame
    set wlabel [label $wf.l -text "Line Width"];
    set wctrl [tixControl $wf.ctl -entrywidth 8 -value $show_attrs(width)\
	    -step 1 -llimit 1 -ulimit 16]
    $wctrl config -command {setAttrLineWidth}
    pack $wlabel $wctrl -side left -expand yes -fill x -padx 0.5m -pady 5

    set rfbtn1 [button $rf.raise -text "Raise" -command \
	    "$canvwin raise \"$tag_name\"; $canvwin raise \"$label_name\""];
    set rfbtn2 [button $rf.lower -text "Lower" -command \
	    "$canvwin lower \"$tag_name\"; $canvwin lower \"$label_name\""];
    pack $rfbtn1 $rfbtn2 -side left -padx 5 -expand 1 -fill x -pady 5
    
    set okbtn [button $okf.ok -text "OK" -command \
	    "polylineConfig_update $sid $lid [list $layer] $attr_indx $canvwin; destroy $w"];
    pack $okbtn -expand 1 -fill x -pady 5

    grab $w;
    tkwait window $w;
    if $debugMode {puts "exit polylineConfig"}
    
}

# procedure to update the layer data structure and screen appearence
# after user is done with the configuration of point adt
proc polylineConfig_update {sid lid layer attr_indx canvwin} {
    global debugMode show_attrs;
    global bitmap_dir PDbitmaps;
    
    if $debugMode {
	puts "polylineConfig_update called";
	puts "\tsid = $sid";
	puts "\tlid = $lid";
	puts "\tlayer = $layer";
	puts "\tattr_indx = $attr_indx";
	puts "\tcanvas path = $canvwin";
    }

    setLayerShowAttrs layer $attr_indx;

    setVisibleLayerById $sid $lid $layer;

    # update the screen appearance thru tag
    set tag_name [format "l%s_%s" $lid $attr_indx];
    if $debugMode {
	puts "\tconfigure items with tag $tag_name";
    }
    $canvwin itemconfigure $tag_name -width $show_attrs(width);
    if [useColorPallete $show_attrs(color)] {
	$canvwin itemconfigure $tag_name -fill $show_attrs(color);
    }
    if {$show_attrs(label) == -1} {
	# remove label
	set ltag [format "l%slb%s" $lid $attr_indx];
	$canvwin delete $ltag;
    }
    
}

proc setAttrTransparent {widget status} {
    global show_attrs
    if {$widget == "transparent"} {
	if {$status == 0} {
	    set show_attrs(trans) 0
	} else {
	    set show_attrs(trans) 1
	}
    }
}

proc setAttrLineWidth {value} {
    global show_attrs
    set show_attrs(width) $value
}

# configure pointAdt appearance
# should have only one instance of configure
# dialog running at a time, so we need to
# make it mono dialog
proc pointConfig {sid attr_indx attr_types attr_names tags canvwin} {
    global session debugMode;
    global show_attrs attrConfigVar;
    
    if $debugMode {
	puts "pointConfig called";
	puts "\tsid = $sid";
	puts "\tattr index = $attr_indx";
	puts "\ttags = $tags";
    }

    set attr_name [lindex $attr_names $attr_indx];
    
    # pop up a mono dialog to configure the color, icon and
    # transparency information of point data

    # first, we need the current presentation information
    # from layer data structure
    set lid [tags::get_lid $tags];
    set layer [getVisibleLayerById $sid $lid];
    set tag_name [format "l%s_%s" $lid $attr_indx];
    set label_name [format "l%slb%s" $lid $attr_indx];
    
    # extract the color, icon and transparency information
    # and put them in an associative array
    getShowAttr $layer show_attrs $attr_indx;

    set intAttrsIndxs {};
    set i 0;
    foreach at $attr_types {
	if {$at == "Integer"} {
	    lappend intAttrsIndxs $i;
	}
	incr i;
    }

    if [useColorPallete $show_attrs(color)] {
	set color_sample "                           ";
	set bgcolr $show_attrs(color);
    } else {
	set color_sample \
		"colored by [lindex $attr_names $show_attrs(color)]";
	set bgcolr "white";
    }
    
    # finally, we are ready to show our dialog
    set w ".pointConfig$sid";
    toplevel $w;
    wm title $w "Configure Appearence";

    set len [string length "Configure $attr_name"]
    set title [label $w.msg -text "Configure $attr_name" \
	    -bd 2 -relief raised \
	    -fg $attrConfigVar(title_fg) \
	    -width $attrConfigVar(title_width) \
	    -height $attrConfigVar(title_height) \
	    -anchor c \
	    -font $attrConfigVar(title_font)];
    set cf [frame $w.cf -bd 1 -relief raised];
    set tf [frame $w.tf -bd 1 -relief raised];
    set bf [frame $w.bf -bd 1 -relief raised];
    set lf [frame $w.lf -bd 1 -relief raised];
    set rf [frame $w.rf -bd 1 -relief raised];
    set okf [frame $w.okf -bd 1 -relief raised];
    pack $title $cf $tf $bf $lf $rf $okf -side top -expand 1 -fill x;

    # set up the color frame
    set cbtn [menubutton $cf.mb -text "Choose Line Color" -menu $cf.mb.m\
	    -bd 2 -relief raised -width $attrConfigVar(button_width)];
    set csample [label $cf.show -text $color_sample -bg $bgcolr\
	    -bd 2 -relief sunken -anchor w];
    pack $cbtn $csample -side left -expand 1 -fill x -padx 0.5m \
	    -pady 5

    menu $cbtn.m;
    $cbtn.m add cascade -label "Color Palette" -menu $cbtn.m.c;
    if {$intAttrsIndxs != {}} {
	$cbtn.m add cascade -label "Integer Attributes" \
		-menu $cbtn.m.i;
	menu $cbtn.m.i;
	foreach i $intAttrsIndxs {
	    set attrn [lindex $attr_names $i];
	    set msg "Colored by <$attrn>"; 
	    set len [string length $msg]; 
	    $cbtn.m.i add command -label "$attrn" -command \
		    "set show_attrs(color) $i; \
		    $csample configure -text \"$msg\" -width $len \
		    -background white";
	}
    }

    global col_palete;
    set msg "                           ";
    set len [string length $msg]; 
    menu $cbtn.m.c;
    foreach color_cell $col_palete {
	$cbtn.m.c add command -label "$color_cell" \
		-background $color_cell -command \
		"set show_attrs(color) $color_cell; \
		$csample configure -text \"$msg\" -width $len \
		-background $color_cell";
    }

    # set up label frame
    if {$show_attrs(label) != -1} {
	set lshow_txt [lindex $attr_names $show_attrs(label)];
	set lshow_txt "Labeled by <$lshow_txt>";
    } else {
	set lshow_txt "None"
    }
    set lbtn [menubutton $lf.mb -text "Choose Label" -menu $lf.mb.m \
	    -bd 2 -relief raised -width $attrConfigVar(button_width)];
    set lshow [label $lf.show -text "$lshow_txt" -bg white -anchor w];
    pack $lbtn $lshow -side left -expand 1 -fill x -padx 0.5m \
	    -pady 5
    menu $lbtn.m;
    set msg "No Label";
    set len [string length $msg];
    $lbtn.m add command -label "None" -command \
	    "set show_attrs(label) -1; \
	    $lshow configure -text \"$msg\" -width $len";
    set i 0;
    foreach attrn $attr_names {
	set msg "Labeled by <$attrn>";
	set len [string length $msg];
	$lbtn.m add command -label "$attrn" -command \
		"set show_attrs(label) $i; \
		$lshow configure -text \"$msg\" -width $len";
	incr i;
    }

    # set up transparency checkbutton
    label $tf.label -text "Transparency: "
    set ttb [tixSelect $tf.sel \
	    -font "-adobe-helvetica-bold-r-normal-*-12-*-*-*-*-*-*-*"]
    $tf.sel add transparent -text "Transparent" -width 12
    $tf.sel add opaque -text "Opaque" -width 12
    if $show_attrs(trans) {
	$tf.sel config -value {transparent}
    } {
	$tf.sel config -value {opaque}
    }
    $tf.sel config -command {setAttrTransparent}
    pack $tf.label -side left -expand yes -fill x -pady 5
    pack $ttb -side left -pady 5
    
    set icon_label [label $bf.label -text "Icon:" \
	    -width $attrConfigVar(button_width)];
    set icon_mb [menubutton $bf.mbtn -menu $bf.mbtn.m \
	    -bitmap $show_attrs(bitmap) -bd 2 -relief raised];
    pack $icon_label $icon_mb -side left -padx 1m -fill x -pady 5 -expand 1;

    global PDbitmaps bitmap_dir;
    set icon_menu [menu $icon_mb.m];
    foreach bm $PDbitmaps {
	$icon_menu add radiobutton -bitmap $bm -variable show_attrs(bitmap) \
		-value $bm -command "$icon_mb configure -bitmap $bm";
    }

    set rfbtn1 [button $rf.raise -text "Raise" -command \
	    "$canvwin raise \"$tag_name\"; $canvwin raise \"$label_name\""];
    set rfbtn2 [button $rf.lower -text "Lower" -command \
	    "$canvwin lower \"$tag_name\"; $canvwin lower \"$label_name\""];
    pack $rfbtn1 $rfbtn2 -side left -padx 5 -expand 1 -fill x -pady 5;
    
    set okbtn [button $okf.ok -text "OK" -command \
	    "pointConfig_update $sid $lid [list $layer] $attr_indx $canvwin; destroy $w"];
    pack $okbtn -expand 1 -fill x -pady 5

    grab $w
    tkwait window $w;
    if $debugMode {puts "exit pointConfig"}
    
}

# procedure to update the layer data structure and screen appearence
# after user is done with the configuration of point adt
proc pointConfig_update {sid lid layer attr_indx canvwin} {
    global debugMode show_attrs;
    global bitmap_dir PDbitmaps;
    
    if $debugMode {
	puts "pointConfig_update called";
	puts "\tsid = $sid";
	puts "\tlid = $lid";
	puts "\tlayer = $layer";
	puts "\tattr_indx = $attr_indx";
	puts "\tcanvas path = $canvwin";
    }

    setLayerShowAttrs layer $attr_indx;

    setVisibleLayerById $sid $lid $layer;

    # update the screen appearance thru tag
    set tag_name [format "l%s_%s" $lid $attr_indx];
    if $debugMode {
	puts "\tconfigure items with tag $tag_name";
    }
    $canvwin itemconfigure $tag_name -opaque [expr 1 - $show_attrs(trans)] \
	    -bitmap $show_attrs(bitmap);
    if [useColorPallete $show_attrs(color)] {
	$canvwin itemconfigure $tag_name -foreground $show_attrs(color);
    }
    if {$show_attrs(label) == -1} {
	# remove label
	set ltag [format "l%slb%s" $lid $attr_indx];
	$canvwin delete $ltag;
    }
    
}

# configure rasterAdt appearance
# should have only one instance of configure
# dialog running at a time, so we need to
# make it mono dialog
proc rasterConfig {sid attr_indx attr_types attr_names tags canvwin} {
    global session debugMode;
    global show_attrs attrConfigVar ptk_map_canv;
    
    if $debugMode {
	puts "rasterConfig called";
	puts "\tsid = $sid";
	puts "\tattr index = $attr_indx";
	puts "\ttags = $tags";
    }

    set attr_name [lindex $attr_names $attr_indx];
    
    # pop up a mono dialog to configure the color, icon and
    # transparency information of point data

    # first, we need the current presentation information
    # from layer data structure
    set lid [tags::get_lid $tags];
    set layer [getVisibleLayerById $sid $lid];
    set tag_name [format "l%s_%s" $lid $attr_indx];
    set label_name [format "l%slb%s" $lid $attr_indx];
    
    # extract the color, icon and transparency information
    # and put them in an associative array
    getShowAttr $layer show_attrs $attr_indx;
    
    set intAttrsIndxs {};
    set i 0;
    foreach at $attr_types {
	if {$at == "Integer"} {
	    lappend intAttrsIndxs $i;
	}
	incr i;
    }

    if [useColorPallete $show_attrs(color)] {
	set color_sample "                           ";
	set bgcolr $show_attrs(color);
    } else {
	set color_sample \
		"colored by [lindex $attr_names $show_attrs(color)]";
	set bgcolr "white";
    }

    # finally, we are ready to show our dialog
    set w ".rasterConfig$sid";
    toplevel $w;
    wm title $w "Configure Appearence";

    set len [string length "Configure $attr_name"]
    set title [label $w.msg -text "Configure $attr_name" \
	    -bd 2 -relief raised \
	    -fg $attrConfigVar(title_fg) \
	    -anchor c \
	    -width $attrConfigVar(title_width) \
	    -height $attrConfigVar(title_height) \
	    -font $attrConfigVar(title_font)];
    set cf [frame $w.cf -bd 1 -relief raised];
    set tf [frame $w.tf -bd 1 -relief raised];
    set wf [frame $w.wf -bd 1 -relief raised];
    set lf [frame $w.lf -bd 1 -relief raised];
    set rf [frame $w.rf -bd 1 -relief raised];
    set okf [frame $w.okf -bd 1 -relief raised];
    pack $title $cf $tf $wf $lf $rf $okf -side top -expand 1 -fill x;

    # set up the color frame
    set cbtn [menubutton $cf.mb -text "Choose Outline Color" -menu $cf.mb.m\
	    -bd 2 -relief raised -width $attrConfigVar(button_width)];
    set csample [label $cf.show -text $color_sample -bg $bgcolr\
	    -bd 2 -relief sunken -anchor w];
    pack $cbtn $csample -side left -expand 1 -fill x -padx 0.5m -pady 5

    menu $cbtn.m;
    $cbtn.m add cascade -label "Color Palette" -menu $cbtn.m.c;
    if {$intAttrsIndxs != {}} {
	$cbtn.m add cascade -label "Integer Attributes" \
		-menu $cbtn.m.i;
	menu $cbtn.m.i;
	foreach i $intAttrsIndxs {
	    set attrn [lindex $attr_names $i];
	    set msg "Colored by <$attrn>"; 
	    set len [string length $msg]; 
	    $cbtn.m.i add command -label "$attrn" -command \
		    "set show_attrs(color) $i; \
		    $csample configure -text \"$msg\" -width $len \
		    -background white";
	}
    }

    global col_palete;
    set msg "                           ";
    set len [string length $msg]; 
    menu $cbtn.m.c;
    foreach color_cell $col_palete {
	$cbtn.m.c add command -label "$color_cell" \
		-background $color_cell -command \
		"set show_attrs(color) $color_cell; \
		$csample configure -text \"$msg\" -width $len \
		-background $color_cell";
    }

    # set up transparency checkbutton
    label $tf.label -text "Transparency: "
    set ttb [tixSelect $tf.sel \
	    -font "-adobe-helvetica-bold-r-normal-*-12-*-*-*-*-*-*-*"]
    $tf.sel add transparent -text "Transparent" -width 12
    $tf.sel add opaque -text "Opaque" -width 12
    if $show_attrs(trans) {
	$tf.sel config -value {transparent}
    } {
	$tf.sel config -value {opaque}
    }
    $tf.sel config -command {setAttrTransparent}
    pack $tf.label -side left -expand yes -fill x -pady 5
    pack $ttb -side left -pady 5

    # set up label frame
    if {$show_attrs(label) != -1} {
	set lshow_txt [lindex $attr_names $show_attrs(label)];
	set lshow_txt "Labeled by <$lshow_txt>";
    } else {
	set lshow_txt "None"
    }
    set lbtn [menubutton $lf.mb -text "Choose Label" -menu $lf.mb.m \
	    -bd 2 -relief raised -width $attrConfigVar(button_width)];
    set lshow [label $lf.show -text "$lshow_txt" -bg white -anchor w];
    pack $lbtn $lshow -side left -expand 1 -fill x -padx 0.5m -pady 5
    menu $lbtn.m;
    set msg "No Label";
    set len [string length $msg];
    $lbtn.m add command -label "None" -command \
	    "set show_attrs(label) -1; \
	    $lshow configure -text \"$msg\" -width $len";
    set i 0;
    foreach attrn $attr_names {
	set msg "Labeled by <$attrn>";
	set len [string length $msg];
	$lbtn.m add command -label "$attrn" -command \
		"set show_attrs(label) $i; \
		$lshow configure -text \"$msg\" -width $len";
	incr i;
    }

    # set up the width frame
    set wlabel [label $wf.l -text "Outline Width"];
    set wctrl [tixControl $wf.ctl -entrywidth 8 -value $show_attrs(width)\
	    -step 1 -llimit 1 -ulimit 16]
    $wctrl config -command {setAttrLineWidth}
    pack $wlabel $wctrl -side left -expand yes -fill x -padx 0.5m -pady 5

    set rfbtn1 [button $rf.raise -text "Raise" -command \
	    "$canvwin raise \"$tag_name\"; $canvwin raise \"$label_name\""];
    set rfbtn2 [button $rf.lower -text "Lower" -command \
	    "$canvwin lower \"$tag_name\"; $canvwin lower \"$label_name\""];
    pack $rfbtn1 $rfbtn2 -side left -padx 5 -expand 1 -fill x -pady 5
    
    set okbtn [button $okf.ok -text "OK" -command \
	    "rasterConfig_update $sid $lid [list $layer] $attr_indx $canvwin; destroy $w"];
    pack $okbtn -expand 1 -fill x -pady 5

    grab $w;
    tkwait window $w;
    if $debugMode {puts "exit pointConfig"}
    
}

# procedure to update the layer data structure and screen appearence
# after user is done with the configuration of point adt
proc rasterConfig_update {sid lid layer attr_indx canvwin} {
    global debugMode show_attrs;
    global bitmap_dir PDbitmaps;
    
    if $debugMode {
	puts "rasterConfig_update called";
	puts "\tsid = $sid";
	puts "\tlid = $lid";
	puts "\tlayer = $layer";
	puts "\tattr_indx = $attr_indx";
	puts "\tcanvas path = $canvwin";
    }

    setLayerShowAttrs layer $attr_indx;

    setVisibleLayerById $sid $lid $layer;

    # update the screen appearance thru tag
    set tag_name [format "l%s_%s" $lid $attr_indx];
    if $debugMode {
	puts "\tconfigure items with tag $tag_name";
    }
    $canvwin itemconfigure $tag_name -opaque [expr 1 - $show_attrs(trans)] \
	    -width $show_attrs(width);
    if [useColorPallete $show_attrs(color)] {
	$canvwin itemconfigure $tag_name -outline $show_attrs(color);
    }
    if {$show_attrs(label) == -1} {
	# remove label
	set ltag [format "l%slb%s" $lid $attr_indx];
	$canvwin delete $ltag;
    }
    
}

# configure closedPolygonAdt appearance
# should have only one instance of configure
# dialog running at a time, so we need to
# make it mono dialog
proc closedPolygonConfig {sid attr_indx attr_types attr_names tags canvwin} {
    global session debugMode;
    global show_attrs ptk_map_canv attrConfigVar;
    
    if $debugMode {
	puts "closedPolygonConfig called";
	puts "\tsid = $sid";
	puts "\tattr index = $attr_indx";
	puts "\ttags = $tags";
    }

    set attr_name [lindex $attr_names $attr_indx];
    
    # pop up a mono dialog to configure the color, icon and
    # transparency information of point data

    # first, we need the current presentation information
    # from layer data structure
    set lid [tags::get_lid $tags];
    set layer [getVisibleLayerById $sid $lid];
    set tag_name [format "l%s_%s" $lid $attr_indx];
    set label_name [format "l%slb%s" $lid $attr_indx];
    
    # extract the color, icon and transparency information
    # and put them in an associative array
    getShowAttr $layer show_attrs $attr_indx;

    set intAttrsIndxs {};
    set i 0;
    foreach at $attr_types {
	if {$at == "Integer"} {
	    lappend intAttrsIndxs $i;
	}
	incr i;
    }

    if [useColorPallete $show_attrs(color)] {
	set color_sample "                           ";
	set bgcolr $show_attrs(color);
    } else {
	set color_sample \
		"colored by [lindex $attr_names $show_attrs(color)]";
	set bgcolr "white";
    }
    
    # finally, we are ready to show our dialog
    set w ".closedPolygonConfig$sid";
    toplevel $w;
    wm title $w "Configure Appearence";

    set len [string length "Configure $attr_name"]
    set title [label $w.msg -text "Configure $attr_name" \
	    -bd 2 -relief raised \
	    -fg $attrConfigVar(title_fg) \
	    -anchor c \
	    -width $attrConfigVar(title_width) \
	    -height $attrConfigVar(title_height) \
	    -font $attrConfigVar(title_font)];
    set cf [frame $w.cf -bd 1 -relief raised];
    set lf [frame $w.lf -bd 1 -relief raised];
    set rf [frame $w.rf -bd 1 -relief raised];
    set okf [frame $w.okf -bd 1 -relief raised];
    pack $title $cf $lf $rf $okf -side top -expand 1 -fill x;

    # set up the color frame
    set cbtn [menubutton $cf.mb -text "Choose Fill Color" -menu $cf.mb.m\
	    -bd 2 -relief raised -width $attrConfigVar(button_width)];
    set csample [label $cf.show -text $color_sample -bg $bgcolr\
	    -bd 2 -relief sunken -anchor w];
    pack $cbtn $csample -side left -expand 1 -fill x -padx 0.5m -pady 5

    menu $cbtn.m;
    $cbtn.m add cascade -label "Color Palette" -menu $cbtn.m.c;
    if {$intAttrsIndxs != {}} {
	$cbtn.m add cascade -label "Integer Attributes" \
		-menu $cbtn.m.i;
	menu $cbtn.m.i;
	foreach i $intAttrsIndxs {
	    set attrn [lindex $attr_names $i];
	    set msg "Colored by <$attrn>"; 
	    set len [string length $msg]; 
	    $cbtn.m.i add command -label "$attrn" -command \
		    "set show_attrs(color) $i; \
		    $csample configure -text \"$msg\" -width $len \
		    -background white";
	}
    }

    global col_palete;
    set msg "                           ";
    set len [string length $msg]; 
    menu $cbtn.m.c;
    foreach color_cell $col_palete {
	$cbtn.m.c add command -label "$color_cell" \
		-background $color_cell -command \
		"set show_attrs(color) $color_cell; \
		$csample configure -text \"$msg\" -width $len \
		-background $color_cell";
    }

    # set up label frame
    if {$show_attrs(label) != -1} {
	set lshow_txt [lindex $attr_names $show_attrs(label)];
	set lshow_txt "Labeled by <$lshow_txt>";
    } else {
	set lshow_txt "None"
    }
    set lbtn [menubutton $lf.mb -text "Choose Label" -menu $lf.mb.m \
	    -bd 2 -relief raised -width $attrConfigVar(button_width)];
    set lshow [label $lf.show -text "$lshow_txt" -bg white -anchor w];
    pack $lbtn $lshow -side left -expand 1 -fill x -padx 0.5m -pady 5
    menu $lbtn.m;
    set msg "No Label";
    set len [string length $msg];
    $lbtn.m add command -label "None" -command \
	    "set show_attrs(label) -1; \
	    $lshow configure -text \"$msg\" -width $len";
    set i 0;
    foreach attrn $attr_names {
	set msg "Labeled by <$attrn>";
	set len [string length $msg];
	$lbtn.m add command -label "$attrn" -command \
		"set show_attrs(label) $i; \
		$lshow configure -text \"$msg\" -width $len";
	incr i;
    }

    set rfbtn1 [button $rf.raise -text "Raise" -command \
	    "$canvwin raise \"$tag_name\"; $canvwin raise \"$label_name\""];
    set rfbtn2 [button $rf.lower -text "Lower" -command \
	    "$canvwin lower \"$tag_name\"; $canvwin lower \"$label_name\""];
    pack $rfbtn1 $rfbtn2 -side left -padx 5 -expand 1 -fill x -pady 5
    set okbtn [button $okf.ok -text "OK" -command \
	    "closedPolygonConfig_update $sid $lid [list $layer] $attr_indx $canvwin; destroy $w"];
    pack $okbtn -expand 1 -fill x -pady 5

    grab $w;
    tkwait window $w;
    if $debugMode {puts "exit pointConfig"}
    
}

# procedure to update the layer data structure and screen appearence
# after user is done with the configuration
proc closedPolygonConfig_update {sid lid layer attr_indx canvwin} {
    global debugMode show_attrs;

    if $debugMode {
	puts "closedPolygonConfig_update called";
	puts "\tsid = $sid";
	puts "\tlid = $lid";
	puts "\tlayer = $layer";
	puts "\tattr_indx = $attr_indx";
	puts "\tcanvas path = $canvwin";
    }

    setLayerShowAttrs layer $attr_indx;
    setVisibleLayerById $sid $lid $layer;

    # update the screen appearance thru tag
    set tag_name [format "l%s_%s" $lid $attr_indx];
    if $debugMode {
	puts "\tconfigure items with tag $tag_name";
    }
    if [useColorPallete $show_attrs(color)] {
	$canvwin itemconfigure $tag_name -fill $show_attrs(color);
    }
    if {$show_attrs(label) == -1} {
	# remove label
	set ltag [format "l%slb%s" $lid $attr_indx];
	$canvwin delete $ltag;
    }
}

# 
# some utility calls to extract information out of $tags
# of spatial objects

# decide whether the color is from color pallete or from another attribute
# if the color is from color pallete, the result of change will be shown
# on screen immediately, otherwise it will be shown only when you do
# refresh
proc useColorPallete color {
    global debugMode;

    if $debugMode {
	puts "useColorPallete $color called";
    }
    
    if [string match {[0-9]*} $color] {
	return 0;
    } else {
	# yes
	return 1;
    }
}

# get the layer_id from tags
proc tags::get_lid {tags} {
    global  debugMode;

    if $debugMode {
	puts "tags::get_lid called";
	puts "\ttags = $tags";
    }

    set layer_id {};
    foreach tag $tags {
	if {$tag == {}} {continue}
	if [string match {l[0-9]*} [lindex $tag 0]] {
	    set layer_id [string range [lindex $tag 0] 1 end];
	    break;
	}
    }

    return $layer_id;
}

# get the oid from tags
proc tags::get_oid {tags} {
    global debugMode
    if $debugMode {
	puts "tags::get_oid $tags"
    }

    set oid {}
    foreach tag $tags {
	if {$tag == {}} {continue}
	if [string match {o[0-9]*} [lindex $tag 1]] {
	    set oid [string range [lindex $tag 1] 1 end];
	    break;
	}
    }
    return $oid
}

# get the tmpf handle

#
# extract the current presentation characteristics of this
# attribute and return them in an associative array
proc getShowAttr {layer show_attrs attr_indx} {
    upvar $show_attrs s;

    set s(color) 	[lindex [getLayerColor $layer] $attr_indx];
    set s(bitmap) [lindex [getLayerBitmap $layer] $attr_indx];
    set s(trans)	[lindex [getLayerTrans $layer] $attr_indx];
    set s(width)	[lindex [getLayerWidth $layer] $attr_indx];
    set s(label)	[lindex [getLayerLabel $layer] $attr_indx];
}

#
# store the update of show_attrs back to a list before
# we put it back to layer data structure.
#
proc setLayerShowAttrs {layer attr_indx} {
    upvar $layer l;
    global show_attrs;

    set oldbmap [lindex [getLayerBitmap $l] $attr_indx];
    set oldcolor [lindex [getLayerColor $l] $attr_indx];
    set oldtrans [lindex [getLayerTrans $l] $attr_indx];
    set oldwidth [lindex [getLayerWidth $l] $attr_indx];
    set oldlabel [lindex [getLayerLabel $l] $attr_indx];

    if {$oldbmap != $show_attrs(bitmap)} {
	set bmaps [lreplace [getLayerBitmap $l] $attr_indx $attr_indx \
		$show_attrs(bitmap)];
	set l [setLayerBitmap $l $bmaps];
    }
    if {$oldcolor != $show_attrs(color)} {
	set colors [lreplace [getLayerColor $l] $attr_indx $attr_indx \
		$show_attrs(color)];
	set l [setLayerColor $l $colors];
	if ![useColorPallete $show_attrs(color)] {
	    pd_dialog .warning {Warning} \
		    {Changes made to color will not take effect until you do Refresh!} \
		    {} warning 0 {OK};
	}
    }
    if {$oldtrans != $show_attrs(trans)} {
	set trans [lreplace [getLayerTrans $l] $attr_indx $attr_indx \
		$show_attrs(trans)];
	set l [setLayerTrans $l $trans];
    }
    if {$oldwidth != $show_attrs(width)} {
	set width [lreplace [getLayerWidth $l] $attr_indx $attr_indx \
		$show_attrs(width)];
	set l [setLayerWidth $l $width];
    }
    if {$oldlabel != $show_attrs(label)} {
	set label [lreplace [getLayerLabel $l] $attr_indx $attr_indx \
		$show_attrs(label)];
	set l [setLayerLabel $l $label];
	if {$oldlabel == -1 && $show_attrs(label) > -1} {
	    pd_dialog .warning {Warning} \
		    {New labels will not show up until you do Refresh!} \
		    {} warning 0 {OK};
	}
    }
}

