# <std-header style=TCL orig-src=PTK>
#
#            Copyright 1998, NCR Corporation.  All Rights Reserved.
#
#                                NCR Confidential.
#
#
#  $Id: dbcmds.tcl,v 1.11 1998/05/19 03:22:22 kupsch Exp $
#
#  This file was originally part of ptk, a front end based on TCL/TK to the
#  Paradise database system developed by the University of Wisconsin-Madison.
#  The above copyright applies to changes made to this file by NCR Corporation.
#  The original copyright which may appear in the source below applies to the
#  original file as obtained by NCR Corporation and unchanged parts of this file.
#
#
#   -- do not edit anything above this line --   </std-header>

#
# extcmds.tcl
#
# Contains all extent related commands.
#

proc crtdb_cancel {} {
  destroy .crtdb;
  return;
}

proc crtdb_done {sid} {
  set dbn [.crtdb.entry get];
  if {$dbn != ""} {
    PDsql_cursorrun "create db $dbn"
    refresh $sid;
  }
  destroy .crtdb;
  return;
}

proc create_db {sid} {
  catch "destroy .crtdb";
  toplevel .crtdb;
  wm title .crtdb "Create New Database";
  wm geometry .crtdb +260+100

  label .crtdb.label -text "Please enter new DB name:";
  entry .crtdb.entry -relief sunken -width 16;
  frame .crtdb.btns;
  button .crtdb.btns.done -relief raised -text "Done" \
	-command "crtdb_done $sid";
  button .crtdb.btns.cancel -relief raised -text "Cancel" \
    -command "crtdb_cancel";
  pack .crtdb.label -side top -expand 1;
  pack .crtdb.entry -side top -expand 1;
  pack .crtdb.btns -side top -expand 1;
  pack .crtdb.btns.done .crtdb.btns.cancel -in .crtdb.btns -side left \
    -padx 3m -pady 3m;
  
  bind .crtdb.entry <Key-Return> "crtdb_done $sid";
  focus .crtdb.entry;
  set old_grab [grab current]
  grab set .crtdb
  tkwait window .crtdb

  # on SPARC, grag set "" will core dump because strlen doesn't
  # like empty string
  if {$old_grab != ""} {
  	grab set $old_grab
  } else {
	grab release .crtdb
  }
}

# a simple table keeping track of whether a db is open or closed
set ptk_dbstat(geo) 0

proc ptk_open_db {sid dbn} {
	global ptk_dbstat session
	global debugMode

        # short circuit for now
        return;

	if [catch "set ptk_dbstat($dbn)"] {
		# dbn haven't been opened before
		set ptk_dbstat($dbn) 1
		if $debugMode {puts "open db $dbn"}
		PDsql_cursorrun "open db $dbn"
		set session($sid,curdb) $dbn
		return
	} 
	if $ptk_dbstat($dbn) {
		# database already opened
		set session($sid,curdb) $dbn
		return
	} else {
		set ptk_dbstat($dbn) 1
		if {$ptk_dbstat($session($sid,curdb)) == 1 &&
				$session($sid,curdb) != $dbn} {
			set ptk_dbstat($session($sid,curdb)) 0
			PDsql_cursorrun "close db $session($sid,curdb)"
			set session($sid,curdb) $dbn
		}
		PDsql_cursorrun "open db $dbn"
		return
	}
}

proc ptk_close_db {sid dbn} {
	global ptk_dbstat session
	global debugMode

	  # short circuit for now
	  return;
	if [catch "set ptk_dbstat($dbn)"] {
		# db not been used before, not opened
		puts "Warning: try to close never used database $dbn!"
		return
	}
	if $ptk_dbstat($dbn) {
		set ptk_dbstat($dbn) 0
		if $debugMode {puts "close db $dbn"}
		PDsql_cursorrun "close db $dbn"
	} else {
		# db already closed, do nothing
		return
	}
}

