# <std-header style=TCL orig-src=PTK>
#
#            Copyright 1998, NCR Corporation.  All Rights Reserved.
#
#                                NCR Confidential.
#
#
#  $Id: drawLayer.tcl,v 1.35 1998/07/23 04:21:56 kristint Exp $
#
#  This file was originally part of ptk, a front end based on TCL/TK to the
#  Paradise database system developed by the University of Wisconsin-Madison.
#  The above copyright applies to changes made to this file by NCR Corporation.
#  The original copyright which may appear in the source below applies to the
#  original file as obtained by NCR Corporation and unchanged parts of this file.
#
#
#   -- do not edit anything above this line --   </std-header>

#----------------------------------------------------------------------
# This file contains the tcl code to draw layer on screen.  It is associated
# with the context of layerdialog.tcl and mapview.tcl.
#----------------------------------------------------------------------

# ------------------------------------------------------------
# <showAllLayers> is used in layerdialog.tcl, associated with
# button refresh.  Is is also used in mapview.tcl, also assoc.
# with button refresh in mapview window.  This proc is intended
# to be called from PDvalid_map.
#
# [Arguments]
#	-canv:		Canvas that the layers should be shown
#	-sid:		session id
#
# [Side effects]
#	layers are draw on map window.
#
# [Notes]
#	It uses the *layer* information stored in *session* global variable.
#	The *hidden_layer* information stored in *session* is not used
#	because these layers are not meant to be displayed.
#
proc showAllLayers {canv sid} {
    global session

    ptkPDWait ON
    ptkPDStatBox ON
    foreach layer $session($sid,layers) {
	ptkPDStatBox::setMsg "Refreshing [getLayerName $layer]..."
	showOneLayer $canv $layer $sid "false"
    }
    ptkPDStatBox OFF
    ptkPDWait OFF
}

# ------------------------------------------------------------
# <showOneLayer> displays the given layer on top of the current
# map view canvas window.  IF the given layer is empty or its
# query is empty, nothing will be displayed, the control returns
# to the caller.
#	Otherwise, we first obtain the description of result
# relation and to check for type of spatial attribute.
#
# [Arguments]
#	-canv		The canvas window to show our pretty pic
#	-layer		The layer description
#	-sid		session id
#
proc showOneLayer {canv layer sid force_redo} {
    global session 
    global debugMode ptk_label_font LAYER_LEVELSUFIX LAYER_UPDATE LAYER_LEVEL

    if $debugMode {puts "Enter showOneLayer $layer"}
    # return if we can't do anything
    if {$layer == {}} {return}

    set query [getLayerQuery $layer $sid "true"]
    if {$query == "NONE"} { return }

    # check whether we have centered the layer and thus
    # we can just scan the temporary file
    if { $session($sid,center_tmpfn) != "NONE" && $force_redo == "false"} {
	set query "select * from $session($sid,center_tmpfn)"
	set session($sid,center_tmpfn) "NONE"
    }



    set level [lindex $layer $LAYER_LEVEL]
    if { ($level != "ALL") && ($level != "all") \
	    && (![regexp {__PTmpF_} "$query"])} {
	set q [lindex $layer $LAYER_LEVELSUFIX]
	if [regexp {where} "$query"] {
	    regsub {where} $q {and} q
	}
	append query $q
		set query [setDispLevel $query $sid $level]
    }
    
    set color [getLayerColor $layer]
    set bmap  [getLayerBitmap $layer]

    set trans_flag [getLayerTrans $layer];
    set opaque_flag {};
    foreach flag $trans_flag {
	lappend opaque_flag [expr 1 - $flag];
    }
    set width [getLayerWidth $layer]
    set lid [getLayerId $layer]
    set lidtag "l$lid"
    set label_indx [getLayerLabel $layer]
    $canv dtag toplayer

    # get the type of spatial attribute;
    set updateflg [lindex $layer $LAYER_UPDATE]
    set rc [PDsql_cursoropen $query "nomimic" $updateflg $session($sid,bx1) \
		$session($sid,by1) $session($sid,bx2) $session($sid,by2) \
		$session($sid,view_w) $session($sid,view_h)]
    if { $rc == "NO_TUPLE" } { return }

    if $debugMode {
	set tmpfn [PDcursorlayerdisp $canv $color $bmap $opaque_flag \
		$width "l$lid" $ptk_label_font $label_indx -v]
    } else {
	set tmpfn [PDcursorlayerdisp $canv $color $bmap $opaque_flag \
		$width "l$lid" $ptk_label_font $label_indx]
    }
    
    # reset query_changed flag and save the tmpfn
    if [getLayerQflag $layer] {
	resetLayerQflag $sid $lid
    }
    set oldtmpfn [getLayerTmpfn $layer]
    if {[string compare $oldtmpfn $tmpfn] != 0}  {
        if [regexp "__PTmpF_" $oldtmpfn] {
	    PDsql_cursorrun "drop table $oldtmpfn"
	}
	setLayerTmpfn $sid $lid $tmpfn
    }
    return
}

