# <std-header style=TCL orig-src=PTK>
#
#            Copyright 1998, NCR Corporation.  All Rights Reserved.
#
#                                NCR Confidential.
#
#
#  $Id: fontChooser.tcl,v 1.5 1998/05/19 03:22:23 kupsch Exp $
#
#  This file was originally part of ptk, a front end based on TCL/TK to the
#  Paradise database system developed by the University of Wisconsin-Madison.
#  The above copyright applies to changes made to this file by NCR Corporation.
#  The original copyright which may appear in the source below applies to the
#  original file as obtained by NCR Corporation and unchanged parts of this file.
#
#
#   -- do not edit anything above this line --   </std-header>

#
# This is a simple font chooser.
# Return a X11 font name chosen.
#
set fontchooser(font) {}

proc chooseFont {} {
	global fontchooser

	set w .fontChooser
	catch "destroy $w"
	toplevel $w
	wm title $w "Font Chooser"
	wm geometry $w +260+100

	set frame1 [mkTrim $w.frame1 {top pady 0.5m} 1m 0.5m raised]
	set fontList [mkListbox $frame1.fontlist {top expand fill}\
		-geometry 80x25]

	set frame2 [mkFrame $w.frame2 {top expand fill pady 1m} -height 1m]
	set preview [mkLabel $frame2.preview "A Brown Fox Jumps ..." \
		{fill expand  padx 1m} -bd 3 -relief sunken \
		-bg white -fg black]

	set frame3 [mkFrame $w.frame3 {top expand fill pady 1m}]
	set prevBut [mkButton $frame3.prev Preview \
		"chooseFontPreviewCmd $fontList $preview"]
	set okBut [mkButton $frame3.ok OK "chooseFontOkCmd $fontList $w"]
	set cancelBut [mkButton $frame3.cancel CANCEL "chooseFontCancelCmd $w"]
	pack append $frame3 $prevBut {left padx 6 pady 3 expand fillx} \
		$okBut {left padx 6 pady 3 expand fillx} \
		$cancelBut {left padx 6 pady 3 expand fillx}

	# some bindings
	bind $fontList <Double-Button-1> "$prevBut invoke"

	# load in fonts and put into list box
	set fd [open |xlsfonts r]
	while {[gets $fd val] != -1} {
		$fontList insert end $val
	}
	close $fd

	# modal dialog
	set old_grab [grab current]
	grab set $w
	tkwait window $w
	if {$old_grab != ""} {
		grab set $old_grab
	}

	return $fontchooser(font)
}

#----------------------------------------------------------------------
# chooseFontCancelCmd: return empty string, and destroy main window
#----------------------------------------------------------------------
proc chooseFontCancelCmd w {
	global fontchooser

	set fontchooser(font) {}
	destroy $w
}

#----------------------------------------------------------------------
# chooseFontOkCmd: set return font as the current selected font
# and destroy the main window
#----------------------------------------------------------------------
proc chooseFontOkCmd {fontw w} {
	global fontchooser

	set selectionOwner [selection own]
	if {$selectionOwner != $fontw} {
		tk_dialog .layerWarning {Warning} {Please select a font \
			first!} warning 0 OK
		return
	}
	set fontchooser(font) [selection get]
	destroy $w
}

#----------------------------------------------------------------------
# preview a font
#----------------------------------------------------------------------
proc chooseFontPreviewCmd {fontw prevw} {
	set selectionOwner [selection own]
	if {$selectionOwner != $fontw} {
		tk_dialog .layerWarning {Warning} {Please select a font \
			first!} warning 0 OK
		return
	}

	set font [selection get]
	$prevw configure -font $font
}

