# <std-header style=TCL orig-src=PTK>
#
#            Copyright 1998, NCR Corporation.  All Rights Reserved.
#
#                                NCR Confidential.
#
#
#  $Id: history.tcl,v 1.7 1998/05/19 03:22:24 kupsch Exp $
#
#  This file was originally part of ptk, a front end based on TCL/TK to the
#  Paradise database system developed by the University of Wisconsin-Madison.
#  The above copyright applies to changes made to this file by NCR Corporation.
#  The original copyright which may appear in the source below applies to the
#  original file as obtained by NCR Corporation and unchanged parts of this file.
#
#
#   -- do not edit anything above this line --   </std-header>

# Implement a general history tool:  add a string to history file
# or view a history file in a toplevel window
#

# configurable history attributes;	
#set history(font) "fixed";

proc history::add {str fn} {
    set fd [open $fn a]

    # put a separation mark
    puts $fd "//===*===*==="
    puts $fd $str

    close $fd
}

proc history_view {fn btn} {
    if ![file exists $fn] {
	tk_dialog .hist_err {Error} "No history file \"$fn\" found!"\
		error 0 OK
	return
    }
    
    global history;
    catch {destroy .history_view}
    set w ".history_view";
    toplevel $w;
    wm title $w "Query History";

    set hist [frame $w.hist];
    scrollbar $hist.scroll -relief flat -command "$hist.t yview";
    text $hist.t -width 80 -height 20 -yscrollcommand "$hist.scroll set" \
	    -bd 2 -relief sunken -setgrid 1 -wrap word \
	    -state normal

    pack append $hist $hist.scroll {right filly} $hist.t {pady 1m};

    set bot [frame $hist.bot];
    pack $hist $bot -side top -pady 1m -expand 1 -fill x;

    set rescan_btn [button $bot.rescan -text "Rescan" -command \
	    "history_rescan $fn $hist.t"];
    set dismiss_btn [button $bot.dismiss -text "Dismiss" -command \
	    "history_dismiss $w $btn"];
    pack $rescan_btn $dismiss_btn -side left -padx .5m -expand 1 -fill x;
    
    history_load $fn $hist.t;

    # check to see view history button is still there
    # it may not, since history view can float around
    # longer than query composer window
    if [winfo exists $btn] {
	$btn configure -state disabled;
    }
}

# read in the contents of history file and put it
# in text widget
proc history_load {fn win} {
    set fd [open $fn r];
    while {[gets $fd line] >= 0} {
	$win insert current "$line\n";
    }
    close $fd;
}  

proc history_rescan {fn win} {
    # clear the text window first
    $win delete 0.0 end;

    # reload the history file content
    history_load $fn $win;
}

proc history_dismiss {w btn} {
    destroy $w;
    if [winfo exists $btn] {
	$btn configure -state normal;
    }
}

