# <std-header style=TCL orig-src=PTK>
#
#            Copyright 1998, NCR Corporation.  All Rights Reserved.
#
#                                NCR Confidential.
#
#
#  $Id: isql.tcl,v 1.17 1998/05/19 03:22:24 kupsch Exp $
#
#  This file was originally part of ptk, a front end based on TCL/TK to the
#  Paradise database system developed by the University of Wisconsin-Madison.
#  The above copyright applies to changes made to this file by NCR Corporation.
#  The original copyright which may appear in the source below applies to the
#  original file as obtained by NCR Corporation and unchanged parts of this file.
#
#
#   -- do not edit anything above this line --   </std-header>

# SQL interpreter:
#
#       It read SQL queries from standard input, execute each one
#       at a time, the result of the query will show up on standard
#       output, unless silent_mode gets turned on.
#
#       Lines begin with "//" are comments and are ignored
#       End of file or "quit" quit the front end
#

proc pd_isql {time_mode silent_mode} {
    global debugMode
    
    puts -nonewline "Ready> "
    set q [get_query]
	while {![eof stdin] && ([string length $q] != 4 || [string tolower $q] != "quit")} {
    if { 0 && [lindex $q 0] == "sys"} {
        set q [lrange $q 1 [expr [llength $q] - 1] ]
		eval $q
	} elseif { [lindex $q 0] == "connect"} {
        set q [lrange $q 1 [expr [llength $q] - 1] ]
		set rc [ catch {PDlogin $q} err]
		if $rc {
	    	puts stderr $err
		}
	} {
        if {$q != ""}   {
            if $time_mode {
                puts "Elapsed time: [time {run_query $q $silent_mode}]"
            } {
                run_query $q $silent_mode
            }
        }
    }
    puts -nonewline "Ready> "
    set q [get_query]
    }
    puts "Client finished"
}

proc get_query {} {
    global debugMode

    set q {}
    gets stdin a
    set a [string trim $a]
    if {[eof stdin] || $a == "quit"} {return "quit"}
    set tail [string length $a]
    incr tail -1
    while {[string index $a $tail] != ";"} {
	if ![regexp "^//" $a] {
	    append q "\n"
	    append q $a
	}
	gets stdin a
	if {[eof stdin] || [string trim $q] == "quit"} {break}
	set a [string trim $a]
	set tail [string length $a]
	incr tail -1
    }
    if ![regexp "^//" $a] {
	append q "\n"
	append q [string trimright $a ";"]
    }
    return [string trim $q]
}

proc run_query {q silent_mode} {
    global debugMode
    
    if $debugMode {puts "($silent_mode) The query is \"$q\""}
    set hasResult [expr [regexp {^select*} $q] || [regexp {^execute plan} $q]]
    if {$hasResult} {

	# run the query
	set rc [ catch {PDsql_cursoropen $q} err]
	if $rc {
	    puts stderr $err
	    return
	}
        # if { $err == "NO_TUPLE" } { return }

	# get result extent schema
	set desc [PDsql_cursorschema]
	set fname [getTmpfnFromDesc $desc]
	
	# empty tmp file name is possible for internal
	# debugging purpose
	if {$fname == ""} {
	    # force silent mode
	    set silent_mode 1
	}
	if {$silent_mode == 0} {
	    set attrn [getAttrNamesFromDesc $desc]
	    set numAttr 0
	    foreach n $attrn {
		puts -nonewline [format "| %-12s" $n]
		incr numAttr
	    }
	    puts "\n-------------------------------------------------------------------------------"

	    set val [PDsql_cursorfetch]
	    while {"$val" != {}} {
		set attrCnt 0
		foreach v $val {
		    incr attrCnt
		    if {$attrCnt < $numAttr} {
			puts -nonewline [format "| %-12s" "$v"]
		    } else {
			puts -nonewline "| AN OID"
		    }
		}
		puts ""
		set val [PDsql_cursorfetch]	    
	    }
	}
	if [is_fname_tmp $fname] {
	    PDsql_cursorrun "drop table $fname"
            # PDsql_cursorclose
	} 
    } else {
	set rc [catch {PDsql_cursorrun $q} err]
	if $rc {
	    puts stderr $err
	}
    }
}

proc is_fname_tmp {fname} {
    return [string match {__PTmpF_*} $fname]
}

