# <std-header style=TCL orig-src=PTK>
#
#            Copyright 1998, NCR Corporation.  All Rights Reserved.
#
#                                NCR Confidential.
#
#
#  $Id: layer.tcl,v 1.27 1998/07/23 04:21:56 kristint Exp $
#
#  This file was originally part of ptk, a front end based on TCL/TK to the
#  Paradise database system developed by the University of Wisconsin-Madison.
#  The above copyright applies to changes made to this file by NCR Corporation.
#  The original copyright which may appear in the source below applies to the
#  original file as obtained by NCR Corporation and unchanged parts of this file.
#
#
#   -- do not edit anything above this line --   </std-header>

# This file contains the procedures to manipulate the layer list
#
# ChangeLog:
# 1. 02/24/1994:	getLayer query will no automatically attach the
#			current bounding box to query, make it transparent
#			to show*Layer routines .
#
# 2. 02/24/94:		added a field curdb to make editlayer possible.
#
# 3. 02/28/94:		deleted the dispAttrType field, now the display
#			attribute are no longer stored in layer, instead,
#			they are extracted from result relation each time
#			the query is executed.
#			To facilitate the later changes to layer data
#			structure, use Named macros instead of magic
#			numbers.
#

# getLayerNames will return a list of the names of all layer
# Argument:
#	+ layers:	The layer data structure
#
# Caveat: Assuming that the first element of a layer is layer name.
#
# Return a list of names.
proc getLayerNames layers {
	global LAYER_LAYERNAME
	# need to init *names* in case the loop is skipped
	set names {}
	foreach i $layers {
		lappend names [lindex $i $LAYER_LAYERNAME]
	}
	return $names
}

# ------------------------------------------------------------
# <getLayerName> returns the name of a given layer.
#
# [Arguments]
#	- layer:	Data structure of a single layer
#
# [Return]
#	- the name of the layer
#
proc getLayerName layer {
	global LAYER_LAYERNAME
	return [lindex $layer $LAYER_LAYERNAME]
}

# ------------------------------------------------------------
# <getLayerQuery> extract the query associated with layer
# and put in the current bounding box info if necessary.
# This one need session id :-(  But we get to change the query
# as the current bounding box changes.
# Now it is even smart enough to return a select * from tmpfn
# if the query is not needed to be re-issued.
#
# [Arguments]
#	- layer:	Data structure of a single layer
#	- sid:		session i
#	- expand_flag:	expand the ##PDMAP## or not,
#			with the fact that only composer set
#			expand_flag to 0, when expand_flag == 0
#			must return the original query, not
#			tmpfile scan
#
# [Return]
#	- the SQL query associated with the layer
#
proc getLayerQuery {layer sid force_redo {expand_flag 1}} {
	global session LAYER_SQLQUERY LAYER_MAPSUFIX LAYER_LEVELSUFIX \
		TMPFN QFLAG LAYER_LEVEL
	global debugMode 

	set qflag [lindex $layer $QFLAG]
	set tmpfn [lindex $layer $TMPFN];
	if {$session($sid,bbox_changed) == 0 && $qflag == 0 && $expand_flag \
	    && $tmpfn != "NONE" && $force_redo == "false"} {
		# don't need to re-issue the query 
		set q "select * from $tmpfn"
		return $q
	}

	if {$debugMode && $expand_flag} {
		puts "Re-issueing query because: bbox_changed = \
			$session($sid,bbox_changed)\t qflag = $qflag"
	}

	set q [lindex $layer $LAYER_SQLQUERY]
	if {$expand_flag == 0} { return $q }
	append q [lindex $layer $LAYER_MAPSUFIX]
	# replace the current bbox macro ##PDMAP## with actual
	# map bounding box
	set sqlBox [makePDBBox  $session($sid,bx1) $session($sid,by1) \
		$session($sid,bx2) $session($sid,by2)]
	regsub -all ##PDMAP## $q $sqlBox query

    set level [lindex $layer $LAYER_LEVEL]
    if { ($level != "ALL") && ($level != "all") && ($level < 1000) } {
	set q [lindex $layer $LAYER_LEVELSUFIX]
	if {[regexp {where} "$query"]} {
		regsub {where} $q {and} q
	}
	append query $q
	set query [setDispLevel $query $sid $level]
    }
	return $query
}

proc setDispLevel {query sid level} {
    global debugMode, session

    if {$level=="NONE" || $level=="ALL" || $level=="all"} {
	if { [regexp {=##HEIGHT##} "$query"] 
	     || [regexp {=##WIDTH##} "$query"] } {	
		PDunsetLevel $query
	}
    }
	
    if { [regexp {=##HEIGHT##} "$query"] || [regexp {=##WIDTH##} "$query"] } {

    set boxwidth [expr abs($session($sid,bx2)-$session($sid,bx1))]
    set boxheight [expr abs($session($sid,by2)-$session($sid,by1))]
    set boxarea [expr $boxwidth*$boxheight]
    set hlevel [expr $boxheight*$level]
    set wlevel [expr $boxwidth*$level]

    regsub -all {=##HEIGHT##}  "$query" "*$level>=$boxheight" query
    regsub -all {=##WIDTH##} "$query" "*$level>=$boxwidth" query
    }

    return $query
}

# ------------------------------------------------------------
# <getLayerWidth> return the line width info of a give layer
#
#

proc getLayerWidth layer {
	global LAYER_WIDTH
	return [lindex $layer $LAYER_WIDTH]
}

proc setLayerWidth {layer new_width} {
	global LAYER_WIDTH
	return [lreplace $layer $LAYER_WIDTH $LAYER_WIDTH $new_width]
}

# ------------------------------------------------------------
# <getLayerColor> return the color of a layer.  this value
# may be overriden by geo_color attribute.
#
proc getLayerColor layer {
	global LAYER_COLOR
	return [lindex $layer $LAYER_COLOR]
}

proc setLayerColor {layer new_color} {
	global LAYER_COLOR
	return [lreplace $layer $LAYER_COLOR $LAYER_COLOR $new_color]
}

# ------------------------------------------------------------
# <getLayerBitmap> get the bitmap of given layer.  Valid for
# layer of point data.
#
#
proc getLayerBitmap layer {
	global  LAYER_BITMAP
	return [lindex $layer $LAYER_BITMAP]
}

proc setLayerBitmap {layer new_bmap} {
	global  LAYER_BITMAP
	return [lreplace $layer $LAYER_BITMAP $LAYER_BITMAP $new_bmap]
}

proc getLayerDb {layer sid} {
	global LAYER_CURDB
	return [lindex $layer $LAYER_CURDB]
}

proc getLayerExtName layer {
	global LAYER_CUREXTNAME
	return [lindex $layer $LAYER_CUREXTNAME]
}

proc getLayerTrans layer {
	global LAYER_TRANS
	return [lindex $layer $LAYER_TRANS]
}

proc setLayerTrans {layer new_trans_flag} {
	global LAYER_TRANS
	return [lreplace $layer $LAYER_TRANS $LAYER_TRANS $new_trans_flag]
}

proc getLayerTmpfn layer {
	global TMPFN
	return [lindex $layer $TMPFN]
}

proc getLayerQflag layer {
	global QFLAG
	return [lindex $layer $QFLAG]
}

proc getLayerId layer {
	global LAYER_ID
	return [lindex $layer $LAYER_ID]
}

proc getLayerLabel layer {
	global LBL_INDXS
	return [lindex $layer $LBL_INDXS]
}

proc setLayerLabel {layer new_label} {
	global LBL_INDXS
	return [lreplace $layer $LBL_INDXS $LBL_INDXS $new_label]
}

proc getLayerQueryDesc layer {
	global LAYER_DESC;
	return [lindex $layer $LAYER_DESC];
}

# ------------------------------------------------------------
# make a layer list element giving all of its values.
# Note: dispAttrType is stored as typeid (int), not ascii name.
#
# Fields are:
set LAYER_LAYERNAME 	0
set LAYER_COLOR 	1
set LAYER_BITMAP	2
set LAYER_WIDTH		3
set LAYER_SQLQUERY	4
set LAYER_MAPSUFIX	5
set LAYER_LEVELSUFIX	6
set LAYER_CUREXTNAME	7
set LAYER_CURDB		8
set LAYER_TRANS		9
set TMPFN		10
# query_changed flag
set QFLAG		11
set LAYER_ID		12
set LBL_INDXS		13
set LAYER_DESC		14
set LAYER_BASETABLE	15
set LAYER_BASEQUERY	16
set LAYER_UPDATE	17
set LAYER_LEVEL		18
set BOX_CHANGE		19

proc makeLayer {sid layerName color bitmap width SQLquery mapSufix levelSufix\
		curExtName curdb is_trans tmpfn qflag lblindxs desc \
		baseTable baseQuery update level {lid -1} {box_change 0}} {
	global session

	set l {}
	lappend l $layerName
	lappend l $color
	lappend l $bitmap
	lappend l $width
	lappend l $SQLquery
	lappend l $mapSufix
	lappend l $levelSufix
	lappend l $curExtName
	lappend l $curdb
	lappend l $is_trans
	lappend l $tmpfn
	lappend l $qflag
	if {$lid == -1} {
		lappend l $session($sid,layer_cnt)
		incr session($sid,layer_cnt)
	} else {
		# for editing old layer, lid persists
		lappend l $lid
	}
	lappend l $lblindxs;
	lappend l $desc;
	lappend l $baseTable;
	lappend l $baseQuery;
	if { $update==1 } {
	    lappend l "update"
	} else {
	    lappend l "no_update"
	}
	lappend l $level;
	lappend l $box_change
	return $l
}

#----------------------------------------------------------------------
# setLayerTmpfn: Given layer id, set its Tmpfn name
#----------------------------------------------------------------------
proc setLayerTmpfn {sid lid new_tmpfn} {
	global TMPFN session
	global debugMode

	if $debugMode {
		puts "In setLayerTmpFn $sid $lid $new_tmpfn"
	}

	set found 0
	set index 0

	foreach layer $session($sid,layers) {
		if {[getLayerId $layer] == $lid} {
			set found 1
			break;
		}
		incr index
	}

	if {$found == 0} {
		# search the layer in hidden layers also
		set index 0
		foreach layer $session($sid,hidden_layers) {
			if {[getLayerId $layer] == $lid} {
				set found 2
				break;
			}
			incr index
		}
	}

	if {$found == 0} {
		# error
		puts "Invalid layer id $lid supplied"
		exit
	}

	# now $layer is layer we are looking for and index is its
	# index in layers or hidden_layers
	set layer [lreplace $layer $TMPFN $TMPFN $new_tmpfn]
	if {$found == 1} {
		set session($sid,layers) [lreplace $session($sid,layers) \
			$index $index $layer]
	} else {
		set session($sid,hidden_layers) [lreplace \
			$session($sid,hidden_layers) \
			$index $index $layer]
	}
	if $debugMode {
		puts "\tnew tmpfn = [getLayerTmpfn $layer]"
	}
}

#----------------------------------------------------------------------
# setVisibleLayerById: Given layer id, update its layer data structure
# Side Effect: the global session value are updated.
#----------------------------------------------------------------------
proc setVisibleLayerById {sid lid new_layer} {
	global session
	global debugMode

	if $debugMode {
		puts "In setVisibleLayerById $sid $lid $new_layer"
	}

	set found 0
	set index 0

	foreach layer $session($sid,layers) {
		if {[getLayerId $layer] == $lid} {
			set found 1
			break;
		}
		incr index
	}

	if {$found == 0} {
		# error
		puts "Fatal: Invalid layer id $lid supplied";
                puts "\tin setVisibleLayerById";
	}

	# now $layer is layer we are looking for and index is its
	# index in layers or hidden_layers

	if {$found == 1} {
		set session($sid,layers) [lreplace $session($sid,layers) \
			$index $index $new_layer]
	}

}

#----------------------------------------------------------------------
# getVisibleLayerById
#----------------------------------------------------------------------
proc getVisibleLayerById {sid lid} {
	global session

	set found 0
	foreach layer $session($sid,layers) {
		if {[getLayerId $layer] == $lid} {
			set found 1
			break;
		}
	}
	if {$found == 0} {
		# error
		puts "Fatal: Invalid layer id $lid supplied"
                puts "\tOccurs in getVisibleLayerById"
		exit
	}

	# now $layer is layer we are looking for
	return $layer 
}

#----------------------------------------------------------------------
# getLayerTmpfnById
#----------------------------------------------------------------------
proc getLayerTmpfnById {sid lid} {
	global TMPFN session

	set found 0
	foreach layer $session($sid,layers) {
		if {[getLayerId $layer] == $lid} {
			set found 1
			break;
		}
	}
	if {$found == 0} {
		# error
		puts "Invalid layer id $lid supplied"
		exit
	}

	# now $layer is layer we are looking for
	return [lindex $layer $TMPFN]
}

#----------------------------------------------------------------------
# getLayerDbById
#----------------------------------------------------------------------
proc getLayerDbById {sid lid} {
	global LAYER_CURDB  session

	set found 0
	foreach layer $session($sid,layers) {
		if {[getLayerId $layer] == $lid} {
			set found 1
			break;
		}
	}
	if {$found == 0} {
		# error
		puts "Invalid layer id $lid supplied"
		exit
	}

	# now $layer is layer we are looking for
	return [lindex $layer $LAYER_CURDB]
}

#----------------------------------------------------------------------
# resetLayerQflag: Given layer id, reset its qflag
# 	this is done after the each run of a query
#----------------------------------------------------------------------
proc resetLayerQflag {sid lid} {
	global QFLAG session debugMode

	set found 0
	set index 0

	foreach layer $session($sid,layers) {
		if {[getLayerId $layer] == $lid} {
			set found 1
			break;
		}
		incr index
	}

	if {$found == 0} {
		# search the layer in hidden layers also
		set index 0
		foreach layer $session($sid,hidden_layers) {
			if {[getLayerId $layer] == $lid} {
				set found 2
				break;
			}
			incr index
		}
	}

	if {$found == 0} {
		# error
		puts "Invalid layer id $lid supplied"
		exit
	}

	# now $layer is layer we are looking for and index is its
	# index in layers or hidden_layers
	set layer [lreplace $layer $QFLAG $QFLAG 0]
	if {$found == 1} {
		set session($sid,layers) [lreplace $session($sid,layers) \
			$index $index $layer]
	} else {
		set session($sid,hidden_layers) [lreplace \
			$session($sid,hidden_layers) \
			$index $index $layer]
	}
	if $debugMode { puts "resetting Qflag for <$layer>" }
}

