# <std-header style=TCL orig-src=PTK>
#
#            Copyright 1998, NCR Corporation.  All Rights Reserved.
#
#                                NCR Confidential.
#
#
#  $Id: layerdialog.tcl,v 1.26 1998/07/23 04:21:56 kristint Exp $
#
#  This file was originally part of ptk, a front end based on TCL/TK to the
#  Paradise database system developed by the University of Wisconsin-Madison.
#  The above copyright applies to changes made to this file by NCR Corporation.
#  The original copyright which may appear in the source below applies to the
#  original file as obtained by NCR Corporation and unchanged parts of this file.
#
#
#   -- do not edit anything above this line --   </std-header>

# Implements the layer dialog to manage all layers in a top level
# window.
#
#

# hidden_layers contains all the layers not to be displayed
# combination of layers and hide_layers will be complete.
global hidden_layers;

# ----------------------------------------------------------------------
# layer_dialog
# ============
#
# Argument:
# + pw:		parent window name, needed since we are going to make
#		pw and w in a group so that iconify pw will automactically.
# + sid:	session id (session name), with this to get the appropriate
#		data structures.
# + canv:       map window 
# + w:		layer window name, default to .ld, this is ok since we
#		are going to have only on layer dialog exist at one time.
#		For the same reason, we can make life easier by making the
#		session copy of layer variables global.  Then we don't have
#		to pass them around :-)
#
set layerCnt 0
set hLayerCnt 0
set showSelect 0
set hideSelect 0
set localLayers {}
set localHiddenLayers {}
set localLayerNames {}
set localHiddenLayerNames {}
set layerdlg_btns_state(null) 0

proc layer_dialog {pw sid canv {w .ld}} {
	post_msg $sid "Managing layers...";
	global session dlg_active ptk_hint
	upvar #0 session s
	global layerCnt hLayerCnt showSelect hideSelect
	upvar #0 localLayers layers
	upvar #0 localHiddenLayers hLayers
	upvar #0 localLayerNames lNames
	upvar #0 localHiddenLayerNames hlNames

	set dlg_active(layerdlg) 1;
	set dlg_active(editdlg) 0;

	set layers [getLayers $sid]
	set hLayers [getHiddenLayers $sid]
	set layerCnt [llength $layers]
	set hLayerCnt [llength $hLayers]
	set showSelect 0
	set hideSelect 0
	set lNames [getLayerNames $layers]
	set hlNames [getLayerNames $hLayers]

	catch {destroy $w};
	toplevel $w;
	wm title $w "Layer Manager";
	wm group $w $pw;
	set wmposx [expr 400 + ($sid * 10) % 250]
	set wmposy [expr 70 + ($sid * 10) % 250]
	wm geometry $w "+$wmposx+$wmposy"

	ptkPDWait ON;

	# there is a panel containing the four buttons: Up, Down, First, Last
	frame $w.lm -bd 1m -relief groove
	button $w.lm.up -text "Up" -command "layer_up $w $sid" \
		-state disabled
	button $w.lm.down -text "Down" -command "layer_down $w $sid" \
		-state disabled
	button $w.lm.first -text "Make Top" -command "layer_first $w $sid" \
		-state disabled
	button $w.lm.last -text "Make Bottom" -command "layer_last $w $sid" \
		-state disabled
	pack $w.lm.up $w.lm.down $w.lm.first $w.lm.last -side top \
		-padx 1m -pady 1m -fill x

	# left side frame contains the listbox for shown layers
	frame $w.l;
	label $w.l.lab -text "Show"
	scrollbar $w.l.show_scrl -command "$w.l.show yview";
	listbox $w.l.show -yscroll "$w.l.show_scrl set" \
		-relief groove 	-geometry 16x10 -bg white -bd 1m -setgrid 1
	pack append $w.l $w.l.lab {top pady 2m} \
		$w.l.show_scrl {right filly} \
		$w.l.show {left expand fill} 

	# middle frame contains two buttons
	frame $w.m;
	button $w.m.hide -text "===>" -command "moveLayer $sid $w.l.show \
		$w.r.hide $w hide" -state disabled;
	button $w.m.show -text "<===" -command "moveLayer $sid $w.r.hide \
		$w.l.show $w show" -state disabled;
	pack append $w.m $w.m.hide {top pady 6m padx 6m} \
		$w.m.show {bottom pady 6m padx 6m};

	# right side frame contains the listbox for hidden layers
	frame $w.r;
	label $w.r.lab -text "Not Show"
	scrollbar $w.r.hide_scrl -command "$w.r.hide yview";	
	listbox $w.r.hide -yscroll "$w.r.hide_scrl set" \
		-bd 1m -relief groove -geometry 16x10 -bg white -setgrid 1
	pack append $w.r $w.r.lab {top pady 2m} \
		$w.r.hide_scrl {right filly} \
		$w.r.hide {left expand fill};

	# bottom frame contains vital commands...
	frame $w.b -bd 1m -relief groove
	frame $w.b.b1
	frame $w.b.b2
	pack $w.b.b1 $w.b.b2 -side top -fill x
	button $w.b.b2.ok -text "Ok" -command "destroy $w";
	button $w.b.b2.help -text "Help" -command \
		{DoHelp HelpSec6}
	button $w.b.b1.edit -text "Edit Layer Query" \
		-command "ptkLdEditItem $w $sid $canv" \
		-state disabled;
	button $w.b.b1.disp -text "Edit Layer Display" \
		-command "ptkLdEditItem $w $sid $canv 0" \
		-state disabled;
	button $w.b.b1.new -text "Create New Layer" \
		-command "ptkLdNewItem $w $sid $canv";
	button $w.b.b1.del -text "Delete Layer" -command "ptkLdDeleteItem $w \
		$sid" -state disabled;
	button $w.b.b1.refresh -text "Refresh" \
		-command "PDrefresh_layer $w $sid $canv" -state disabled;
		#-command "PDrefresh_map $sid $canv" -state disabled;
#############################################################################
        button $w.b.b1.draw -text "Update Layer" -relief raised \
	        -command "set dlg_active(editdlg) 1; \
			  updateLayerBtnState; \
			  layer_edit $w $sid $canv; \
			  $w.b.b1.new config -state normal; \
			  $w.b.b2.ok config -state normal; \
			  set dlg_active(editdlg) 0; \
			  updateLayerBtnState;"
#############################################################################
 
	pack $w.b.b1.new $w.b.b1.disp $w.b.b1.edit $w.b.b1.del \
		$w.b.b1.refresh $w.b.b1.draw -side left \
		-expand 1 -fill x -padx 1m -pady 1m;
	pack $w.b.b2.ok $w.b.b2.help -side left -expand 1 \
		-fill x -padx 1m -pady 1m

	# put four frames in upper portion together
	pack $w.lm $w.l $w.m $w.r -side left -expand 1 -pady 1m \
		-fill x -padx 2m;
	pack $w.b -before $w.lm -side bottom -anchor s -expand 1 \
		-fill x -pady 3m -padx 1c;

	# local bindings, make the listboxes single select
	tk_listboxSingleSelect $w.l.show $w.r.hide

	# 2. Make a copy of layers and hidden layers for this session
	# and put the contents into listboxes

	eval [concat layerDisp $w.l.show $lNames]
	eval [concat layerDisp $w.r.hide $hlNames]

	# 3. Modify the select binding for listbox
	bind $w.l.show <1> {%W select from [%W nearest %y]; \
		updateLayerBtnState}
	bind $w.r.hide <1> {%W select from [%W nearest %y]; \
		updateLayerBtnState}

	ptkPDWait OFF

	# 4. list of the buttons to be diabled when query composer is active
	set btns [list $w.lm.up $w.lm.down $w.lm.first $w.lm.last $w.m.hide \
		$w.m.show  $w.b.b1.new $w.b.b1.del $w.b.b1.edit $w.b.b1.disp \
		$w.b.b1.refresh $w.b.b2.ok];
	trace variable dlg_active w "layerdlg_set_btn_state \"$btns\"";

	# 5. set hints
	if $ptk_hint {
	    tixAddBalloon $w.lm.up {Move the selected layer up one level \
		    on map}
	    tixAddBalloon $w.lm.down {Move the selected layer down one level\
		    on map}
	    tixAddBalloon $w.lm.first {Move the selected layer to the top\
		    layer on map}
	    tixAddBalloon $w.lm.last {Move the selected layer to the bottom\
		    layer on map}
	    tixAddBalloon $w.l.show {All visible layers are here, listed in\
		    the order of bottom layer to top layer.  Click on the name\
		    of the layer to select a layer.  Selected layer will be\
		    highlighted.}
	    tixAddBalloon $w.r.hide {All hidden layers are here.  \
		    Click on the name of the layer to select a layer.  \
		    Selected layer will be highlighted.}
	    tixAddBalloon $w.m.hide {Click to hide the selected visible layer}
	    tixAddBalloon $w.m.show {Click to view the selected hidden layer}
	    tixAddBalloon $w.b.b2.ok {Click after you are done with layer\
		    management}
	    tixAddBalloon $w.b.b1.edit {Edit the SQL query of this layer}
	    tixAddBalloon $w.b.b1.disp {Edit the display attribute of this\
		    layer}
	    tixAddBalloon $w.b.b1.new {Create a new layer}
	    tixAddBalloon $w.b.b1.del {Delete the selected layer}
	    tixAddBalloon $w.b.b1.refresh {Refresh selected layer}
	}
	
	# 6. wait until complete
	tkwait window $w
	# trace vdelete dlg_active w "layerdlg_set_btn_state \"$btns\"";
	set dlg_active(layerdlg) 0;
	# Balloon seems to have a bug: the balloon sticks around
	# if the window it attaches to are deleted
	catch {tixBalPopdown}
}

proc layerdlg_set_btn_state {btns name1 name2 op} {
	global $name1 layerdlg_btns_state;
	global debugMode;

	if $debugMode {
		puts "tracing $name1 $name2";
	}
	if {$name2 != "qcomposer" || $name2 != "querycfg"} {return}
	if {$name2 == "querycfg" && $dlg_active(qcomposer)} {
		# qcomposer has already save the old button states
		# another save will overwrite those old values
		return;
	}

	set newdlg_active [set [format "%s(%s)" $name1 $name2]];

	if $newdlg_active {
		# record current state and disable them
		foreach btn $btns {
			set layerdlg_btns_state($btn) \
				[getLastListElem [$btn config -state]];
			$btn config -state disabled;
		}
	} else {
		# restore the old state
		foreach btn $btns {
			$btn config -state $layerdlg_btns_state($btn);
		}
	}
}

proc show_new_layers {sid} {
	post_msg $sid "Done with layer management"
	update idletasks;
}

proc layerDisp {win args} {
	# 1. clear the display box
	$win delete 0 end
	foreach i $args {
		$win insert end $i
	}
}

proc updateLayerBtnState {{w .ld}} {
	global layerCnt hLayerCnt showSelect hideSelect
	global dlg_active

	set showSelect [llength [$w.l.show curselection]]
	set hideSelect [llength [$w.r.hide curselection]]

	# showBtns will be enabled whenever a show layer is selected
	# hideBtns will be enabled whenever a hide layer is selected
	set showBtns "$w.lm.up $w.lm.down $w.lm.first $w.lm.last \
		$w.m.hide $w.b.b1.edit $w.b.b1.del $w.b.b1.refresh $w.b.b1.disp"
	set hideBtns {$w.m.show}

	# just disable all showBtns - since refresh is now
	# a showBtn, no need to check layerCnt
	eval [concat setBtnState disabled $showBtns]

	#if {$layerCnt == 0} {
	#eval [concat setBtnState disabled $showBtns $w.b.b1.refresh]
	#} else {
	#$w.b.b1.refresh configure -state normal 
	#}
	if {$dlg_active(editdlg) == 1 || $showSelect == 0} {
		eval [concat setBtnState disabled $showBtns]
	} else {
		eval [concat setBtnState normal $showBtns]
	}
	if {$hideSelect == 0} {
		eval [concat setBtnState disabled $hideBtns]
	} else {
		eval [concat setBtnState normal $hideBtns]
	}
}

# move a layer selected in srcWin to destWin; should check that srcWin
# owns the selection; otherwise update button states and issue warning
proc moveLayer {sid srcWin destWin w hint} {
	upvar #0 localLayers l
	upvar #0 localHiddenLayers h
	upvar #0 localLayerNames ln
	upvar #0 localHiddenLayerNames hln
	global layerCnt hLayerCnt session ptk_map_canv
	global TMPFN BOX_CHANGE

	set selectOwner [selection own]
	if {$selectOwner != $srcWin} {
		tk_dialog .layerWarning {Warning} {Please select a layer \
			in source window first!} warning 0 OK
		updateLayerBtnState $w
		return
	}

	# move the selected item
	set itemIndx [$srcWin curselection]
	if {$hint == "hide"} {
		# remove the layer from map
		set lid [getLayerId [lindex $session($sid,layers) $itemIndx]]
		set lid "l$lid";
		$ptk_map_canv delete $lid;

		listMoveItem session($sid,layers) session($sid,hidden_layers) \
			$itemIndx
		set l [getLayers $sid]
		set ln [getLayerNames $l]
		set h [getHiddenLayers $sid]
		set hln [getLayerNames $h]
	} else {
		set show_layer [lindex $session($sid,hidden_layers) $itemIndx]
		set box_change [lindex $show_layer $BOX_CHANGE]
		if { $box_change == 1 } {
		    set show_layer [lreplace $show_layer $TMPFN $TMPFN NONE]
		}
		listMoveItem session($sid,hidden_layers) session($sid,layers) \
			$itemIndx
		set l [getLayers $sid]
		set ln [getLayerNames $l]
		set h [getHiddenLayers $sid]
		set hln [getLayerNames $h]

		# reveal the layer on screen
		showOneLayer $ptk_map_canv $show_layer $sid "false";
	}

	# extract the name of new top layer
	set session($sid,topqname) [getLastListElem $ln];

	set layerCnt [llength $l]
	set hLayerCnt [llength $h]

	# refresh the two listboxes
	eval [concat layerDisp $w.l.show $ln]
	eval [concat layerDisp $w.r.hide $hln]

	# clear the selection and update button states
	selection clear $srcWin
	updateLayerBtnState $w
}

# layer_up: move a selected layer in show box up, the show box path is
# $w.l.show
proc layer_up {w sid} {
	upvar #0 localLayers l
	upvar #0 localLayerNames ln
	global session ptk_map_canv

	set itemIndx [$w.l.show curselection]
	if {$itemIndx == ""} {
		tk_dialog .layer_up_warning {Warning} {Please select \
			a layer in show box} warning 0 OK
		updateLayerBtnState $w
	}
	set len [llength $l]
	if {$itemIndx==[expr $len - 1]} { return }
	set newIndx [expr $itemIndx + 1]
	set srclid [getLayerId [lindex $session($sid,layers) $itemIndx]];
	set destlid [getLayerId [lindex $session($sid,layers) $newIndx]];
	set srclid "l$srclid";
	set destlid "l$destlid";
	listSwapItems session($sid,layers) $itemIndx $newIndx 

	# reload the layer name list and counter
	set l [getLayers $sid]
	set ln [getLayerNames $l]
	eval [concat layerDisp $w.l.show $ln]

	# extract the name of new top layer
	set session($sid,topqname) [getLastListElem $ln];

	# clear the selection and update button states
	selection clear $w.l.show
	updateLayerBtnState $w

	# swap two layers on screen
	if { $newIndx == [expr $len - 1] } {
	    $ptk_map_canv dtag toplayer
	    $ptk_map_canv addtag toplayer withtag $srclid
	}
	$ptk_map_canv raise $srclid $destlid
}

proc layer_down {w sid} {
	upvar #0 localLayers l
	upvar #0 localLayerNames ln
	global session ptk_map_canv

	set len [llength $l]
	set itemIndx [$w.l.show curselection]
	if {$itemIndx == ""} {
		tk_dialog .layer_up_warning {Warning} {Please select \
			a layer in show box} warning 0 OK
		updateLayerBtnState $w
	}
	if $itemIndx==0 { return }
	set newIndx [expr $itemIndx - 1]
	set srclid [getLayerId [lindex $session($sid,layers) $itemIndx]];
	set destlid [getLayerId [lindex $session($sid,layers) $newIndx]];
	set srclid "l$srclid";
	set destlid "l$destlid";
	listSwapItems session($sid,layers) $newIndx $itemIndx  

	# reload the layer name list and counter
	set l [getLayers $sid]
	set ln [getLayerNames $l]
	eval [concat layerDisp $w.l.show $ln]

	# extract the name of new top layer
	set session($sid,topqname) [getLastListElem $ln];

	# clear the selection and update button states
	selection clear $w.l.show
	updateLayerBtnState $w

	# swap two layers on screen
	set toplid [getLayerId [lindex $session($sid,layers) [expr $len - 1]]];
	set toplid "l$toplid"
	if { $itemIndx == [expr $len - 1] } {
	    $ptk_map_canv dtag toplayer
	    $ptk_map_canv addtag toplayer withtag $toplid
	}
	$ptk_map_canv lower $srclid $destlid
}

proc layer_first {w sid} {
	upvar #0 localLayers l
	upvar #0 localLayerNames ln
	global session ptk_map_canv

	set itemIndx [$w.l.show curselection]
	if {$itemIndx == ""} {
		tk_dialog .layer_up_warning {Warning} {Please select \
			a layer in show box} warning 0 OK
		updateLayerBtnState $w
	}
	set len [llength $l]
	if {$itemIndx==[expr $len - 1]} { return }
	set newIndx [expr $len - 1]
	set lid [getLayerId [lindex $session($sid,layers) $itemIndx]];
	set lid "l$lid";
	moveListItem session($sid,layers) $itemIndx $newIndx

	# reload the layer name list and counter
	set l [getLayers $sid]
	set ln [getLayerNames $l]
	eval [concat layerDisp $w.l.show $ln]

	# extract the name of new top layer
	set session($sid,topqname) [getLastListElem $ln];

	# clear the selection and update button states
	selection clear $w.l.show
	updateLayerBtnState $w

	# raise the layer on map
	$ptk_map_canv dtag toplayer
	$ptk_map_canv addtag toplayer withtag $lid
	$ptk_map_canv raise $lid;
}

proc layer_last {w sid} {
	upvar #0 localLayers l
	upvar #0 localLayerNames ln
	global session ptk_map_canv

	set len [llength $l]
	set itemIndx [$w.l.show curselection]
	if {$itemIndx == ""} {
		tk_dialog .layer_up_warning {Warning} {Please select \
			a layer in show box} warning 0 OK
		updateLayerBtnState $w
	}
	if $itemIndx==0 { return }
	set lid [getLayerId [lindex $session($sid,layers) $itemIndx]];
	set lid "l$lid";
	moveListItem session($sid,layers) $itemIndx 0

	# reload the layer name list and counter
	set l [getLayers $sid]
	set ln [getLayerNames $l]
	eval [concat layerDisp $w.l.show $ln]

	# extract the name of new top layer
	set session($sid,topqname) [getLastListElem $ln];

	# clear the selection and update button states
	selection clear $w.l.show
	updateLayerBtnState $w

	# lower the layer on map
	if { $itemIndx == [expr $len-1] } {
	    set toplayer [lindex $session($sid,layers) [expr $len-1]]];
	    set seclayer [lindex $session($sid,layers) [expr $len-2]]];
	    set toplid [getLayerId $toplayer]
	    set toplid "l$toplid";
	    $ptk_map_canv dtag toplayer
	    $ptk_map_canv addtag toplayer withtag $toplid
	}
	$ptk_map_canv lower $lid;
}

#
# delete the currently selected item in $w.l.show for session $sid
#
proc ptkLdDeleteItem { w sid } {
	global session ptk_map_canv
	upvar #0 localLayers l
	upvar #0 localLayerNames ln
	
	set itemIndx [$w.l.show curselection]
	if {$itemIndx == ""} {
		tk_dialog .layer_up_warning {Warning} {Please select \
			a layer in show box} warning 0 OK
		updateLayerBtnState $w
	}
	set del_lid [getLayerId [lindex $session($sid,layers) $itemIndx]];
	set del_lid "l$del_lid"
	set session($sid,layers) [lreplace $session($sid,layers) \
		$itemIndx $itemIndx]

	# reload the layer name list and counter
	set l [getLayers $sid]
	set ln [getLayerNames $l]

	# extract the name of new top layer
	set session($sid,topqname) [getLastListElem $ln];

	# clear the selection and update button states
	selection clear $w.l.show
	eval [concat layerDisp $w.l.show $ln]
	updateLayerBtnState $w

	# remove the layer from screen
	$ptk_map_canv delete $del_lid;
}

#
# add a new layer
#
proc ptkLdNewItem {w sid canv} {
	global session 
	upvar #0 localLayers l
	upvar #0 localLayerNames ln
	global layerCnt 

	# Since we are creating new layers, the current layer
	# for query composer will be null
	# call qselect, don't return until done
	set session($sid,curlayer) {};
	lappend session($sid,layers) [qcomposer $sid]
  
	# now layer list contains a new layer, we want to
	# 1. 	check to see whether the new layer is empty or not.
	# 	If it is empty, Cancel button is invoked in browser,
	#	we should remove this empty layer
	set l [getLayers $sid]
	set layerCnt [llength $l]
	set lastLayer [lindex $l [expr $layerCnt - 1]]
	if {$lastLayer == {}} {
		# remove this empty layer (generated by cancel button in
		# browser)
		set session($sid,layers) [lreplace $session($sid,layers) \
			[expr $layerCnt - 1] [expr $layerCnt - 1]]
		# reload the layer name list and counter
		set l [getLayers $sid]
		set layerCnt [expr $layerCnt - 1]
		assert {$layerCnt == [llength $l]}
		return
	}

	# if we are here, we know that a new layer has indeed been
	# created, we have to update the view in layer list box
	set ln [getLayerNames $l]

	eval [concat layerDisp $w.l.show $ln]
	updateLayerBtnState $w

	# Then, we should show the new layer on mapview, using
	# the canvas provided by the caller (i.e. the mapview).
	ptkPDWait ON
	ptkPDStatBox ON
	ptkPDStatBox::setMsg "Loading [getLayerName $lastLayer]..."

	# show new layer, if center is invoked, we have to
	# delete all previous layers
	if {$session($sid,center_tmpfn) == "NONE"} {
		showOneLayer $canv $lastLayer $sid "false"
	} else {
		$canv delete all
		showOneLayer $canv $lastLayer $sid "false"
		set session($sid,bbox_changed) 0
	}

	# set top query name
	set session($sid,topqname) [getLayerName $lastLayer]
	ptkPDStatBox OFF
	ptkPDWait OFF
}	

#----------------------------------------------------------------------
# Edit the selected layer; The successful completion of editing
# will automatically invoke refresh command
# If showQuery flag is set to 1, the query composer will bring
# up the text editor for editing sql query, otherwise, this step
# will be bypassed and the query configuration panel will be directly
# invoked.
#----------------------------------------------------------------------
proc ptkLdEditItem {w sid canv {showQuery 1}} {
	global session ptk_map_canv
 	upvar #0 localLayers l
	upvar #0 localLayerNames ln
	global layerCnt debugMode

	# find the selected layer
	set itemIndx [$w.l.show curselection]
	set topIndx [expr [llength $l] - 1]
	if {$itemIndx == ""} {
		tk_dialog .layer_up_warning {Warning} {Please select \
			a layer in show box} warning 0 OK
		updateLayerBtnState $w
	}
	# directly use session layers to avoid the trouble
	# caused by upvar 
	set session($sid,curlayer) [lindex $session($sid,layers) $itemIndx]

	# invoke browser with the above contextual information
	set new_layer [qcomposer $sid $showQuery 1]
	set changed 0
	if $session($sid,bbox_changed) {
		set changed 1;
	}
	if {$new_layer != $session($sid,curlayer) && $changed == 0 && \
			$itemIndx == $topIndx} {
		set changed 2;
	}
	if { [lindex $session($sid,layers) $itemIndx] == {} } {
		pd_dialog .warning {Warning} {Layer is not active} {} \
				warning 0 {OK};
		return;
	}

	updateLayerBtnState $w
	set session($sid,layers) [lreplace $session($sid,layers) $itemIndx \
		$itemIndx $new_layer]

	# reload the layer name list and counter
	set l [getLayers $sid]
	set ln [getLayerNames $l]
	eval [concat layerDisp $w.l.show $ln]

	# do something according to changs
	if {$changed == 1} {
		# need refresh
		if $debugMode {
			puts "Box changed, refresh all"
		}
		PDrefresh_map $sid $canv 0
	}
	if {$changed == 2} {
		# only need to refresh the top layer
		set $session($sid,center_tmpfn) NONE
		if $debugMode {
			puts "top layer query changed, refresh it"
		}
		set tag_id "l[getLayerId $session($sid,curlayer)]"
		if $debugMode {
			puts "\ttop layer id = $tag_id"
		}
		$ptk_map_canv delete $tag_id
		showOneLayer $canv $new_layer $sid "false"
	}
	if $debugMode {
		puts "Leaving ptkLdEditItem"
	}
}		

proc PDrefresh_layer {w sid map} {
    global session ptk_map_canv
    upvar #0 localLayers l

    # find the selected layer
    set itemIndx [$w.l.show curselection]
    set topIndx [expr [llength $l] - 1]
    if {$itemIndx == ""} {
	tk_dialog .layer_up_warning {Warning} {Please select \
		a layer in show box} warning 0 OK
	updateLayerBtnState $w
    }
    # directly use session layers to avoid the trouble
    # caused by upvar 
    set session($sid,curlayer) [lindex $session($sid,layers) $itemIndx]

    set tag_id "l[getLayerId $session($sid,curlayer)]"
    #if $debugMode {
    #	puts "\ttop layer id = $tag_id"
    #}

    # delete this layer
    $ptk_map_canv delete $tag_id
   
    ptkPDWait ON
    ptkPDStatBox ON
    ptkPDStatBox::setMsg "Refreshing [getLayerName $session($sid,curlayer)]..."
    showOneLayer $map $session($sid,curlayer) $sid "true"
    ptkPDStatBox OFF
    ptkPDWait OFF
}

#----------------------------------------------------------------------
# refresh the map upon hitting the refresh button,
# we don't call PDvalidate_map because we don't
# want to call ptk_bbox::set twice.  But code
# has to be partly duplicated here
#----------------------------------------------------------------------
proc PDrefresh_map {sid map {force 0}} {
	global session

	# clear map
	$map delete all;

	# if force refresh, as is the case when a old session is
	# loaded, ptk_bbox::set will have bbox_changed == 0
	# and query will not be run.  This is not desirable.
	if $force { set session($sid,bbox_changed) 1}

	# reload each layer
	showAllLayers $map $sid

	# reset bbox_changed flag
	set session($sid,bbox_changed) 0
    	set session($sid,newelm_added) 0

	# reload grids and labels
	setGridInfo $sid
	mapShowGrid $map $sid
	mapShowGridLabel $map $sid

	post_msg $sid "Refreshing done!";
}

