# <std-header style=TCL orig-src=PTK>
#
#            Copyright 1998, NCR Corporation.  All Rights Reserved.
#
#                                NCR Confidential.
#
#
#  $Id: mapview.tcl,v 1.50 1998/05/19 03:22:26 kupsch Exp $
#
#  This file was originally part of ptk, a front end based on TCL/TK to the
#  Paradise database system developed by the University of Wisconsin-Madison.
#  The above copyright applies to changes made to this file by NCR Corporation.
#  The original copyright which may appear in the source below applies to the
#  original file as obtained by NCR Corporation and unchanged parts of this file.
#
#
#   -- do not edit anything above this line --   </std-header>

# mapview: create a mapview widget with the name $map.
#
# Argument:
#	-map	map window path
#	-sid    session id
#	-newSessionFlag		Is this a map for a new session?
#				Default is yes (1).

#
#
# The code below creates a main window, consisting of a
# menu bar, a canvas to view the map, and a few buttons
# for further actions
#

# save the tag of the selected item in the following
# variable
set mapview_tcl(0) ""
set itemtag ""
set restore_cmd ""

# save the position of clicking B1 here
set mapview_item_sel(x) -1
set mapview_item_sel(y) -1

# used to save the button states
set mapview_btns_state(null) 0;

proc mapview {map sid {newSessionFlag 1}} {  
    #	proc mapview
    global globalVarList session dlg_active
    global debugMode
    global ptk_map_canv ptk_hint bitmap_dir
    upvar #0 session s

    set ptk_map_canv "$map.frame1.map"
    catch "destroy $map"
    toplevel $map
    set wmposx [expr 0 + ($sid * 10) % 250]
    set wmposy [expr 60 + ($sid * 10) % 250]
    set wm_width [expr $s($sid,view_w) + 80]
    set wm_height [expr $s($sid,view_h) + 120]
    set geo_opt [format "%dx%d+%d+%d" $wm_width $wm_height $wmposx $wmposy] 
    wm geometry $map $geo_opt
    wm minsize $map $s($sid,view_w) $s($sid,view_h)
    wm iconbitmap $map "$bitmap_dir/earth2.xbm"
    wm iconmask $map "$bitmap_dir/earth2.xbm.mask"
    wm iconname $map "Paradise"
    
    # We probably only need the tail of a file name
    # on window title
    wm title $map "PARADISE MAP <$session($sid,sname)>"
    ptkPDWait ON

    # fill up an array of dividing points for grids, must at least have
    setGridInfo $sid

    frame $map.menubar -relief raised -borderwidth 2;
    frame $map.toolbar -bd 3
    frame $map.frame1 -bd 0.8m -relief groove;
    canvas $map.frame1.map  -width $s($sid,view_w) -height $s($sid,view_h) \
	    -background $s($sid,bg_color) -cursor crosshair
    frame $map.frame1.status -bd 2 -relief groove;
    pack $map.menubar $map.toolbar -side top -fill x
    pack $map.frame1 -side top
    pack $map.frame1.map $map.frame1.status -side top \
	    -anchor w -padx 1m -pady 1m
    button $map.toolbar.setq -text "Layer Mgr" -relief raised \
	    -command "layer_dialog $map $sid $map.frame1.map" 
    frame $map.toolbar.coord -width 60c -relief groove -bd 2
    frame $map.toolbar.topq -width 32c -relief groove -bd 2
    message $map.toolbar.coord.lat -width 18c -anchor w \
	    -textvariable session($sid,laeay)
    message $map.toolbar.coord.long -width 18c -anchor w \
	    -textvariable session($sid,laeax)
    message $map.toolbar.topq.topqmsg -width 16c -text "Top Layer:" \
	    -font "-*-helvetica-bold-r-normal-*-14-*-*-*-*-*-*-*"
    message $map.toolbar.topq.topqname -width 16c -textvariable \
	    session($sid,topqname)
    pack $map.toolbar.coord.long $map.toolbar.coord.lat -side left -padx 1m
    pack $map.toolbar.topq.topqmsg $map.toolbar.topq.topqname -side left \
	    -padx 1m
    pack $map.toolbar.setq -side left -padx 1m
    pack $map.toolbar.topq $map.toolbar.coord -side right -padx 1m

    # put some message inside $map.frame1.status
    message $map.frame1.status.msg0 -width 8c -anchor w -text "Status: " \
	    -font "-*-helvetica-bold-r-normal-*-14-*-*-*-*-*-*-*" \
	    -justify left;
    message $map.frame1.status.msg -width 26c -anchor w \
	    -textvariable session($sid,ptk_stat) \
	    -justify left;
    pack $map.frame1.status.msg0 -side left;
    pack $map.frame1.status.msg -side left -fill x;

    # fill in the menus on menubar
    menubutton $map.menubar.file -text "File" -menu $map.menubar.file.m \
	    -underline 0
    menu $map.menubar.file.m
    $map.menubar.file.m add command -label "New" -underline 0 \
	    -command "mapviewNewSession $sid" -accelerator "  Ctrl-n"
    $map.menubar.file.m add command -label "Open" -underline 0 \
	    -command "mapviewOpenSession $sid" -accelerator "  Ctrl-o"
    $map.menubar.file.m add separator
    $map.menubar.file.m add command -label "Save & Quit" \
	    -underline 0 -accelerator "  Ctrl-s" -command "ptkQuit $sid"
    $map.menubar.file.m add command -label "Quit" -underline 0 \
	    -command {ptkBailout} -accelerator "  Ctrl-q"

    menubutton $map.menubar.cmd -text "View" -menu $map.menubar.cmd.m \
	    -underline 0
    menu $map.menubar.cmd.m
    $map.menubar.cmd.m add command -label "Refresh"  \
	    -command "PDrefresh_map $sid $map.frame1.map";
    $map.menubar.cmd.m add cascade -label "Zoom Out" \
	    -menu $map.menubar.cmd.m.zo
    $map.menubar.cmd.m add command -label "Raise Labels" -command \
	    "$map.frame1.map raise lb"
    $map.menubar.cmd.m add separator
    $map.menubar.cmd.m add checkbutton  \
	    -label "Show Grids" -variable session($sid,show_grid)\
	    -command "mapShowGrid $map.frame1.map $sid"
    $map.menubar.cmd.m add checkbutton \
	    -label "Label Grids" -variable \
	    session($sid,show_label) \
	    -command "mapShowGridLabel $map.frame1.map $sid"

    $map.menubar.cmd.m add separator
#    $map.menubar.cmd.m add command -label "Set Display" \
#	    -command "resetDispLevel $sid $map.frame1.map"

    menu $map.menubar.cmd.m.zo
    $map.menubar.cmd.m.zo add command -label "X 2" \
	    -command "mapZoomOut $sid $map.frame1.map 2"
    $map.menubar.cmd.m.zo add command -label "X 4" \
	    -command "mapZoomOut $sid $map.frame1.map 4"
    $map.menubar.cmd.m.zo add command -label "X 8" \
	    -command "mapZoomOut $sid $map.frame1.map 8"
    $map.menubar.cmd.m.zo add command -label "X 32" \
            -command "mapZoomOut $sid $map.frame1.map 32"
    $map.menubar.cmd.m.zo add command -label "X 64" \
            -command "mapZoomOut $sid $map.frame1.map 64"
    $map.menubar.cmd.m.zo add command -label "X 128" \
            -command "mapZoomOut $sid $map.frame1.map 128"

    menubutton $map.menubar.help -text "Help" -menu $map.menubar.help.m \
	    -underline 0
    menu $map.menubar.help.m
    $map.menubar.help.m add command -label "Contents" \
	    -command { DoHelp HelpSec5 }
    $map.menubar.help.m add command -label "Current BBox" -state normal\
	    -command "map_showbbox $sid"

    menubutton $map.menubar.hint -bitmap "$bitmap_dir/ballon_question.xbm"\
	    -menu $map.menubar.hint.m
    menu $map.menubar.hint.m
    $map.menubar.hint.m add checkbutton \
	    -label "Show Balloon Hint" -variable ptk_hint \
	    -command {ptkHintToggle}
    
    pack $map.menubar.file $map.menubar.cmd -side left
    pack $map.menubar.help $map.menubar.hint -side right -padx 1m

    # set keyboard focus, otherwish the keypress will discarded by Tk
    focus $map.frame1.map

    bind $map.frame1.map <Any-Motion> "setnewcrd %x %y $sid"

    # rubber banding to get the new bounding box::zooming in
    bind $map.frame1.map <ButtonPress-2> \
	    "$map.frame1.map create rectangle %x %y %x %y -tags band"
    bind $map.frame1.map <B2-Motion> "
    set cors \[$map.frame1.map coords band\]
    set x0 \[lindex \$cors 0\]
    set y0 \[lindex \$cors 1\]
    set xdiff \[expr %x - \$x0\]
    set ydiff \[expr %y - \$y0\]
    if {\$xdiff > \$ydiff} {
	set diff \$xdiff
    } else { set diff \$ydiff}
    set cor \[lreplace \[$map.frame1.map coords band\] 2 3 \
	    \[expr \$x0 + \$diff\] \[expr \$y0 + \$diff\] \];
    eval \"$map.frame1.map coords band \$cor\""
    bind $map.frame1.map <ButtonRelease-2> "
    set raw_bbox \[$map.frame1.map coords band\]
    $map.frame1.map delete band
    PDvalidate_map $sid  $map.frame1.map \$raw_bbox"

    # random selection using button 1
    bind $map.frame1.map <ButtonPress-1> "
    $map.frame1.map addtag cur_select withtag current
    itemSelect $map.frame1.map $sid \
	    \[$map.frame1.map itemconfigure current -tags\] %x %y"
    bind $map.frame1.map <ButtonRelease-1> "
    itemSelect2 $map.frame1.map $sid  \
	    \[$map.frame1.map itemconfigure current -tags\] %x %y
    $map.frame1.map dtag cur_select"

    # new center by B3
    bind $map.frame1.map <3> "newCenter \$sid $map.frame1.map %x %y"

    #
    # set map to default focus window, so we don't lose focus
    # later and thus key binding will always work
    #
    focus default $map.frame1.map

    # add hint bindings
    tixAddBalloon $map.toolbar.setq {Add/edit layers on map, short-cut\
	key is "q"}
    tixAddBalloon $map.menubar.file {Create a new session, Open prievious\
	    session, Save current session or quit}
    tixAddBalloon $map.menubar.cmd {Adjust the view of map}
    tixAddBalloon $map.menubar.help {Detailed help}
    tixAddBalloon $map.menubar.hint {Toggle balloon hint}
    if {$ptk_hint == 0} {
	tixDisableBalloon
    }
    
    #
    # File menu accelerator binding
    #
    bind $map.frame1.map <Control-Key-q> { ptkBailout }
    bind $map.frame1.map  <Control-Key-s> { ptkQuit $sid}
    bind $map.frame1.map <Control-Key-n> "mapviewNewSession $sid"
    bind $map.frame1.map <Control-Key-o> "mapviewOpenSession $sid"
    bind $map.frame1.map <Key-q> "$map.toolbar.setq invoke"

    # list of the buttons to be diabled when layer dialog is active
    set btns [list $map.menubar.file $map.menubar.cmd $map.toolbar.setq]
    trace variable dlg_active w "mapview_set_btn_state \"$btns\"";

    # if this is for a previously saved session, bring up the map
    # but first, we have to make sure the right database is open
    if {$newSessionFlag == 0} {
	# set force flag to force refresh of map view
	PDrefresh_map $sid $map.frame1.map 1
    }
    ptkPDWait OFF

}

proc mapview_set_btn_state {btns name1 name2 op} {
    global $name1 mapview_btns_state;
    global debugMode;

    if $debugMode {
	puts "tracing $name1 $name2";
    }
    if {$name2 != "layerdlg"} {return}
    set layerdlg_active [format "$%s(layerdlg)" $name1];

    if $layerdlg_active {
	# record current state and disable them
	foreach btn $btns {
	    set mapview_btns_state($btn) \
		    [getLastListElem [$btn config -state]];
	    $btn config -state disabled;
	}
    } else {
	# restore the old state
	foreach btn $btns {
	    $btn config -state $mapview_btns_state($btn);
	}
    }
}

#
# toggle hint mode
#
proc ptkHintToggle {} {
    global ptk_hint
    if {$ptk_hint == 0} {
	# turn off hint mode
	tixDisableBalloon
    } {
	# turn on hint mode
	tixEnableBalloon
    }
}



# ====================================================================

#
# procedure to update the coordinates when pointer moves in
# mapview
#
proc setnewcrd {nx ny sid} {
    #reference the global variable
    global session
    
    set session($sid,laeax) [format "X: %10s" [x2laeax $nx $sid]]
    set session($sid,laeay) [format "Y: %10s" [y2laeay $ny $sid]]
}

#=========================================================================
# new center
#=========================================================================
proc newCenter {sid w nx ny} {
    global session
    set nlon [x2laeax $nx $sid]
    set nlat [y2laeay $ny $sid]
    set xdiff [expr $session($sid,bx2) - $session($sid,bx1)]
    set ydiff [expr $session($sid,by2) - $session($sid,by1)]
    set nx1 [expr $nlon - $xdiff / 2]
    set nx2 [expr $nlon + $xdiff / 2]
    set ny1 [expr $nlat - $ydiff / 2]
    set ny2 [expr $nlat + $ydiff /2]

    ptk_bbox::set $sid $nx1 $ny1 $nx2 $ny2
    PDrefresh_map $sid $w
}

#======================================================================
# map zoom out
#======================================================================
proc mapZoomOut {sid w fac} {
    global session
    global debugMode

    # fac is the factor of increase in area
    # convert it to increase in side
    set sfac [expr sqrt($fac)]

    set xdiff  [expr $session($sid,bx2) - $session($sid,bx1)]
    set ydiff [expr $session($sid,by2) - $session($sid,by1)]
    set nx1 [expr int($session($sid,bx1) - $sfac * $xdiff / 2)]
    set nx2 [expr int($session($sid,bx2) + $sfac * $xdiff / 2)]
    set ny1 [expr int($session($sid,by1) - $sfac * $ydiff / 2)]
    set ny2 [expr int($session($sid,by2) + $sfac * $ydiff / 2)]

    # Dealing with zooming distortion.
    #if {$ny2 < 0} {set ny2 0}
    #if {$ny1 > 90} {set ny1 90}
    #if {$nx1 < -180.0} { set nx1 [expr $nx1 + 360] }
    #if {$nx2 > 180} {set nx2 [expr 360 - $nx2]}
    ptk_bbox::set $sid $nx1 $ny1 $nx2 $ny2
    if {$debugMode == 1} {
	puts "ZoomOut: x1 = $session($sid,bx1), ny1 = $session($sid,by1), nx2 = $session($sid,bx2), ny2 = $session($sid,by2)"
    }

    PDrefresh_map $sid $w
}


proc x2laeax {x sid} {
    global session debugMode
    return [PDScrLaea x2laeax $x $session($sid,bx1) \
	    $session($sid,x_scale_fac)]
}

proc y2laeay {y sid} {
    upvar #0 session s
    return [PDScrLaea y2laeay $y $s($sid,by1) $s($sid,y_scale_fac)]
}

proc laeax2x {laeax sid} {
    global session
    return [PDScrLaea laeax2x $laeax $session($sid,bx1) \
	    $session($sid,x_scale_fac)]
}

proc laeay2y {laeay sid} {
    global session
    return [PDScrLaea laeay2y $laeay $session($sid,by1) \
	    $session($sid,y_scale_fac)]
}

proc mapShowGrid {c sid } {
    upvar #0 session s
    global debugMode

    if {$debugMode == 1} { puts [format "show_grid = %s" $s($sid,show_grid)]}
    if {$s($sid,show_grid) != 0} {
	foreach i $s($sid,xGridLine) {
	    $c create line $i 0 $i $s($sid,view_h) -tag GridLine
	}

	foreach i $s($sid,yGridLine) {
	    $c create line 0 $i $s($sid,view_w) $i -tag GridLine
	}
    } else {
	catch "$c delete GridLine"
    }
}

proc mapShowGridLabel {c sid} {
    upvar #0 session s

    if {$s($sid,show_label) !=0} {
	setGridInfo $sid
	for {set i 0} {$i < $s($sid,grid_x)} {incr i 1} {
	    set lbl [lindex $s($sid,xGridLineLabel) $i]
	    set x [lindex $s($sid,xGridLine) $i]
	    $c create text $x 0 -anchor ne -text $lbl -tag GridLabel
	    $c create text $x $s($sid,view_h) -anchor se -text $lbl \
		    -tag GridLabel
	}

	for {set i 0} {$i < $s($sid,grid_y)} {incr i 1} {
	    set lbl [lindex $s($sid,yGridLineLabel) $i]
	    set y [lindex $s($sid,yGridLine) $i]
	    $c create text 0 $y -anchor nw -text $lbl -tag GridLabel
	    $c create text $s($sid,view_w) $y -anchor ne -text $lbl \
		    -tag GridLabel
	}
    } else { catch "$c delete GridLabel" }
}

# close the current session and open a new session
proc mapviewNewSession {sid} {
    if [ptkCloseSession $sid] {
	# don't forget to delete dialog tracing
	delete_trace 
	ptkNewSession
    }
}

proc mapviewOpenSession {sid} {
    if [ptkCloseSession $sid] {
	# open session only when user decides to
	# close previous session
	# don't forget to delete dialog tracing
	delete_trace
	ptkOpenSession
    }
}

proc delete_trace {} {
    global dlg_active
    set ti [trace vinfo dlg_active]
    foreach t $ti {
	trace vdelete dlg_active [lindex $t 0] [lindex $t 1]
    }
}
#----------------------------------------------------------------------
# proc to close the current session, if current session has been
# changed, ask user to save it.
# It is robust, means if a session has already been closed, it will
# return immediately.
# 
# Arguments:
#	-snum		session number, note that sid is used as global
#			in this routine
#
# Return:
#	return 1 if a session is closed, 0 otherwise.
#
#----------------------------------------------------------------------
proc ptkCloseSession {snum} {
    global session 
    global debugMode
    global close_ret_val
    set close_ret_val 1
    
    if {$debugMode == 1} {
	puts "ptkCloseSession: closing session $snum \
		<[file tail $session($snum,sname)]>"
    }

    # prompt to save the changes to a file
    # always do "save as" to save troubles
    # this way, the ptk_session_dir is obsolete
    toplevel .close
    wm title .close "Close Session"

    set x 440
    set y 350 
    wm geometry .close +$x+$y

    frame .close.top -relief raised -border 1
    label .close.top.icon -bitmap questhead
    message .close.top.msg -relief sunken -bd 1 -anchor n -padx 20 \
	    -width 300 -text "Save current session?" \
	    -font "-adobe-helvetica-bold-r-normal-*-14-*-*-*-*-*-*-*"
    frame .close.btns
    button .close.btns.yes -text "Yes, Save" -width 10 \
	    -command "map_save_dlg $snum"
    button .close.btns.no -text "No Save" -width 10 \
	    -command "map_nosave_cmd $snum"
    button .close.btns.cancel -text "Cancel" -width 10 \
	    -command "set close_ret_val 0; destroy .close"

    pack .close.top.icon -side left -expand yes -fill both -padx 10 \
	    -pady 10
    pack .close.top.msg -expand yes -fill both -padx 10 -pady 10 \
	    -side right
    pack .close.top -expand yes -fill both 
    pack .close.btns -fill x
    pack .close.btns.yes .close.btns.no .close.btns.cancel -side left \
	    -padx 10 -pady 10
    
    tkwait window .close
    return $close_ret_val
}

proc map_save_dlg {snum} {
    global session globalVarList PTK_VERSION sid
    global debugMode map_save_snum

    if $debugMode {
	puts "calling map_save_dlg"
    }
    
    # now the guy really wants to close the current session
    # let him/her choose a file
    set map_save_snum $snum
    if {![winfo exists .save_file]} {
	tixMkFileDialog .save_file map_save_cmd
	wm title .save_file "Save Session File"
    } else {
	wm deiconify .save_file
    }
}

proc map_save_cmd {sfn} {
    global session globalVarList PTK_VERSION sid
    global debugMode map_save_snum

    if $debugMode {
	puts "map_save_cmd $sfn"
    }
    
    set snum $map_save_snum
    
    # drop all the temporaries owned by this session
    # note that some temporaries are actual table
    # name, we don't drop those.
    foreach layer $session($snum,layers) {
	if $debugMode {
	    puts "dropping tables owned by session $snum"
	}
	set tmpfn [getLayerTmpfn $layer]
	if [is_fname_tmp $tmpfn] {
	    # PDsql_cursorclose
	    PDsql_cursorrun "drop table $tmpfn"
	}
	# set all tmpfn to NONE, so that next time we
	# don't drop non-existent tables
	set lid [getLayerId $layer]
	setLayerTmpfn $snum $lid "NONE"
    }
    foreach layer $session($snum,hidden_layers) {
	if $debugMode {
	    puts "dropping tables owned by session $snum"
	}
	set tmpfn [getLayerTmpfn $layer]
	if [is_fname_tmp $tmpfn] {
	    #PDsql_cursorclose
	    PDsql_cursorrun "drop table $tmpfn"
	}
	# set all tmpfn to NONE, so that next time we
	# don't drop non-existent tables
	set lid [getLayerId $layer]
	setLayerTmpfn $snum $lid "NONE"
    }

    # drop the tmpfile used for centering purpose
    # note that this can lead to problem once you
    # have data from multiple database, since the
    # db information is not carried along
    if [is_fname_tmp $session($snum,center_tmpfn)] {
	# PDsql_cursorclose
	PDsql_cursorrun "drop table $center_tmpfn"
	set session($snum,center_tmpfn) "NONE"
    }

    set fd [open $sfn w]
    puts $fd "ptk$PTK_VERSION"
    foreach i $globalVarList {
	puts $fd $session($snum,$i)
	puts $fd "***===***===***";
    }
    close $fd

    # don't close db, can lead to core dump if not careful
    # but have to do it anyway, need better support from
    # server
    PDsql_cursorrun "close db $session($snum,curdb)"
    destroy .close
}

proc map_nosave_cmd {snum} {
    global session globalVarList PTK_VERSION sid
    global debugMode 

    # drop all the temporaries owned by this session
    # note that some temporaries are actual table
    # name, we don't drop those.
    foreach layer $session($snum,layers) {
	if $debugMode {
	    puts "dropping tables owned by session $snum"
	}
	set tmpfn [getLayerTmpfn $layer]
	if [is_fname_tmp $tmpfn] {
	    # PDsql_cursorclose
	    PDsql_cursorrun "drop table $tmpfn"
	}
	# set all tmpfn to NONE, so that next time we
	# don't drop non-existent tables
	set lid [getLayerId $layer]
	setLayerTmpfn $snum $lid "NONE"
    }
    foreach layer $session($snum,hidden_layers) {
	if $debugMode {
	    puts "dropping tables owned by session $snum"
	}
	set tmpfn [getLayerTmpfn $layer]
	if [is_fname_tmp $tmpfn] {
	    #PDsql_cursorclose
	    PDsql_cursorrun "drop table $tmpfn"
	}
	# set all tmpfn to NONE, so that next time we
	# don't drop non-existent tables
	set lid [getLayerId $layer]
	setLayerTmpfn $snum $lid "NONE"
    }

    # drop the tmpfile used for centering purpose
    # note that this can lead to problem once you
    # have data from multiple database, since the
    # db information is not carried along
    if [is_fname_tmp $session($snum,center_tmpfn)] {
	# PDsql_cursorclose
	PDsql_cursorrun "drop table $center_tmpfn"
	set session($snum,center_tmpfn) "NONE"
    }

    # don't close db, can lead to core dump if not careful
    # but have to do it anyway, need better support from
    # server
    PDsql_cursorrun "close db $session($snum,curdb)"
    destroy .close
}

#----------------------------------------------------------------------
# quit the application totally,
# We used to close all active sesssions, but now we can
# only have one active session, so that is irrelevant now.
#----------------------------------------------------------------------
proc ptkQuit {sid} {
    global session globalVarList LAYER_CUREXTNAME
    global debugMode ptk_root

    if {[ptkCloseSession $sid] == 0} {
	# session not closed, can't quit
	return
    }

    set layers [getLayers $sid]
    set hLayers [getHiddenLayers $sid]

    foreach layer $layers {
	set extName [lindex $layer $LAYER_CUREXTNAME]
	if [regexp "__PTmpF_" $extName] {
	    PDsql_cursorrun "drop table $extName"
 	}
    }

    foreach layer $hLayers {
	set extName [lindex $layer $LAYER_CUREXTNAME]
	if [regexp "__PTmpF_" $extName] {
	    PDsql_cursorrun "drop table $extName"
 	}
    }
    # note that we don't need to close any db here, because each
    # session will close its own open db, thus we should be clean
    # at this point.
    ptk_playad $ptk_root/sounds/Goodbye.au
    exit
}

#----------------------------------------------------------------------
# quit the application totally, means we close all the currently
# active sesssions
#----------------------------------------------------------------------
proc ptkBailout {} {
    global session globalVarList sid LAYER_CUREXTNAME
    global debugMode ptk_root

    foreach layer $session($sid,layers) {
	if $debugMode {
	    puts "dropping tables owned by session $snum"
	}
	set tmpfn [getLayerTmpfn $layer]
	if [is_fname_tmp $tmpfn] {
	    # PDsql_cursorclose
	    PDsql_cursorrun "drop table $tmpfn"
	}
	set lid [getLayerId $layer]
	setLayerTmpfn $sid $lid "NONE"
    }

    foreach layer $session($sid,hidden_layers) {
	if $debugMode {
	    puts "dropping tables owned by session $snum"
	}
	set tmpfn [getLayerTmpfn $layer]
	if [is_fname_tmp $tmpfn] {
	    #PDsql_cursorclose
	    PDsql_cursorrun "drop table $tmpfn"
	}
	set lid [getLayerId $layer]
	setLayerTmpfn $sid $lid "NONE"
    }

    # session will close its own open db, thus we should be clean
    # at this point.
    PDsql_cursorrun "close db $session($sid,curdb)";
    ptk_playad $ptk_root/sounds/Goodbye.au
    exit
}

#
# PDvalidate_map: re-validate the map after bounding box is changed
# 	It takes a list consisting of the coordinates of the raw
#	bounding box as the input.
#	It invokes showAllLyers to actually showing each layer.
#	It also invokes the gird line and label routine to update
#	grids and labels.
#
proc PDvalidate_map {sid {map .frame1.map} raw_bbox {force 0}} {
    #	post_msg $sid "Reload each layer...";
    update idletasks;

    global session view_w view_h;
    global debugMode

    # layers is an array of list of <type, color, filenames, name> pair
    # indexed by layer order (integer) to be loaded 
    global layers;

    # scale factors used when converting screen coordinates to laea
    global x_scale_fac y_scale_fac;

    set rx1 [lindex $raw_bbox 0];
    set ry1 [lindex $raw_bbox 1];
    set rx2 [lindex $raw_bbox 2];
    set ry2 [lindex $raw_bbox 3];

    # check for erraneous click of B2
    if {$rx1 == $rx2 && $ry1 == $ry2} {
	post_msg $sid "Please use B1 to pick, B2 to sketch\n and B3 to center"
	return
    }

    set bx1 $session($sid,bx1)
    set bx2 $session($sid,bx2)
    set by1 $session($sid,by1)
    set by2 $session($sid,by2)

    # convert the new raw bounding box to laea coordinates
    # remember to cast scale factor to double and cast bounding
    # box to integer
    set x_scale_fac [expr (double($bx2 - $bx1))/$view_w];
    set y_scale_fac [expr (double($by2 - $by1))/$view_h];
    set nbx1 [expr int($rx1 * $x_scale_fac + $bx1)];
    set nby1 [expr int($ry1 * $y_scale_fac + $by1)];
    set nbx2 [expr int($rx2 * $x_scale_fac + $bx1)];
    set nby2 [expr int($ry2 * $y_scale_fac + $by1)];

    ptk_bbox::set $sid $nbx1 $nby1 $nbx2 $nby2

    # clear map
    $map delete all;

    # if force refresh, as is the case when a old session is
    # loaded, ptk_bbox::set will have bbox_changed == 0
    # and query will not be run.  This is not desirable.
    if $force { set session($sid,bbox_changed) 1}

    # reload each layer
    showAllLayers $map $sid

    # reset bbox_changed flag
    set session($sid,bbox_changed) 0

    # reload grids and labels
    setGridInfo $sid
    mapShowGrid $map $sid
    mapShowGridLabel $map $sid

    post_msg $sid "Zooming done!";
}

proc itemSelect {c sid tags x y} {
    global bg_color mapview_tcl debugMode
    global mapview_item_sel
    
    # get the type before current is gone
    # can't depend on oid tag, it may refer to more
    # than one objects of different types and only
    # first one is returned by $c type
    set type [$c type cur_select]

    set mapview_item_sel(x) $x
    set mapview_item_sel(y) $y

    if {$debugMode == 1} {
	puts "itemSelect $c $sid $tags $x $y"
    }
    set mapview_tcl($sid,itemtag) {}
    foreach tag [concat $tags] {
	if {[string index [lindex $tag 1] 0] == "o"} {
	    # this is the one that contains the oid
	    set mapview_tcl($sid,itemtag) [lindex $tag 1]
	    break;
	}
    }
    # 	set mapview_tcl($sid,restore_cmd) "";;
    # 	if {$type != "" && $type != "bitmap" && $type != "raster" \;
    # 			&& $type != "text"} {;
    # 		post_msg $sid "Found a $type with oid $mapview_tcl($sid,itemtag)!";;
    # 		set fill [lindex [$c itemconfig $mapview_tcl($sid,itemtag) \;
    # 			-fill] 4];
    # 		set mapview_tcl($sid,restore_cmd) \;
    # 			"$c itemconfig cur_select -fill $fill";;
    # 		$c itemconfig cur_select -fill $bg_color;;
    # 	} else {;
    # 		if {$debugMode == 1} {;
    # 			post_msg $sid "$mapview_tcl($sid,itemtag)";
    # 		} ;
    # 	};
}

# this is invoked when button 1 is released, we consider an item selected
# only when release also happens on top of the item being pressed
proc itemSelect2 {c sid tags x y} {
    global mapview_tcl debugMode LAYER_BASEQUERY LAYER_UPDATE
    global mapview_item_sel session

    set layer_id "";
    set oid "";
    if $debugMode {		
	puts "in itemSelect2 $c $sid $tags $x $y" }
    # 	if {$mapview_tcl($sid,restore_cmd) != ""} {;
    # 		eval $mapview_tcl($sid,restore_cmd);
    # 		set mapview_tcl($sid,restore_cmd) "";
    # 	};
    if {($x != $mapview_item_sel(x)) || ($y != $mapview_item_sel(y))} {
	# pointer moved
	post_msg $sid "Pointer moved!"
	return
    }
    foreach tag $tags {
	if {$tag == {}} {continue}
	if { [string index [lindex $tag 1] 0] == "o"} {
	    # this is the one that contains the oid
	    set oid [string trimleft [lindex $tag 1] o]
	}
	if [string match {l[0-9]*} [lindex $tag 0]] {
	    set layer_id [string range [lindex $tag 0] 1 end]
	    set tmpfn [getLayerTmpfnById $sid $layer_id]
	}
    }

    if {$layer_id == "" || $oid == ""} {
	post_msg $sid "No object found!"
	return
    }

    set l [getLayers $sid]
    set layerCnt [llength $l]
    set Layer [getVisibleLayerById $sid $layer_id]
    set baseQuery [lindex $Layer $LAYER_BASEQUERY]
    set updatable [lindex $Layer $LAYER_UPDATE]

    # now we are sure an object has been selected
    global ParserOidAttributeName # this added by Navin Kabra: 7/1/96
    set q "select * from $tmpfn where $ParserOidAttributeName = \"$oid\""
    if $debugMode {puts "\tRun query: $q"}
    ptkPDWait ON
    set rc [PDsql_cursoropen $q]
    if { $rc == "NO_TUPLE" } { return }

    set desc [PDsql_cursorschema]
    set tmpfn [getTmpfnFromDesc $desc]
    set tuple [PDsql_cursorfetch]

    ptk_tpview $sid $tuple $desc $tags $c $q $baseQuery $updatable

    ptkPDWait OFF
    if [regexp "__PTmpF_" $tmpfn] {
	PDsql_cursorrun "drop table $tmpfn"
    }
    if $debugMode {puts "Exit itemSelect2"}
}

proc ptkAspect {sid map} {
    global session
    global debugMode

    set nbox [aspect_bbox $session($sid,bx1) $session($sid,by1) \
	    $session($sid,bx2) $session($sid,by2)]
    ptk_bbox::set $sid [lindex $nbox 0] [lindex $nbox 1] \
	    [lindex $nbox 2]  [lindex $nbox 3]
    
    if $session($sid,bbox_changed) {
	if $debugMode { puts "re-freshing because aspect\
		changed current bounding box!" }
	PDrefresh_map $sid $map
    }
}

# if auto_unpost is set to 1, then message will be unposted after
# delay milliseconds;
proc post_msg {sid {msg ""} {auto_unpost 0} {delay 5000}} {
    global session
    set session($sid,ptk_stat) $msg;
    update;
    if {$auto_unpost == 1} {
	after $delay {set session($sid,ptk_stat) ""};
	update;
    }
}

proc map_showbbox sid {
    global session
    post_msg $sid "Current BBox = (upper left: (x $session($sid,bx1)) (y $session($sid,by1))\n lower right: (x $session($sid,bx2)) (y $session($sid,by2)))"
}

