# <std-header style=TCL orig-src=PTK>
#
#            Copyright 1998, NCR Corporation.  All Rights Reserved.
#
#                                NCR Confidential.
#
#
#  $Id: misc.tcl,v 1.38 1999/01/12 18:34:00 paradise Exp $
#
#  This file was originally part of ptk, a front end based on TCL/TK to the
#  Paradise database system developed by the University of Wisconsin-Madison.
#  The above copyright applies to changes made to this file by NCR Corporation.
#  The original copyright which may appear in the source below applies to the
#  original file as obtained by NCR Corporation and unchanged parts of this file.
#
#
#   -- do not edit anything above this line --   </std-header>

# setBtnState
#
# Argument:
#	+ stat:		new state to set to;
#	+ args:		list of buttons to be changed.
# 
proc setBtnState {stat args} {
	foreach i $args {
		$i configure -state $stat
	}
}

# assert: assert facility like in C
#
proc assert exp {
	set bool [uplevel 1 "expr $exp"]
	if {$bool == 0} {
		eval [list tk_dialog .assert_error {Assertion Failure} \
			"Assertion ($exp) failed!"  warning 0 OK]
		return
	}
}

# listMoveItem: move an item in source list to destination list
#
# Argument:
#	+ srcList:	source list; reference argument
#	+ destList:	destination list; reference argument
#	+ srcItemIndx:	src item index (the one to be taken out)
#	+ destItemIndx: destination index (inserted before this point)
#			Default to end (0 means the first element)
#
# Notes: if srcList is empty or srcItemIndx is out of range, issue an
# 	warning and return to caller.
#
# Side effect: srcList and destList will be thus changed.
#
proc listMoveItem {src_List dest_List srcItemIndx {destItemIndx end}} {
	upvar $src_List srcList
	upvar $dest_List destList
	global BOX_CHANGE

	set srcListLength [llength $srcList]
	assert {$srcListLength > $srcItemIndx}
	assert {$srcItemIndx >= 0}

	set item [lindex $srcList $srcItemIndx]
	set srcList [lreplace $srcList $srcItemIndx $srcItemIndx]
	if {$destItemIndx == "end"} {
		# linsert does not recognize "end", too bad
		set destItemIndx [llength $destList]
	}
	set item [lreplace $item $BOX_CHANGE $BOX_CHANGE 0]
	set destList [linsert $destList $destItemIndx $item]
}

proc moveListItem {list oldIndx newIndx} {
	upvar $list l
	set item [lindex $l $oldIndx]
	set l [lreplace $l $oldIndx $oldIndx]
	set l [linsert $l $newIndx $item]
}

proc listSwapItems {list indx1 indx2} {
	upvar $list l

	if {$indx1 == $indx2} {return}
	set item1 [lindex $l $indx1]
	set item2 [lindex $l $indx2]
	set l [lreplace $l $indx1 $indx1 $item2]
	set l [lreplace $l $indx2 $indx2 $item1]
}

proc getLastListElem l {
  if {$l == {}} { return {}};
  return [lindex $l [expr [llength $l] -1]];
}

#
# ptkSetPDtypes: load the ADT type names that we can use in SQL queries
# 	Also load the length of each type in terms of num_chars
proc ptkSetPDtypes {} {
	global PDtypes ptk_typewd

	set PDtypes {{Integer 0} {Real 1} {Boolean 2} {String 3} {Point 4} \
	{Rectangle 5} {ClosedPolygon 6} {PolyLine 7} \
	{Circle 8} {Raster8 9} \ 
	{Raster16 10} {Raster24 11} {ArrayUInt1 12} \
        {Gif 17} {Text 18} {Video 19} {Audio 20} {Blob 21} <Dicom3 22} {Video1 23} {Video2 24} {Clob 25} {Url 26}\
	{Oid 27} {Date 28} {FidAdt 29} {StoreAdt 30} {TimeAdt 31} \
        {HistAdt 32} {CountApx 33} {SwCheese 34} {Smallint 35} {TinyInt 36} {BigInt 37} {Decimal 38} {TimeStamp 39} }

	# For update and changes, see adtInfo.h
	set ptk_typewd(Integer) 20;
	set ptk_typewd(Smallint) 20;
        set ptk_typewd{TinyInt} 20;
        set ptk_typewd{BigInt} 20;
	set ptk_typewd(Real) 20;
	set ptk_typewd(Boolean) 20;
	set ptk_typewd(String) 36;
	set ptk_typewd(ClosedPolygon) 60;
	set ptk_typewd(SwCheese) 60;
	set ptk_typewd(PolyLine) 60;
	set ptk_typewd(Point) 20;
	set ptk_typewd(Rectangle) 20;
	set ptk_typewd(Raster) 60;
	set ptk_typewd(Circle) 60;
	set ptk_typewd(Video) 10;
	set ptk_typewd(Blob) 60;
	set ptk_typewd(Dicom3) 60;
	set ptk_typewd(Video1) 60;
	set ptk_typewd(Video2) 60;
	set ptk_typewd(Url) 36;
	set ptk_typewd(Raster8H) 60;
	set ptk_typewd(Raster8) 60;
	set ptk_typewd(Raster16) 60;
	set ptk_typewd(Raster24) 60;
	set ptk_typewd(ArrayUInt1) 60;
	set ptk_typewd(Oid) 36;
	set ptk_typewd(Date) 10;
	set ptk_typewd(Gif) 10;
	set ptk_typewd(Text) 10;
	set ptk_typewd(Audio) 10;
        set ptk_typewd(FidAdt) 40;
        set ptk_typewd(StoreAdt) 40;
        set ptk_typewd(HistAdt) 10;
}

#
# ptkTypename2id: convert a type string to its id, it takes a type string
# and return its corresponding typeid
#
proc ptkTypename2id {typeStr} {
	global PDtypes debugMode

	set typeNames {}
	set typeIds {}
	foreach pair $PDtypes {
		lappend typeNames [lindex $pair 0]
		lappend typeIds [lindex $pair 1]
	}
	set indx [lsearch $typeNames $typeStr]
	assert {$indx != -1}
	return [lindex $typeIds $indx]
}

#
# ptkTypeid2name converts a type id to its name (ASCII string)
# So that the type id returned by metadata can be tranformed to
# type strings displayable in menus.  For type update, see adtInfo.h
# It would be nice if this information is available from server,
# then we don't have to hard code this in.
proc ptkTypeid2name id {
	global PDtypes debugMode

	set typeNames {}
	set typeIds {}
	foreach pair $PDtypes {
		lappend typeNames [lindex $pair 0]
		lappend typeIds [lindex $pair 1]
	}
	set indx [lsearch $typeIds $id]
	assert {$indx != -1}
	return [lindex $typeNames $indx]
}

#
# set cursor according to the current state
#
proc ptkPDWait state {
	set topLevelWins [winfo children .]
	if {$state == "ON"} {
		set curs "watch"
	} { set curs {} }
	foreach w $topLevelWins {
		$w config -cursor $curs
	}
	update idletasks
}

proc parseQuery {sid q} {
	global session

	# replace the current bbox macro ##PDMAP## with actual
	# map bounding box
	set sqlBox [makePDBBox  $session($sid,bx1) $session($sid,by1) \
		$session($sid,bx2) $session($sid,by2)]
	regsub -all ##PDMAP## $q $sqlBox query
	return $query
}

proc aspect_bbox {bx1 by1 bx2 by2} {
	set x1 $bx1
	set x2 $bx2
	set y1 $by1
	set y2 $by2

	set xdiff [expr $bx2 - $bx1]
	set ydiff [expr $by2 - $by1]

	if {[expr abs($xdiff)] < [expr abs($ydiff)]} {
		if {$xdiff > 0} {
			set x2 [expr $x1 + abs($ydiff)]
		} { set x2 [expr $x1 - abs($ydiff)] }
	}
	if {[expr abs($xdiff)] > [expr abs($ydiff)]} {
		if {$ydiff > 0} {
			set y2 [expr $y1 + abs($xdiff)]
		} { set y2 [expr $y1 - abs($xdiff)] }
	}
	return [list $x1 $y1 $x2 $y2]
}

# Added by Navin Kabra: 7/1/96
# A variable that indicates the attribute name of the OID
# in a relation
# This variabe should be the same as the ParserOidAttributeName
# constant declared in paradise/server/include/catalog.h
# If that variable is changed, then this must be changed
set ParserOidAttributeName "__oid"

