# <std-header style=TCL orig-src=PTK>
#
#            Copyright 1998, NCR Corporation.  All Rights Reserved.
#
#                                NCR Confidential.
#
#
#  $Id: qcomposer.tcl,v 1.31 1998/07/23 04:21:57 kristint Exp $
#
#  This file was originally part of ptk, a front end based on TCL/TK to the
#  Paradise database system developed by the University of Wisconsin-Madison.
#  The above copyright applies to changes made to this file by NCR Corporation.
#  The original copyright which may appear in the source below applies to the
#  original file as obtained by NCR Corporation and unchanged parts of this file.
#
#
#   -- do not edit anything above this line --   </std-header>

#----------------------------------------------------------------------
# create a query composer at top level window
# need lib/general2.tcl for mkWidget procs
#
# If the current map bounding box is not meant to be
# used, user should type in a complete SQL query;
# Otherwise, the query should end with "overlaps ";
# the bounding box will be automatically attatched
# each time the query is invoked.
#----------------------------------------------------------------------
set qcomposer_return_val "NONE"
set qcomposer_old_val "NONE"
set qcomposer_prev_query ""
set qcomposer(font) "-*-helvetica-bold-r-normal-*-14-*-*-*-*-*-*-*"
set qcomposer(hf) "ptk_queryHistory"

proc qcomposer {sid {showQuery 1} {oldlayer 0}} {
    global qcomposer_return_val qcomposer qcomposer_old_val;
    global session debugMode dlg_active ptk_hint

    # set the initial query in session($sid,curquery)
    qcomposer_initQuery $sid;

    # save the old value for cancel purpose
    set qcomposer_old_val $session($sid,curlayer)

    # if showQuery is 0, we jump directly to query configuration
    if {$showQuery == 0} {
	qcomposer_customize $sid "$session($sid,curquery)" $showQuery \
			    $oldlayer;
	if {$debugMode == 1} {
	    puts "qcomposer return: $qcomposer_return_val";
	}
	return $qcomposer_return_val;
    }


    # go thru query compose window
    set dlg_active(qcomposer) 1;
    
    set w ".qcomposer"
    catch "destroy $w"
    toplevel $w
    wm title $w "Query Composer"
    set wmposx [expr 10 + ($sid * 10) % 250]
    set wmposy [expr 220 + ($sid * 10) % 250]
    wm geometry $w "+$wmposx+$wmposy"
    wm minsize $w 200 100

    # make a menu bar
    set menuBar [mkFrame $w.mbar {top expand fillx} -bd 2 -relief raised]
    set sqlCmdMenuBtn [ mkMenubutton $w.mbar.sql "\"SQL Command\"" \
	    $w.mbar.sql.m {left padx 10} -underline 0]
    set catMenuBtn [mkMenubutton $w.mbar.cat "\"Catalog Info\"" \
	    $w.mbar.cat.m {left padx 10} -underline 0]
    set autoQueryMenuBtn [mkMenubutton $w.mbar.auto "\"View Extent\"" \
	    $w.mbar.auto.m {left padx 10} -underline 0]
    set tranXMenuBtn [mkMenubutton $w.mbar.tranx "\"Transaction\"" \
	    $w.mbar.tranx.m {left padx 10} -underline 0]
    set helpMbtn [mkMenubutton $w.mbar.help "\"Help\"" \
	    $w.mbar.help.m {right padx 10} -underline 0]
    tk_menuBar {$w.mbar $sqlCmdMenuBtn $catMenuBtn $autoQueryMenuBtn \
	$tranXMenuBtn $helpMbtn};

    set toolbar [mkToolbar $sid $w]	
    set queryPane [frame $w.qpane]
    pack $queryPane -side top 
    set queryEditor [text $queryPane.qed \
	    -wrap word -width 80\
	    -height 20 -state normal\
	    -bd 2 -relief sunken\
	    -yscrollcommand "$queryPane.scroll set" \
	    -insertbackground white]
    scrollbar $queryPane.scroll -relief flat -command "$queryEditor yview" 
    qcomposerChangeFont $sid $queryEditor $toolbar
    pack $queryEditor -side left
    pack $queryPane.scroll -side left -fill y
    
    set bot [frame $w.bot]
    pack $bot -expand 1 -fill x
    mkButton $bot.ok "Show On Map" \
	    "qcomposerDone $sid $queryEditor $w $oldlayer" {left padx 1 \
	     pady 10 expand fillx};
    mkButton $bot.run "Show In Table" "qcomposerExec $sid $queryEditor" \
	    {left padx 1 pady 10 expand fillx};
    mkButton $bot.clear Clear "$queryEditor delete 0.0 end" \
	    {left padx 1 pady 10 expand fillx};
    mkButton $bot.addhis "Add To History" \
	    "qcomposerAddHist $queryEditor" {left padx 1 pady 10 \
	    expand fillx};
    mkButton $bot.his "View History" \
	    "qcomposerViewHist $qcomposer(hf) $bot.his" {left padx 1 pady 10 \
	    expand fillx};
    mkButton $bot.cancel Cancel "qcomposerCancel $sid $w" \
	    {left padx 1 pady 10 expand fillx};

    if [winfo exists ".history_view"] {
	$bot.his config -state disabled;
    }

    # if we have a preset query, we will just use it
    if {$session($sid,curquery) != "NONE"} {
	$queryEditor insert 0.0 $session($sid,curquery)
    }

    # make up the sql command menu
    menu $sqlCmdMenuBtn.m
    $sqlCmdMenuBtn.m  add command -label "select" \
	    -command "$queryEditor insert insert select" -state normal;  
    $sqlCmdMenuBtn.m  add command -label "where" \
	    -command "$queryEditor insert insert {where}" \
	    -state normal;  
    $sqlCmdMenuBtn.m add command -label "##PDMAP##" \
	    -command "$queryEditor insert insert {##PDMAP##}" \
	    -state normal;
    # make up the catalog menu
    menu $catMenuBtn.m
    qcomposerMkCatMenu $sid $catMenuBtn.m $queryEditor

    # make up the auto query menu
    menu $autoQueryMenuBtn.m
    qcomposerMkQueryMenu $sid $autoQueryMenuBtn.m $queryEditor

    menu $tranXMenuBtn.m
    $tranXMenuBtn.m  add command -label "Begin" \
	    -command "Xact \"begin transaction\"" -state normal;  
    $tranXMenuBtn.m  add command -label "Commit" \
	    -command "Xact \"commit transaction\"" \
	    -state normal;  
    $tranXMenuBtn.m add command -label "Abort" \
	    -command "Xact \"abort transaction\"" \
	    -state normal;

    # make up the help menu
    menu $helpMbtn.m
    $helpMbtn.m add command -label "Contents" -command {DoHelp HelpSec7}
    
    # don't return until done
    focus $queryEditor

    # set hint messages
    if $ptk_hint {
	tixAddBalloon $sqlCmdMenuBtn {Choose some SQL commands}
	tixAddBalloon $catMenuBtn {Get catalog information here}
	tixAddBalloon $autoQueryMenuBtn {Choose an extent and I will\
		compose a SQL query for you}
	tixAddBalloon $helpMbtn {Still in the dark?}
	tixAddBalloon $bot.ok {Click to display the query result\
		on map.  Only make sense if you have spatial attribute\
		in you result.  Otherwise, use "Show In Table" button.}
	tixAddBalloon $bot.run {Click to show your result in tabular\
		format.}
	tixAddBalloon $bot.clear {Click to clean the editor window}
	tixAddBalloon $bot.addhis {Click to add the current contents\
		of the query editor to history}
	tixAddBalloon $bot.his {Click to view the query history}
	tixAddBalloon $bot.cancel {Click to cancel query composition\
		and go back to layer manager}
    }
    tkwait window $w

    # get around the stray balloon
    catch {tixBalPopdown}
    
    set dlg_active(qcomposer) 0;

    if {$debugMode == 1} {
	puts "qcomposer return: $qcomposer_return_val"
    }
    return $qcomposer_return_val
}	

#----------------------------------------------------------------------
# create a tool bar
# Arguments:
#  + pw:     parent window path
#----------------------------------------------------------------------
proc mkToolbar {sid pw} {
    global session
    global LAYER_UPDATE

    set tbar $pw.tbar
    frame $tbar -relief flat
    pack $tbar -side top -fill both

    # tool bar font
    label $tbar.fontlabel -text "Display Font:" \
	    -font "-adobe-helvetica-bold-r-normal-*-12-*-*-*-*-*-*-*"
    tixCombobox $tbar.font -type static \
	    -height 5 -width 15 \
	    -command "qcomposerChangeFont $sid $pw.qpane.qed $tbar"
    tixCombobox $tbar.size -width 4 -height 5 \
	    -command "qcomposerChangeFont $sid $pw.qpane.qed $tbar"
    
    $tbar.font appendhistory "Courier"
    $tbar.font appendhistory "Helvetica"
    $tbar.font appendhistory "New Century Schoolbook"
    $tbar.font appendhistory "Times"
    $tbar.font appendhistory "Lucida"

    $tbar.size appendhistory 8
    $tbar.size appendhistory 10
    $tbar.size appendhistory 12
    $tbar.size appendhistory 14
    $tbar.size appendhistory 18

    $tbar.font pick $session($sid,qcomposer_font)
    $tbar.size pick $session($sid,qcomposer_size)
    
    set session($sid,update) 0
    set layer $session($sid,curlayer)
    if {[lindex $layer $LAYER_UPDATE]=="update"} {
	set session($sid,update) 1
    }
    set session($sid,old_update) $session($sid,update)
    checkbutton $tbar.update -text "For Update" -variable \
	session($sid,update) -anchor w 

    pack $tbar.fontlabel -side left -padx 3 -pady 4
    pack $tbar.font -side left -pady 4
    pack $tbar.size -side left -padx 4 -pady 4
    pack $tbar.update -side left -pady 4
    return $tbar
}

#----------------------------------------------------------------------
# save the result query in qcomposer_return_val and destroy the
# composer window
# Arguments:
#	+ ew:		editor window
#	+ pw:		parent window path
#----------------------------------------------------------------------
proc qcomposerDone {sid ew pw oldlayer} {
    global qcomposer_return_val;

    set q [qcomposerGetQuery $ew];
    if {$q == {}} {
	# accept no empty query
	pd_dialog .warning {Warning} {No query is found, please \
		type in your query first!} {} warning 0 {OK};
	return;
    }

    # now we can invode attribute customization
    # dialog
    # wm withdraw $pw;

    qcomposer_customize $sid "$q" 1 $oldlayer
}

#----------------------------------------------------------------------
# Cancel the editing of query, restore the previous value of query
# and return
#----------------------------------------------------------------------
proc qcomposerCancel {sid w} {
    global session qcomposer_old_val qcomposer_return_val

    set session($sid,curquery) $qcomposer_old_val
    set qcomposer_return_val $qcomposer_old_val
    destroy $w
}

#----------------------------------------------------------------------
# Execute the current query, if the query has results, display the
# results in a table.  If the query is NONE, the whole table will
# be shown.
#----------------------------------------------------------------------
proc qcomposerExec {sid w} {
    global session

    set q [qcomposerGetQuery $w];
    if {$q == {}} {
	# accept no empty query
	pd_dialog .warning {Warning} {No query is found, please \
		type in your query first!} {} warning 0 {OK};
	return;
    }

    # check for selection query, note that string match won't
    # work if $q has new line in it, use regexp will surely work
    set hasResult [regexp "select*" $q]

    if {$hasResult == 1} {
	# use show table to run the query
	set session($sid,curquery) [parseQuery $sid $q]
	showTab $session($sid,curext) $sid
    } else {
	# just run the query, no results
	PDsql_cursorrun [parseQuery $sid $q]
    }
}

#----------------------------------------------------------------------
# return the query in a text widgets, possibly do some processing
# to make it syntactically right
#----------------------------------------------------------------------
proc qcomposerGetQuery  w  {
    return [string trim [$w get 0.0 end]]
}

#----------------------------------------------------------------------
# make up the catalog menu
#----------------------------------------------------------------------
proc qcomposerMkCatMenu {sid catMenu queryEditor} {
    global session attrinfo

    set rc [catch {array names attrinfo} err]
    if $rc {
	tk_dialog .cat_err {Error} {Empty database, exiting...} \
		error  0 OK
	exit
    } 
    set extnames [lsort $err]

    # for each extent name, we create an cascade entry
    foreach extn $extnames {
	if [is_fname_tmp $extn] { continue }
	$catMenu add cascade -label $extn -menu $catMenu.w$extn

	# create the menu for this cascade entry, namely
	# all the attribute
	menu $catMenu.w$extn
	foreach el $attrinfo($extn) {
	    set attrName [lindex $el 0]
	    $catMenu.w$extn add command -label "$attrName" \
		    -command "$queryEditor insert insert \
		    $extn.$attrName"
	}
    }
}

proc qcomposerMkQueryMenu {sid extmenu qew} {
    global session attrinfo;

    set extnames [lsort [array names attrinfo] ];
    foreach extn $extnames {
	if [is_fname_tmp $extn] { continue }
	# set q [querygen $sid $extn];
	set q "select * from $extn";
	$extmenu add command -label "$extn" -command \
		"$qew delete 0.0 end; \
		$qew insert insert \"$q\"; \
		set session($sid,qname) \"$extn\"";
    }
}

#----------------------------------------------------------------------
# change editor font
#----------------------------------------------------------------------
proc qcomposerChangeFont {sid queryEditor tbar args} {
    global debugMode session
    if $debugMode {
	puts "qcomposerChangeFont $sid $queryEditor $tbar $args"
    }

    set session($sid,qcomposer_font) [$tbar.font get]
    set session($sid,qcomposer_size) [$tbar.size get]

    set f [string tolower $session($sid,qcomposer_font)]
    set new_font [format "-*-%s-bold-r-normal-*-%s-*-*-*-*-*-*-*" \
	    $f $session($sid,qcomposer_size)]
    $queryEditor config -font $new_font
}

#----------------------------------------------------------------------
# add current query to history file
#----------------------------------------------------------------------
proc qcomposerAddHist queryEditor {
    global debugMode qcomposer

    set q [qcomposerGetQuery $queryEditor]
    if $debugMode { puts "qcomposerAddHist $q" }
    history::add $q $qcomposer(hf)
}

#----------------------------------------------------------------------
# view history
#----------------------------------------------------------------------
proc qcomposerViewHist {fn btn} {
    history_view $fn $btn;
}

# this procedure will set initial query when composer
# is invoked
proc qcomposer_initQuery sid {
    global session qcomposer_prev_query;
    global LAYER_LEVEL LAYER_LEVELSUFIX

    if {$session($sid,curlayer) == {}} {
	set session($sid,curquery) "";
	set session($sid,curdesc) {};
	set qcomposer_prev_query "";
    } else {
	set session($sid,curquery) \
		[getLayerQuery $session($sid,curlayer) $sid "false" 0];
	set curlevel [lindex $session($sid,curlayer) $LAYER_LEVEL]
	if {$curlevel != "NONE"} {
	  	set levelsufix [lindex $session($sid,curlayer) \
			$LAYER_LEVELSUFIX]
		append session($sid,curqery) levelsufix
	}
	set qcomposer_prev_query $session($sid,curquery);
	set session($sid,curdesc) \
		[getLayerQueryDesc $session($sid,curlayer)];
    }
}

# set the initial display characteristic after result schema
# is available
proc qcomposer_initLayer {sid desc} {
    global session curcol curbitmap qcomposer_return_val;

    # notice that we don't need to init session($sid,curquery) here
    # anymore, it is already been set before this is called.

    set attrcnt [llength $desc];
    if {$session($sid,curlayer) == {}} {
	# new layer
	set session($sid,curwidth) {};
	set session($sid,curcol) {};
	set session($sid,curbitmap) {};
	set session($sid,attr_label) {};
	set session($sid,is_trans) {};
	set qcomposer_return_val {};

	for {set i 0} {$i < $attrcnt} {incr i} {
	    lappend session($sid,curcol) $curcol;
	    lappend session($sid,curwidth) 1;
	    lappend session($sid,curbitmap) $curbitmap;	
	    lappend session($sid,is_trans) 0;
	    lappend session($sid,attr_label) -1;
	}
    } else {
	set session($sid,curcol) [getLayerColor $session($sid,curlayer)];
	set session($sid,curbitmap) [getLayerBitmap $session($sid,curlayer)];
	set session($sid,qname) [getLayerName $session($sid,curlayer)];
	set session($sid,is_trans) [getLayerTrans $session($sid,curlayer)];
	set session($sid,curwidth) [getLayerWidth $session($sid,curlayer)];
	set session($sid,attr_label) [getLayerLabel $session($sid,curlayer)];
	set qcomposer_return_val $session($sid,curlayer);
    }
}

# customize the appearence of result of the query, the query
# is guaranteed to be non-empty, it is unparsed query directly
# obtained from qcomposer window
# 
proc qcomposer_customize {sid q showQuery oldlayer} {
    global session qcomposer_mpe;
    global qcomposer_return_val;
    global qcomposer_prev_query;
    global LAYER_UPDATE LAYER_LEVEL;

    set descTmpfnNotValid 0
    set descTmpfnValid 1

    if {$session($sid,curdesc) == {}} {
    	set session($sid,level) ALL
    } else {
	set session($sid,level) [lindex $session($sid,curlayer) $LAYER_LEVEL]
    }

    if {($session($sid,level) == "ALL" || $session($sid,level) == "all") && \
	($showQuery == 0 || ($session($sid,curdesc) != {} \
	&& $q == $qcomposer_prev_query \
	 && $session($sid,update) == $session($sid,old_update)))} {
	# this is a previously created layer
	# no need to run the query anymore
	qcomposer_initLayer $sid $session($sid,curdesc)
	set session($sid,level) [lindex $session($sid,curlayer) $LAYER_LEVEL]
	set qcomposer_return_val \
		[queryConfigure $sid $session($sid,curdesc) \
		$descTmpfnNotValid 1];
	if {$showQuery != 0} {
	    destroy .qcomposer
 	}
    } else {

    set tmpfn [getTmpfnFromDesc $session($sid,curdesc)];
    if [is_fname_tmp $tmpfn] {
	# PDsql_cursorclose
	PDsql_cursorrun "drop table $tmpfn"
    }
    
    # execute the query
    ptkPDWait ON
    ptkPDStatBox ON
    ptkPDStatBox::setMsg "Processing Query..."

    set nq [parseQuery $sid $q];

    set rc [catch {PDsql_cursoropen $nq "nomimic"} err]
    if $rc {
	# error occurred, do some cleanups
	# and then return without invoking
	# queryConfigure
	ptkPDStatBox OFF
	ptkPDWait OFF
	pd_dialog .error {Query Processing Error} "$err" {} \
		error 0 {Ok}
	return
    }
    set session($sid,query_opend) 1

    # otherwise, the result of open query is in err
    # and we can withdraw our composer to avoid confusion
    set desc [PDsql_cursorschema]
    set session($sid,curquery) $q
    if { $showQuery != 0 } {
    	wm withdraw .qcomposer
    }

    # set default customization options
    qcomposer_initLayer $sid $desc;
    
    ptkPDStatBox OFF;
    ptkPDWait OFF;

    # invoke the query configuration panel, and return
    # the configured layer data structure
    # delete composer window to return to layerdialog
    set qcomposer_return_val [queryConfigure $sid $desc $descTmpfnValid \
					     $oldlayer];
    if { $showQuery != 0 } {
        destroy .qcomposer
    }
    }
}

proc resetDispLevel {sid} {
    global session 

    toplevel .setl
    wm title .setl "Set New Display Level"

    set x 200
    set y 150 
    wm geometry .setl +$x+$y

    frame .setl.frl
    label .setl.frl.label -text "Type in the new display level you want for \
			    	this session"
    pack .setl.frl.label -expand yes -fill both -padx 10 -pady 10
    frame .setl.fre
    set oldvalue $session($sid,level)
    label .setl.fre.label -text "Enter Value: "
    set en [entry .setl.fre.entry -width 20 -relief sunken -bd 2 -textvariable\
 			  session($sid,level)]
    set session($sid,level_changed) 1
    set valid 0
    bind $en <Return> "Checkvalid $sid $oldvalue; \
		       destroy .setl;"
    
    pack .setl.fre.label .setl.fre.entry -side left -fill both -padx 10 \
	 -pady 10

    frame .setl.frb
    button .setl.frb.save -text "Save" -width 10 \
	    -command "Checkvalid $sid $oldvalue; \
		      destroy .setl;"
    button .setl.frb.cancel -text "Cancel" -width 10 \
	    -command "set session($sid,level) $oldvalue; destroy .setl"
    pack .setl.frb.save .setl.frb.cancel -expand yes -side left -padx 20 \
	-pady 20 

    pack .setl.frl -expand yes -fill both 
    pack .setl.fre -expand yes -fill both 
    pack .setl.frb -expand yes -fill both 

    tkwait window .setl

}

proc Checkvalid {sid oldvalue} {
	global session

	set number $session($sid,level)
    if  {[regexp {[^0-9]} $number] && (![regexp -nocase {^all$} $number])} {
	pd_dialog .error {Number Entery Error} "Please Enter a Number Value" \
                  {} error 0 {Ok}
	set session($sid,level) $oldvalue
	return TCL_ERROR;
    }
    return TCL_OK;
}

proc Xact {query} {
    set rc [catch {PDsql_cursorrun $query} err]
    if $rc {
	# error occurred, do some cleanups
	# and then return without invoking
	# queryConfigure
	ptkPDStatBox OFF
	ptkPDWait OFF
	pd_dialog .error {Query Processing Error} "Invalid Transactional \
		Operation" {} error 0 {Ok}
	return
    }
}

