# <std-header style=TCL orig-src=PTK>
#
#            Copyright 1998, NCR Corporation.  All Rights Reserved.
#
#                                NCR Confidential.
#
#
#  $Id: queryConfig.tcl,v 1.32 1998/05/19 03:22:32 kupsch Exp $
#
#  This file was originally part of ptk, a front end based on TCL/TK to the
#  Paradise database system developed by the University of Wisconsin-Madison.
#  The above copyright applies to changes made to this file by NCR Corporation.
#  The original copyright which may appear in the source below applies to the
#  original file as obtained by NCR Corporation and unchanged parts of this file.
#
#
#   -- do not edit anything above this line --   </std-header>

set qselect_retval {}

proc queryConfigure {sid desc descTmpfnValid oldlayer {w .qs}} {
    global session globalVarList2 qselect_retval maxnattr;
    global dlg_active ptk_hint

    set dlg_active(querycfg) 1;
    
    # declare application wide global variables
    foreach i $globalVarList2 {
	global $i
    }

    catch {destroy $w};
    toplevel $w;
    wm title $w "Presentation Configuration";
    set wmposx [expr 500 + ($sid * 10) % 250]
    set wmposy [expr 220 + ($sid * 10) % 250]
    wm geometry $w "+$wmposx+$wmposy"

    ptkPDWait ON

    # maxnattr should always be multiples of 3 to look good on screen
    set maxnattr 18;

    #
    # The following code will generate the query name entry
    #
    set qnamef [frame $w.qname -bd 1m]
    set qname_entry [entry $qnamef.entry -relief sunken -width 16 \
	    -textvariable session($sid,qname) -bg white -fg blue];
    set qname_label [label $w.qname.label -text "Query Name:"];
    pack $qname_label $qname_entry -side left -pady 1m -padx 1m -expand 1 \
	    -fill x;

    set qlevelf [frame $w.qlevel -bd 1m]
    set qlevel_button [button $qlevelf.level -text "Set" \
	    -command "resetDispLevel $sid"]
    set qlevel_label [label $w.qlevel.label -text "Set Display Level:"];
    pack $qlevel_label $qlevel_button -side left -pady 1m -padx 1m -expand 1 \
	    -fill x;
    focus $qname_entry;

    #
    # The following put a frame at the bottom to hold the ok buttom
    #
    set botf [frame $w.botFrame];
    button $w.botFrame.ok -text DONE 
    button $w.botFrame.cancel -text CANCEL \
	    -command "ptkQconfigCancel $sid $w";
    pack  $w.botFrame.ok  $w.botFrame.cancel \
	    -side left -pady 1m -padx 2m -expand 1

    if { ([llength $session($sid,layers)]>0)
	  && ($session($sid,bbox_valid)==1) } {
	# can not do center if we have no visible layer so far.
	# This is to prevent user from clicking center when
	# they don't mean to.
	$w.botFrame.ok config -command \
		"ptkQconfigDone $sid $w [list $desc] $descTmpfnValid $oldlayer"
    } {
	# on the other hand, center is the right thing to do
	# if user is constructing the first visible layer
	$w.botFrame.ok config -command \
		"ptkQconfigCenter $sid $w [list $desc] $descTmpfnValid \
		                  $oldlayer"
    }
    #
    # The following creates the frame that contains all the show
    # attributes
    #
    set showattrf [frame $w.attr -bd 1m -relief groove -width 600 -height 600];
    label $w.attr.label -text "Show Attributes";
    pack $w.attr.label -side top -pady 3m;

    pack $qnamef $qlevelf $showattrf $botf -side top -padx 1 -expand 1 -fill x;
    
    fill_attr_frame $sid $showattrf $desc;

    if $ptk_hint {
	tixAddBalloon $w.botFrame.ok {Click to see your query results\
		show up on map screen.  So click before you look\
		that way.}
	tixAddBalloon $w.botFrame.cancel {Click to cancel what you have\
		done so far.}
	tixAddBalloon $qname_entry {Type in the name for this layer.\
		Take the default name if you wish.}
    }
    ptkPDWait OFF

    tkwait window $w;
    catch {tixBalPopdown}
    
    set dlg_active(querycfg) 0;
    return $qselect_retval
}

#
# called only once initially to create the attriubte frame
# it is contained in a scrollable canvas if it is too big
#
proc fill_attr_frame {sid parent_win desc} {
    global session ptk_hint

    set attrcnt [llength $desc];
    set attrnames [getAttrNamesFromDesc $desc];
    set attrtypestrs [getAttrTypesFromDesc $desc];
    set tmpfn [getTmpfnFromDesc $desc];

    # create a canvas within the parent window
    set canv [canvas $parent_win.c];
    set w [frame $canv.w];
    $canv create window 0 0 -anchor nw -window $w;

    # determine the max attribute name length
    set maxlen 0;
    for {set i 0} {$i < $attrcnt} {incr i} {
	set len [string length [lindex $attrnames $i]];
	if {$len > $maxlen} {
	    set maxlen $len;
	}
    }
    
    for {set i 0} {$i < $attrcnt} {incr i} {
	set attr_name [lindex $attrnames $i];
	set attr_typestr [lindex $attrtypestrs $i];
	set attr_typeid [ptkTypename2id $attr_typestr]

	set attrframe [frame $w.attr$i];
	pack $attrframe;
	label $attrframe.an$i -width $maxlen -anchor w -text "$attr_name" \
		-bd 2 -relief ridge -fg red;
	label $attrframe.at$i -width 16 -anchor w -text "($attr_typestr)" \
		-bd 2 -relief ridge -fg blue;
	frame $attrframe.bf$i

# For the demo, uncomment the following lines, as it sets the defaults in 
# the demo so that less attribute customization is required - Jignesh
#        puts "set default"
#	SetDefault $sid $i $attrtypestrs $attrnames;

	button $attrframe.bf$i.cust$i -text "Customize" \
		-command "Customize $sid $i [list $attrtypestrs] [list $attrnames]" \
		-state disabled;
	pack $attrframe.bf$i.cust$i
	pack $attrframe.an$i $attrframe.at$i $attrframe.bf$i -side left \
		-padx 3 -pady 1;
	
	if [isAttrDispType $attr_typeid] {
	    # enable the customize button of spatial attribute
	    $attrframe.bf$i.cust$i configure -state normal;
	    if $ptk_hint {
		tixAddBalloon $attrframe.bf$i.cust$i {Click to customize\
			the presentation of this spatial attribute\
			on screen.  Otherwise you will have to bear with\
			the poor taste of default value}
	    }
	}
    }

    # figure out how big the $w has grown to and decide how to set
    # the scroll region of canvas
    update;
    set width [winfo reqwidth $w];
    set height [winfo reqheight $w];
    set useXScrollBar 0;
    set useYScrollBar 0;
    
    if {$width > 500} {
	set useXScrollBar 1;
    }
    if {$height > 500} {
	set useYScrollBar 1;
    }

    if $useXScrollBar {
	set xbar [scrollbar $parent_win.sx -orient horiz -command "$canv xview" \
		-relief sunk];
	pack append $parent_win $xbar {bottom fillx};
	$canv config -xscroll "$xbar set";
    } else {
	$canv config -width $width;
    }
    if $useYScrollBar {
	set ybar [scrollbar $parent_win.sy -command "$canv yview" -relief sunk];
	pack append $parent_win $ybar {right filly};
	$canv config -yscroll "$ybar set";
    } else {
	$canv config -height $height;
    }
    pack append $parent_win $canv {expand fill};
    $canv configure -scrollregion "0 0 $width $height";
}

#======================================================================
# ptkQdCenter: invoke to change the current mapview boudning box
# to the maximum bounding box of the resulting tuple of current
# query, otherwise it is similar to ptkQdDone.
#
# The query composed by query composer that meant to use the current
# bounding box (i.e. the query that ends with "overlaps" will of
# course cause trouble when used with "Center".
#======================================================================
proc ptkQconfigCenter {sid w desc descTmpfnValid oldlayer} {
    global session qselect_retval
    global debugMode LAYER_ID

    if $debugMode {
	puts "\tdesc = $desc";
    }

    set num_attrs [llength $desc];
    set attrnames [getAttrNamesFromDesc $desc];
    set attrtypestrs [getAttrTypesFromDesc $desc];
    if $descTmpfnValid {
	set tmpfn [getTmpfnFromDesc $desc];
    } else {
	set tmpfn [getLayerTmpfn $session($sid,curlayer)];
    }
    
    set color {};
    set bitmap {};
    set width {};
    set lbl_idxs {};

    ptkPDWait ON

    if {$num_attrs != [llength $session($sid,curcol)]} {
	# in this case, we are dealing with creating a new layer
	# and we have to set proper default value first
	for {set i 0} {$i < $num_attrs} {incr i} {
	    lappend color [lindex $session($sid,curcol) 0]; 
	    lappend bitmap [lindex $session($sid,curbitmap) 0];
	    lappend width [lindex $session($sid,curwidth) 0];
	    lappend lbl_idxs [lindex $session($sid,attr_label) 0];
	}
    } else {
	set color $session($sid,curcol);
	set bitmap $session($sid,curbitmap);
	set width $session($sid,curwidth);
	set lbl_idxs $session($sid,attr_label);
    }

    # check whether the query is changed or not
    set query_changed 0
    set layer_id -1
    if {$oldlayer==1} {
	set layer_id [lindex $session($sid,curlayer) $LAYER_ID]
    }
    if {$session($sid,curquery) == ""} {
	# encounter empty query
	tk_dialog .qselect_err {Error} {Please enter a query!} \
		error 0 OK
	return
    }
    

    # if we have valid tmpfn, then we don't need to run the
    # query again

    if { $tmpfn == "" } { set tmpfn NONE }
    if {$tmpfn == "NONE"} {
	if {$session($sid,query_opend)} {
	    set box [PDsql_bboxtmp "$session($sid,curquery)" "opend"]
#	    set $session($sid,query_opend) 0
	} else {
	    set box [PDsql_bboxtmp "$session($sid,curquery)"]
 	}
    } else {
	if {$session($sid,query_opend)} {
	    set box [PDsql_bboxtmp "select * from $tmpfn" "opend"]
#	    set $session($sid,query_opend) 0
	} else {
	    set box [PDsql_bboxtmp "select * from $tmpfn"]
 	}
    }
    if $debugMode { puts "new bounding box = $box" }

    set session($sid,bbox_valid) [lindex $box 5]
    # change the current bounding box
    # but note that the incoming box use lower-left	#
    # and upper right as the two points		#
    set session($sid,bx1) [lindex $box 0]
    set session($sid,by1) [lindex $box 3]
    set session($sid,bx2) [lindex $box 2]
    set session($sid,by2) [lindex $box 1]
    set nbox [aspect_bbox $session($sid,bx1) $session($sid,by1) \
	    $session($sid,bx2) $session($sid,by2)]
    ptk_bbox::set $sid  [lindex $nbox 0] [lindex $nbox 1] \
	    [lindex $nbox 2] [lindex $nbox 3]


    if {[is_fname_tmp $tmpfn] && $session($sid,query_opend) == 0} {
	set session($sid,query_opend) 0
	# PDsql_cursorclose
	PDsql_cursorrun "drop table $tmpfn"
    }

    # as a temporary hack, save the tmpfn, avoid
    # a query next time, session($sid,center_tmpfn)
    # should be reset to NONE once it's been used
    # tmpdesc is used to retrieve attrname and attrtype info when append
    # ##PDMAP## macro to "select * from $center_tmpfn
    set tmpfn [lindex $box 4]
    set session($sid,center_tmpfn) $tmpfn
    set session($sid,tmpdesc) $desc
    set sqlQuery $session($sid,curquery)
    set mapsufix [bindQueryToScr $sqlQuery $attrnames \
                  $attrtypestrs] 
    set levelsufix [appendLevel $sqlQuery $sid]

    regexp {(select.*from\ *([a-zA-Z0-9]*))\ *} $sqlQuery tmp baseQuery baseTable
    set newLayer  [makeLayer $sid $session($sid,qname) \
	    $color $bitmap $width $sqlQuery $mapsufix $levelsufix \
	    $session($sid,curext) $session($sid,curdb) \
	    $session($sid,is_trans) $tmpfn $query_changed \
	    $lbl_idxs $desc $baseTable $baseQuery $session($sid,update) \
	    $session($sid,level) $layer_id]
    if {$debugMode == 1} { puts $newLayer }

    set qselect_retval $newLayer
    ptkPDWait OFF
    destroy $w
    return

}

#======================================================================
proc ptkQconfigCancel {sid w} {
    global session qselect_retval
    set qselect_retval $session($sid,curlayer)
    destroy $w
}

#----------------------------------------------------------------------
# return the new layer information, no side effect now.
#----------------------------------------------------------------------
proc ptkQconfigDone {sid w desc descTmpfnValid oldlayer} {
    global session qselect_retval
    global debugMode LAYER_ID

    set num_attrs [llength $desc];
    set attrnames [getAttrNamesFromDesc $desc];
    set attrtypestrs [getAttrTypesFromDesc $desc];
    set session($sid,tmpdesc) $desc
    if $descTmpfnValid {
	set tmpfn [getTmpfnFromDesc $desc];
    } else {
	set tmpfn [getLayerTmpfn $session($sid,curlayer)];
    }
    if { $tmpfn == "" } { set tmpfn NONE }
    set color {};
    set bitmap {};
    set width {};
    set lbl_idxs $session($sid,attr_label);

    if {$num_attrs != [llength $session($sid,curcol)]} {
	# in this case, we are dealing with creating a new layer
	# and we have to set proper default value first
	for {set i 0} {$i < $num_attrs} {incr i} {
	    lappend color [lindex $session($sid,curcol) 0]; 
	    lappend bitmap [lindex $session($sid,curbitmap) 0];
	    lappend width [lindex $session($sid,curwidth) 0];
	}
    } else {
	set color $session($sid,curcol);
	set bitmap $session($sid,curbitmap);
	set width $session($sid,curwidth);
    }
    
    # check whether the query is changed or not
    set query_changed 0
    set layer_id -1
    if {$oldlayer==1} {
	set layer_id [lindex $session($sid,curlayer) $LAYER_ID]
    }

    if {$session($sid,curquery) == ""} {
	# encounter empty query
	tk_dialog .qselect_err {Error} {Please enter a query!} \
		error 0 OK
	return
    }

    set sqlQuery $session($sid,curquery)
    set mapsufix [bindQueryToScr $sqlQuery $attrnames $attrtypestrs]
    set levelsufix [appendLevel $sqlQuery $sid]

    regexp {(select.*from\ *([a-zA-Z0-9]*))\ *} $sqlQuery tmp baseQuery baseTable

    set newLayer  [makeLayer $sid $session($sid,qname) \
	    $color $bitmap $width $sqlQuery $mapsufix $levelsufix \
	    $session($sid,curext) $session($sid,curdb) \
	    $session($sid,is_trans) $tmpfn $query_changed \
	    $lbl_idxs $desc $baseTable $baseQuery $session($sid,update) \
	    $session($sid,level) $layer_id]
    if {$debugMode == 1} { puts "qselect: $newLayer" }
    set qselect_retval $newLayer
    destroy $w
    return
}

proc SetDefault {sid attr_indx attr_types attr_names} {
    global session;
    global debugMode;

    if $debugMode {
        puts "\tattr_indx = $attr_indx";
    }

    set attr_type [lindex $attr_types $attr_indx];
    set attr_name [lindex $attr_names $attr_indx];

    # hack for the demo (pick default based on attribute name)
    if {$attr_name == ""} {
        set session($sid,curcol) [lreplace $session($sid,curcol) \
                $attr_indx $attr_indx $custopts(color)];
    }

    if {$attr_name == ""} {
        set session($sid,curwidth) [lreplace $session($sid,curwidth) \
                $attr_indx $attr_indx $custopts(width)];
    }

    if {$attr_name == ""} {
        set session($sid,curbitmap) [lreplace \
                $session($sid,curbitmap) \
                $attr_indx $attr_indx $custopts(bmap)];
    }

    if {$attr_name == ""} {
        set session($sid,is_trans) [lreplace $session($sid,is_trans) \
                $attr_indx $attr_indx $custopts(trans)];
    }

    if {$attr_name == ""} {
        set session($sid,attr_label) [lreplace \
                $session($sid,attr_label) \
                $attr_indx $attr_indx $custopts(label)];
    }

    if {$attr_type == "ClosedPolygon" || $attr_type == "Rectangle" } {
        set session($sid,curcol) [lreplace $session($sid,curcol) \
                $attr_indx $attr_indx "gold"];
        return;
    }
    if {$attr_type == "Point"} {
        set session($sid,is_trans) [lreplace $session($sid,is_trans) \
                $attr_indx $attr_indx "1"];
        return;
    }
    if {$attr_type == "PolyLine"} {
        set session($sid,curcol) [lreplace $session($sid,curcol) \
                $attr_indx $attr_indx "black"];
        set session($sid,curwidth) [lreplace $session($sid,curwidth) \
                $attr_indx $attr_indx "2"];
        return;
    }

    if {$attr_type == "Raster" || $attr_type == "Raster8H" \
	|| $attr_type == "Raster8" || $attr_type == "Raster16" \
	|| $attr_type == "Raster24"} {
        set session($sid,curwidth) [lreplace $session($sid,curwidth) \
                $attr_indx $attr_indx "0"];
        return;
    }

    if {$attr_type == "Video" || $attr_type == "Text" || $attr_type == "Gif"} {
        return;
    }

    # Other types are not configurable at this time yet
    # disable the option button
    if $debugMode {puts "SetDefault exit"}
}


proc Customize {sid attr_indx attr_types attr_names} {
    global session;
    global debugMode;

    if $debugMode {
	puts "\tattr_indx = $attr_indx";
    }

    set attr_type [lindex $attr_types $attr_indx];
    
    if {$attr_type == "ClosedPolygon" || $attr_type == "Rectangle" } {
	configPolygon $sid $attr_indx $attr_types $attr_names;
	return;
    }
    if {$attr_type == "Point"} {
	configPoint $sid $attr_indx $attr_types $attr_names;
	return;
    }
    if {$attr_type == "PolyLine"} {
	configPline $sid $attr_indx $attr_types $attr_names;
	return;
    }
    if {$attr_type == "Raster" || $attr_type == "Raster8H" || $attr_type == "Raster8" || $attr_type == "Raster16" || $attr_type == "Raster24"} {
	configRaster $sid $attr_indx $attr_types $attr_names;
	return;
    }
    if {$attr_type == "Video"} {
	configVideo $sid $attr_indx $attr_types $attr_names;
	return;
    }
    if {$attr_type == "Text"} {
	configText $sid $attr_indx $attr_types $attr_names;
	return;
    }

    # Other types are not configurable at this time yet
    # disable the option button
    pd_dialog .warning {warning} \
	    "Warning: $attr_type is not a configurable type" {} warning 0 {OK};
    $btn configure -state disabled
    if $debugMode {puts "Customize exit"}
}

set custopts(null) "";
proc initCustopts {sid attrindx} {
    global session custopts;

    if {$attrindx > [llength $session($sid,curcol)]} {
	# new layer, default color list has only one element
	set custopts(color) [lindex $session($sid,curcol) 0];
	set custopts(width) [lindex $session($sid,curwidth) 0];
	set custopts(bmap) [lindex $session($sid,curbitmap) 0];
	set custopts(label) [lindex $session($sid,attr_label) 0];
	set custopts(trans) [lindex $session($sid,is_trans) 0];
    } else {
	set custopts(color) [lindex $session($sid,curcol) $attrindx];
	set custopts(width) [lindex $session($sid,curwidth) $attrindx];
	set custopts(bmap) [lindex $session($sid,curbitmap) $attrindx];
	set custopts(label) [lindex $session($sid,attr_label) \
		$attrindx];
	set custopts(trans) [lindex $session($sid,is_trans) $attrindx];
    }
}

proc configPolygon {sid attrindx attr_types attr_names} {
    global session custopts 
    upvar 1 oldlayer old

    set attrcnt [llength $attr_types];
    set attr_name [lindex $attr_names $attrindx];
    initCustopts $sid $attrindx;

    set intAttrsIndxs {};
    set i 0;
    foreach at $attr_types {
	if {$at == "Integer"} {
	    lappend intAttrsIndxs $i;
	}
	incr i;
    }

    if [useColorPallete $custopts(color)] {
	set color_sample "                           ";
	set bgcolr $custopts(color);
    } else {
	set color_sample \
		"colored by [lindex $attr_names $custopts(color)]";
	set bgcolr "white";
    }

    set w ".pgconfig";
    catch {destroy .pgconfig};
    toplevel $w;
    wm title $w "Configure Appearance";
    wm geometry $w +300+300;

    set len [string length "Configure Polygon $attr_name"];
    set title [label $w.msg -text "Configure $attr_name" -fg blue \
	    -bd 1 -relief raised -width $len -height 5 \
	    -anchor c];

    set cf [frame $w.cf -bd 1 -relief raised];
    set lf [frame $w.lf -bd 1 -relief raised];
    set okf [frame $w.okf -bd 1 -relief raised];

    pack $title $lf $cf $okf -side top -expand 1 -fill x;


    # set up the label frame
    if {$custopts(label) != -1} {
	set lshow_txt [lindex $attr_names $custopts(label)];
	set lshow_txt "Labeled by <$lshow_txt>";
    } else {
	set lshow_txt "No Label"
    }
    set lbtn [menubutton $lf.mb -text "Choose Label" -menu $lf.mb.m \
	    -bd 2 -relief raised];
    set lshow [label $lf.show -text "$lshow_txt" -bg white];
    pack $lbtn $lshow -side left -expand 1 -fill x -padx 0.5m \
	    -pady 5
    menu $lbtn.m;

    set msg "No Label";
    set len [string length $msg];
    $lbtn.m add command -label "None" -command \
	    "set custopts(label) -1; \
	    $lshow configure -text \"$msg\" -width $len";

    set i 0;
    foreach attrn $attr_names {
	set msg "Labeled by <$attrn>";
	set len [string length $msg];
	$lbtn.m add command -label "$attrn" -command \
		"set custopts(label) $i; \
		$lshow configure -text \"$msg\" -width $len";
	incr i;
    }

    # set up the color frame
    set cbtn [menubutton $cf.mb -text "Choose Color" -menu $cf.mb.m\
	    -bd 2 -relief raised];
    set csample [label $cf.show -text $color_sample -bg $bgcolr\
	    -bd 2 -relief sunken];
    pack $cbtn $csample -side left -expand 1 -fill x -padx 0.5m \
	    -pady 5

    menu $cbtn.m;
    $cbtn.m add cascade -label "Color Palette" -menu $cbtn.m.c;
    if {$intAttrsIndxs != {}} {
	$cbtn.m add cascade -label "Integer Attributes" \
		-menu $cbtn.m.i;
	menu $cbtn.m.i;
	foreach i $intAttrsIndxs {
	    set attrn [lindex $attr_names $i];
	    set msg "Colored by <$attrn>"; 
	    set len [string length $msg]; 
	    $cbtn.m.i add command -label "$attrn" -command \
		    "set custopts(color) $i; \
		    $csample configure -text \"$msg\" -width $len \
		    -background white";
	}
    }

    global col_palete;
    set msg "                           ";
    set len [string length $msg]; 
    menu $cbtn.m.c;
    foreach color_cell $col_palete {
	$cbtn.m.c add command -label "$color_cell" \
		-background $color_cell -command \
		"set custopts(color) $color_cell; \
		$csample configure -text \"$msg\" -width $len \
		-background $color_cell";
    }


    # set up the ok button
    set okbtn [button $okf.ok -text "OK" -command \
	    "configDone $sid $attrindx $w"];
    pack $okbtn -expand 1 -fill x -pady 0.5m \
	    -pady 5

    grab $w;
    tkwait window $w;
}

proc setTransparent {widget status} {
    global custopts
    if {$widget == "transparent"} {
	if {$status == 0} {
	    set custopts(trans) 0
	} else {
	    set custopts(trans) 1
	}
    }
}

proc configPoint {sid attrindx attr_types attr_names} {
    global session custopts;

    set attrcnt [llength $attr_types];
    set attr_name [lindex $attr_names $attrindx];
    initCustopts $sid $attrindx;

    set intAttrsIndxs {};
    set i 0;
    foreach at $attr_types {
	if {$at == "Integer"} {
	    lappend intAttrsIndxs $i;
	}
	incr i;
    }

    if [useColorPallete $custopts(color)] {
	set color_sample "                           ";
	set bgcolr $custopts(color);
    } else {
	set color_sample \
		"colored by [lindex $attr_names $custopts(color)]";
	set bgcolr "white";
    }

    set w ".pgconfig";
    toplevel $w;
    wm title $w "Configure Appearance";
    wm geometry $w +300+300;

    set len [string length "Configure Point $attr_name"];
    set title [label $w.msg -text "Configure $attr_name" -fg blue \
	    -bd 1 -relief raised -width $len -height 5 \
	    -anchor c];

    set cf [frame $w.cf -bd 1 -relief raised];
    set tf [frame $w.tf -bd 1 -relief raised];
    set bf [frame $w.bf -bd 1 -relief raised];
    set lf [frame $w.lf -bd 1 -relief raised];
    set okf [frame $w.okf -bd 1 -relief raised];
    pack $title $cf $tf $bf $lf $okf -side top -expand 1 -fill x 

    # set up the color frame
    set cbtn [menubutton $cf.mb -text "Choose Color" -menu $cf.mb.m\
	    -bd 2 -relief raised];
    set csample [label $cf.show -text $color_sample -bg $bgcolr\
	    -bd 1 -relief sunken];
    pack $cbtn $csample -side left -expand 1 -fill x -padx 0.5m \
	    -pady 5

    menu $cbtn.m;
    $cbtn.m add cascade -label "Color Palette" -menu $cbtn.m.c;
    if {$intAttrsIndxs != {}} {
	$cbtn.m add cascade -label "Integer Attributes" \
		-menu $cbtn.m.i;
	menu $cbtn.m.i;
	foreach i $intAttrsIndxs {
	    set attrn [lindex $attr_names $i];
	    set msg "Colored by <$attrn>"; 
	    set len [string length $msg]; 
	    $cbtn.m.i add command -label "$attrn" -command \
		    "set custopts(color) $i; \
		    $csample configure -text \"$msg\" -width $len \
		    -background white";
	}
    }

    global col_palete;
    set msg "                           ";
    set len [string length $msg]; 
    menu $cbtn.m.c;
    foreach color_cell $col_palete {
	$cbtn.m.c add command -label "$color_cell" \
		-background $color_cell -command \
		"set custopts(color) $color_cell; \
		$csample configure -text \"$msg\" -width $len \
		-background $color_cell";
    }

    # set up transparency checkbutton
    label $tf.label -text "Transparency: "
    set ttb [tixSelect $tf.sel \
	    -font "-adobe-helvetica-bold-r-normal-*-12-*-*-*-*-*-*-*"]
    $tf.sel add transparent -text "Transparent" -width 12
    $tf.sel add opaque -text "Opaque" -width 12
    if $custopts(trans) {
	$tf.sel config -value {transparent}
    } {
	$tf.sel config -value {opaque}
    }
    $tf.sel config -command {setTransparent}
    pack $tf.label -side left -pady 5 -expand yes -fill x
    pack $ttb -side left -pady 5

    # setup bitmap menu
    set icon_label [label $bf.label -text "Icon:" -anchor w];
    set icon_mb [menubutton $bf.mbtn -menu $bf.mbtn.m \
	    -bitmap $custopts(bmap) -bd 2 -relief raised];
    pack $icon_label $icon_mb -side left -padx 1m -fill x -pady 1m \
	    -expand 1 -pady 5

    global PDbitmaps bitmap_dir;
    set icon_menu [menu $icon_mb.m];
    foreach bm $PDbitmaps {
	$icon_menu add radiobutton -bitmap $bm \
		-variable custopts(bmap) \
		-value $bm -command "$icon_mb configure -bitmap $bm";
    }

    # set up the label frame
    if {$custopts(label) != -1} {
	set lshow_txt [lindex $attr_names $custopts(label)];
	set lshow_txt "Labeled by <$lshow_txt>";
    } else {
	set lshow_txt "None"
    }
    set lbtn [menubutton $lf.mb -text "Choose Label" -menu $lf.mb.m \
	    -bd 2 -relief raised];
    set lshow [label $lf.show -text "$lshow_txt" -bg white];
    pack $lbtn $lshow -side left -expand 1 -fill x -padx 0.5m \
	    -pady 5
    menu $lbtn.m;
    set msg "No Label";
    set len [string length $msg];
    $lbtn.m add command -label "None" -command \
	    "set custopts(label) -1; \
	    $lshow configure -text \"$msg\" -width $len";
    set i 0;
    foreach attrn $attr_names {
	set msg "Labeled by <$attrn>";
	set len [string length $msg];
	$lbtn.m add command -label "$attrn" -command \
		"set custopts(label) $i; \
		$lshow configure -text \"$msg\" -width $len";
	incr i;
    }

    # set up the ok button
    set okbtn [button $okf.ok -text "OK" -command \
	    "configDone $sid $attrindx $w"];
    pack $okbtn -expand 1 -fill x -pady 0.5m -pady 5

    grab $w;
    tkwait window $w;
}

proc setLineWidth {value} {
    global custopts
    set custopts(width) $value
}

proc configPline {sid attrindx attr_types attr_names} {
    global session custopts;

    set attrcnt [llength $attr_types];
    set attr_name [lindex $attr_names $attrindx];
    initCustopts $sid $attrindx;

    set intAttrsIndxs {};
    set i 0;
    foreach at $attr_types {
	if {$at == "Integer"} {
	    lappend intAttrsIndxs $i;
	}
	incr i;
    }

    if [useColorPallete $custopts(color)] {
	set color_sample "                           ";
	set bgcolr $custopts(color);
    } else {
	set color_sample \
		"colored by [lindex $attr_names $custopts(color)]";
	set bgcolr "white";
    }

    set w ".pgconfig";
    toplevel $w;
    wm title $w "Configure Appearance";
    wm geometry $w +300+300;

    set len [string length "Configure PolyLine $attr_name"];
    set title [label $w.msg -text "Configure $attr_name" -fg blue \
	    -bd 1 -relief raised -width $len -height 5 \
	    -anchor c];
 
    set cf [frame $w.cf -bd 1 -relief raised];
    set wf [frame $w.wf -bd 1 -relief raised];
    set lf [frame $w.lf -bd 1 -relief raised];
    set okf [frame $w.okf -bd 1 -relief raised];
    pack $title $cf $wf $lf $okf -side top -expand 1 -fill x

    # set up the label frame
    if {$custopts(label) != -1} {
	set lshow_txt [lindex $attr_names $custopts(label)];
	set lshow_txt "Labeled by <$lshow_txt>";
    } else {
	set lshow_txt "None"
    }
    set lbtn [menubutton $lf.mb -text "Choose Label" -menu $lf.mb.m \
	    -bd 2 -relief raised];
    set lshow [label $lf.show -text "$lshow_txt" -bg white];
    pack $lbtn $lshow -side left -expand 1 -fill x -padx 0.5m \
	    -pady 5
    menu $lbtn.m;
    set msg "No Label";
    set len [string length $msg];
    $lbtn.m add command -label "None" -command \
	    "set custopts(label) -1; \
	    $lshow configure -text \"$msg\" -width $len";
    set i 0;
    foreach attrn $attr_names {
	set msg "Labeled by <$attrn>";
	set len [string length $msg];
	$lbtn.m add command -label "$attrn" -command \
		"set custopts(label) $i; \
		$lshow configure -text \"$msg\" -width $len";
	incr i;
    }

    # set up the color frame
    set cbtn [menubutton $cf.mb -text "Choose Color" -menu $cf.mb.m\
	    -bd 2 -relief raised];
    set csample [label $cf.show -text $color_sample -bg $bgcolr\
	    -bd 2 -relief sunken];
    pack $cbtn $csample -side left -expand 1 -fill x -padx 0.5m \
	    -pady 5

    menu $cbtn.m;
    $cbtn.m add cascade -label "Color Palette" -menu $cbtn.m.c;
    if {$intAttrsIndxs != {}} {
	$cbtn.m add cascade -label "Integer Attributes" \
		-menu $cbtn.m.i;
	menu $cbtn.m.i;
	foreach i $intAttrsIndxs {
	    set attrn [lindex $attr_names $i];
	    set msg "Colored by <$attrn>"; 
	    set len [string length $msg]; 
	    $cbtn.m.i add command -label "$attrn" -command \
		    "set custopts(color) $i; \
		    $csample configure -text \"$msg\" -width $len \
		    -background white";
	}
    }

    global col_palete;
    set msg "                           ";
    set len [string length $msg]; 
    menu $cbtn.m.c;
    foreach color_cell $col_palete {
	$cbtn.m.c add command -label "$color_cell" \
		-background $color_cell -command \
		"set custopts(color) $color_cell; \
		$csample configure -text \"$msg\" -width $len \
		-background $color_cell";
    }

    # set up the width frame
    set wlabel [label $wf.l -text "Line Width"];
    set wctrl [tixControl $wf.ctl -entrywidth 8 -value $custopts(width) \
	    -step 1 -llimit 1 -ulimit 16]
    $wctrl config -command {setLineWidth}
    pack $wlabel $wctrl -side left -expand yes -fill x -padx 0.5m -pady 5

    # set up the ok button
    set okbtn [button $okf.ok -text "OK" -command \
	    "configDone $sid $attrindx $w"];
    pack $okbtn -expand 1 -fill x -pady 0.5m -pady 5

    grab $w;
    tkwait window $w;
}

proc configRaster {sid attrindx attr_types attr_names} {
    global session custopts;

    set attrcnt [llength $attr_types];
    set attr_name [lindex $attr_names $attrindx];
    initCustopts $sid $attrindx;

    set intAttrsIndxs {};
    set i 0;
    foreach at $attr_types {
	if {$at == "Integer"} {
	    lappend intAttrsIndxs $i;
	}
	incr i;
    }

    if [useColorPallete $custopts(color)] {
	set color_sample "                           ";
	set bgcolr $custopts(color);
    } else {
	set color_sample \
		"colored by [lindex $attr_names $custopts(color)]";
	set bgcolr "white";
    }

    set w ".pgconfig";
    toplevel $w;
    wm title $w "Configure Appearance";
    wm geometry $w +300+300;

    set len [string length "Configure Raster $attr_name"];
    set title [label $w.msg -text "Configure $attr_name" -fg blue \
	    -bd 1 -relief raised -width $len -height 5];
    set cf [frame $w.cf -bd 1 -relief raised];
    set wf [frame $w.wf -bd 1 -relief raised];
    set tf [frame $w.tf -bd 1 -relief raised];
    set lf [frame $w.lf -bd 1 -relief raised];
    set okf [frame $w.okf -bd 1 -relief raised];
    pack $title $cf $wf $tf $lf $okf -side top -expand 1 -fill x

    # set up the label frame
    if {$custopts(label) != -1} {
	set lshow_txt [lindex $attr_names $custopts(label)];
	set lshow_txt "Labeled by <$lshow_txt>";
    } else {
	set lshow_txt "None"
    }
    set lbtn [menubutton $lf.mb -text "Choose Label" -menu $lf.mb.m \
	    -bd 2 -relief raised];
    set lshow [label $lf.show -text "$lshow_txt" -bg white];
    pack $lbtn $lshow -side left -expand 1 -fill x -padx 0.5m \
	    -pady 5
    menu $lbtn.m;
    set msg "No Label";
    set len [string length $msg];
    $lbtn.m add command -label "None" -command \
	    "set custopts(label) -1; \
	    $lshow configure -text \"$msg\" -width $len";
    set i 0;
    foreach attrn $attr_names {
	set msg "Labeled by <$attrn>";
	set len [string length $msg];
	$lbtn.m add command -label "$attrn" -command \
		"set custopts(label) $i; \
		$lshow configure -text \"$msg\" -width $len";
	incr i;
    }

    # set up transparency checkbutton
    label $tf.label -text "Transparency: "
    set ttb [tixSelect $tf.sel \
	    -font "-adobe-helvetica-bold-r-normal-*-12-*-*-*-*-*-*-*"]
    $tf.sel add transparent -text "Transparent" -width 12
    $tf.sel add opaque -text "Opaque" -width 12
    if $custopts(trans) {
	$tf.sel config -value {transparent}
    } {
	$tf.sel config -value {opaque}
    }
    $tf.sel config -command {setTransparent}
    pack $tf.label -side left -expand yes -fill x -pady 5
    pack $ttb -side left -pady 5
    
    # set up the color frame
    set cbtn [menubutton $cf.mb -text "Choose Outline Color" \
	    -menu $cf.mb.m\
	    -bd 2 -relief raised];
    set csample [label $cf.show -text $color_sample -bg $bgcolr\
	    -bd 2 -relief sunken];
    pack $cbtn $csample -side left -expand 1 -fill x -padx 0.5m \
	    -pady 5

    menu $cbtn.m;
    $cbtn.m add cascade -label "Color Palette" -menu $cbtn.m.c;
    if {$intAttrsIndxs != {}} {
	$cbtn.m add cascade -label "Integer Attributes" \
		-menu $cbtn.m.i;
	menu $cbtn.m.i;
	foreach i $intAttrsIndxs {
	    set attrn [lindex $attr_names $i];
	    set msg "Colored by <$attrn>"; 
	    set len [string length $msg]; 
	    $cbtn.m.i add command -label "$attrn" -command \
		    "set custopts(color) $i; \
		    $csample configure -text \"$msg\" -width $len \
		    -background white";
	}
    }

    global col_palete;
    set msg "                           ";
    set len [string length $msg]; 
    menu $cbtn.m.c;
    foreach color_cell $col_palete {
	$cbtn.m.c add command -label "$color_cell" \
		-background $color_cell -command \
		"set custopts(color) $color_cell; \
		$csample configure -text \"$msg\" -width $len \
		-background $color_cell";
    }

    # set up the width frame
    set wlabel [label $wf.l -text "Line Width"];
    set wctrl [tixControl $wf.ctl -entrywidth 8 -value $custopts(width)\
	    -step 1	-llimit 0 -ulimit 16]
    $wctrl config -command {setLineWidth}
    pack $wlabel $wctrl -side left -expand yes -fill x -padx 0.5m -pady 5

    # set up the ok button
    set okbtn [button $okf.ok -text "OK" -command \
	    "configDone $sid $attrindx $w"];
    pack $okbtn -expand 1 -fill x -pady 0.5m -pady 5

    grab $w;
    tkwait window $w;
}

# store the changes made by configuration of a particular attribute
# into session variable so that it can be stored back to layer data
# structure.
proc configDone {sid attrindx pw} {
    global session custopts;

    # scr_change flag may eventually be used to do fast
    # update on screen, without even scanning the temporary
    # file.
    set scr_change 0;
  if {[lindex $session($sid,curcol) $attrindx] != $custopts(color)} {
	set session($sid,curcol) [lreplace $session($sid,curcol) \
		$attrindx $attrindx $custopts(color)];
	set scr_change 1;
    }
    if {[lindex $session($sid,curwidth) $attrindx] != $custopts(width)} {
	set session($sid,curwidth) [lreplace $session($sid,curwidth) \
		$attrindx $attrindx $custopts(width)];
	set scr_change 1;
    }
    if {[lindex $session($sid,curbitmap) $attrindx] != $custopts(bmap)} {
	set session($sid,curbitmap) [lreplace \
		$session($sid,curbitmap) \
		$attrindx $attrindx $custopts(bmap)];
	set scr_change 1;
    }
    if {[lindex $session($sid,is_trans) $attrindx] != $custopts(trans)} {
	set session($sid,is_trans) [lreplace $session($sid,is_trans) \
		$attrindx $attrindx $custopts(trans)];
	set scr_change 1;
    }
    if {[lindex $session($sid,attr_label) $attrindx] != $custopts(label)} {
	set session($sid,attr_label) [lreplace \
		$session($sid,attr_label) \
		$attrindx $attrindx $custopts(label)];
	set scr_change 1;
    }

    destroy $pw;
}

# if a query has spatial attribute, add overlap attribute if
# this hasn't been done already, also, need to add where if
# is isn't there.
proc bindQueryToScr {query attrns attr_typestrs} {
    upvar #0 debugMode mode
    set sufix ""

    if [regexp {##PDMAP##} "$query"] {
	if $mode {
	    puts "<$query> is already binded to screen";
	}
	return $sufix;
    }

    set where_key "";
    if [regexp {where} $query] {
	if $mode {
	    puts "<$query> already has where clause";
	}
	set hasWhere 1;
    } else {
	set hasWhere 0;
	append where_key " where ";
    }

    set i 0;
    set first_pred 1;
    set pred "";
    foreach tstr $attr_typestrs {
	set id [ptkTypename2id $tstr];
	if {[isAttrDispType $id] && ![isAttrRasterAdt $id]} {
	    set attrname [lindex $attrns $i];
	    # check to see if attribute is function or not
	    if {![isAttrFunction $attrname] && ![isAttrAggregate $attrname]} {
		if {$hasWhere || !$first_pred} {
		    append pred \
			    " and $attrname overlaps ##PDMAP##";
		} else {
		    append pred " $attrname overlaps ##PDMAP##";
		    set first_pred 0;
		}
	    }
	}
	incr i;
    }
    if {$pred == ""} {
	set where_key "";
    }
    append sufix "$where_key $pred";
    return $sufix
}

proc appendLevel {query sid} {
    global debugMode
    global session
    
    set sufix ""
 	if { [regexp {=##HEIGHT##} "$query"] \
	     || [regexp {=##WIDTH##} "$query"] } {
		if $debugMode {
	    	puts "<$query> is already binded to screen";
		}
		return $sufix;
    	}

	if [regexp "__PTmpF_" "$query"] { 
		if $debugMode {
	    	puts "Select from ParaTemFile, can not append zoom level."
		}
		return $sufix 
	}

	set desc $session($sid,tmpdesc)
	set attrns [getAttrNamesFromDesc $desc]

	set attr_typestrs [getAttrTypesFromDesc $desc]

	set q $query

	set where_key ""

     	if [regexp {where} $query] {
        	set hasWhere 1;
    	} else {
        	set hasWhere 0;
        	append where_key " where ";
    	}
	if { [regexp {##HEIGHT##} $query] || [regexp {##WIDTH##} $query] } {
		set isfirst 0
	} else {
		set isfirst 1
	}

    set i 0;
    set first_pred 1;
    set pred "";
    foreach tstr $attr_typestrs {
        set id [ptkTypename2id $tstr];
        if {[isAttrDispType $id] && ![isAttrRasterAdt $id]} {
	    if {[isAttrHasMbrType $id]} {

            set attrname [lindex $attrns $i];
            # check to see if attribute is function or not
            if {![isAttrFunction $attrname] && ![isAttrAggregate $attrname]} {
                if {$hasWhere || !$first_pred} {
		    if $isfirst {
                    	append pred \
                            " and ( ($attrname.mbr().height()=##HEIGHT## or \
				$attrname.mbr().width()=##WIDTH##)";
			set isfirst 0;
		    } else {
			append pred \
                            " or ($attrname.mbr().height()=##HEIGHT## or \
				$attrname.mbr().width()=##WIDTH##)";
		    }
                } else {
		    if $isfirst {
                    	append pred "( ($attrname.mbr().height()=##HEIGHT## \
				or $attrname.mbr().width()=##WIDTH##)";
		    	set isfirst 0;
                    	set first_pred 0;
		    } else {
			append pred "or ($attrname.mbr().height()=##HEIGHT## \
				or $attrname.mbr().width()=##WIDTH##)";
		    }
                }
            }
  	   }
        }
        incr i;
    }
    if {$pred != ""} { 
	append pred " )" 
    }
    if {$pred == ""} {
        set where_key "";
    }

    append sufix "$where_key$pred" 
    return $sufix
}

proc isAttrFunction {attrn} {
    return [regexp {\^func$} $attrn]
}

proc isAttrAggregate {attrn} {
    return [regexp {\^aggr$} $attrn]
}

