# <std-header style=TCL orig-src=PTK>
#
#            Copyright 1998, NCR Corporation.  All Rights Reserved.
#
#                                NCR Confidential.
#
#
#  $Id: session.tcl,v 1.8 1998/05/19 03:22:33 kupsch Exp $
#
#  This file was originally part of ptk, a front end based on TCL/TK to the
#  Paradise database system developed by the University of Wisconsin-Madison.
#  The above copyright applies to changes made to this file by NCR Corporation.
#  The original copyright which may appear in the source below applies to the
#  original file as obtained by NCR Corporation and unchanged parts of this file.
#
#
#   -- do not edit anything above this line --   </std-header>

# This file contains the procedures to manipulate the session specifice
# data structures.  Most procedures take session id as argument.
#

# setLayers will set the value of session sid's layer to the argument
# layer.
#
# Argument:
# 	+ sid		session id;
#	+ layer		the value we want to set to;
#
# Side Effect: global variable session($sid,layers) will be set to the
# 	value of layer.
#
proc setLayers {sid, layer} {
	global session
	set session($sid,layers) $layer
}

proc setHiddenLayers {sid, hiddenLayer} {
	global session
	set session($sid,hidden_layers) $hiddenLayer
}

# getLayers will return a list which is the contents of the layer
# data structure of the given session
#
# Argument:
#	+ sid 		session id;
#
# Return value: a list
#
proc getLayers sid {
	global session
	return $session($sid,layers)
}

proc getHiddenLayers sid {
	global session;
	return $session($sid,hidden_layers)
}

# pops up the session form, requesting user to choose
# a database for the session

proc fillSessionForm sid {
	global session NewSession_Cancel

	set NewSession_Cancel 0
	set w ".session_form";
	toplevel $w;
	wm title $w "New Session Information";
	wm iconname $w "Paradise New Session";
	wm geometry $w +260+200;

	set topf [frame $w.top -bd 1 -relief raised];
	set namef [frame $w.namef -bd 1 -relief raised];
	set dbf [frame $w.dbf -bd 1 -relief raised];
	set btnf [frame $w.btnf -bd 1 -relief raised];
	pack $topf $namef $dbf $btnf -side top -fill both;

	# fill the top part with message
	message $topf.msg -width 5i -text "Create A New Paradise Session" \
		-font -Adobe-Times-Medium-R-Normal-*-180-*;
	pack $topf.msg -expand 1 -fill both -pady 8m;

	set namelbl [label $namef.lbl -text "Session Name:"];
	set nameentry [entry $namef.entry -relief sunken -width 16 \
		-textvariable session($sid,sname) -bg white -fg blue];
	pack $namelbl $nameentry -side left -padx 1m -expand 1 -fill x;

	set dbflabel [label $dbf.lbl -text "Choose a database:"];
	set dbfmb [menubutton $dbf.mb -text $session($sid,curdb) \
		-textvariable session($sid,curdb) -bd 1 \
		-menu $dbf.mb.m -relief raised -width 16];
	set dbfmbm [menu $dbf.mb.m];
	pack $dbflabel $dbfmb -side left -padx 1m -expand 1 -fill x;

	set okbtnf [frame $btnf.okf -bd 1 -relief sunken]
	set okbtn [button $btnf.okf.ok -text "Ok" -command \
		"fillSessionFormOk $sid $w"];
	pack $okbtn -padx 3 -pady 3 -ipadx 3 -ipady 1 -expand 1 -fill x;

	set helpbtn [button $btnf.help -text "Help" -state disabled];
	set helpbtn [button $btnf.cancel -text "Cancel" -command \
		"destroy $w; \
		 set NewSession_Cancel 1; \
		 return"];
	set quitbtn [button $btnf.quit -text "Quit Now" -command exit];
	pack $okbtnf $helpbtn $quitbtn -padx 3 -side left -expand 1 -fill x \
		-pady 1m;

	session_fillDbMenu $sid $dbfmbm;

	# bind <Return> to OK button
	bind $nameentry <Return> "$okbtn flash; $okbtn invoke";
	grab $nameentry;
	focus $nameentry;

	grab $w;
	tkwait window $w;
}

proc session_fillDbMenu {sid m} {
  global session

  set dbnames [PDgetdbnames];
  $m delete 0 last;

  if {$dbnames == {}} {
	pd_dialog .error {Error} {No existing database found to \
		work with.  Can not continue.} {} error 0 {OK};
	exit;
  }

  foreach i $dbnames {
    $m add radiobutton -label $i -variable session($sid,curdb) -value $i;
  }

  # set the default dbname to the first database available
  set session($sid,curdb) [lindex $dbnames 0];
}

proc fillSessionFormOk {sid w} {
	global session;

	if {[string trim $session($sid,sname)] == {}} {
	    set session($sid,sname) "NONAME"
	}

	# else we can open the current database and fill metacache
	PDsql_cursorrun "open db $session($sid,curdb)";
	fillMetaCache;
	destroy $w;
}	


