# <std-header style=TCL orig-src=PTK>
#
#            Copyright 1998, NCR Corporation.  All Rights Reserved.
#
#                                NCR Confidential.
#
#
#  $Id: showTab.tcl,v 1.27 1998/05/19 03:22:33 kupsch Exp $
#
#  This file was originally part of ptk, a front end based on TCL/TK to the
#  Paradise database system developed by the University of Wisconsin-Madison.
#  The above copyright applies to changes made to this file by NCR Corporation.
#  The original copyright which may appear in the source below applies to the
#  original file as obtained by NCR Corporation and unchanged parts of this file.
#
#
#   -- do not edit anything above this line --   </std-header>

# This file contains the tk code to support "show table"
#
# the following are the global variables used in this file only
# + showTab_attrnames, showTab_attrtypes
# + pixperchar, xgap, ygap, tuple_num, showTab_attrwd
#

set showTab_attrnames {}
set showTab_attrtypes {}
set showTab_attrwd {}

# A local variable pixperchar is maintained to convert char
# to number of pixels.
#
set pixperchar 10;
set xgap 1.5m;
set ygap 0;

# a global counter to keep track of the tuple number of the
# current tuple we are processing, used to create a unique
# frame name path for that tuple;
set tuple_num 0;

# use both extName and session id as argument
# but sid is not used yet
proc showTab {extName sid} {
    global showTab_attrnames showTab_attrtypes
    global debugMode session
    if {$debugMode == 1} { puts "Show table: $extName" }

    # begin processing, set cursor to watch
    ptkPDWait ON
    
    # set up the canvas.  Note that we don't know the scroll region
    # now because we don't now how many tuples we are going to have
    # yet.

    # if there is an existing query, use it, otherwise,
    # scan the extent and show tuples
    if {$session($sid,curquery) == "NONE"} {
	set q "select * from $extName"
    } else {
	set q $session($sid,curquery)
	set sqlBox [makePDBBox  \
		$session($sid,bx1) $session($sid,by1) \
		$session($sid,bx2) $session($sid,by2)]
	regsub -all ##PDMAP## $q $sqlBox q
    }

    set rc [catch {PDsql_cursoropen $q} err]
    if $rc {
	# error occurred, do some cleanups and return
	ptkPDWait OFF
	tk_dialog .error {Query Processing Error} "$err" error 0 {Ok}
	return
    }
    # if { $err == "NO_TUPLE" } { return }

    # get result extent schema
    set desc [PDsql_cursorschema]

    set showTab_attrnames [getAttrNamesFromDesc $desc]
    set showTab_attrtypes [getAttrTypesFromDesc $desc]
    set tmpfn [getTmpfnFromDesc $desc]
    set nattrs [llength $showTab_attrnames]
    set showTab_attrwd {}

    if $debugMode {
	puts "\tdesc = $desc"
	puts "\tshowTab_attrnames = $showTab_attrnames"
	puts "\tshowTab_attrtypes = $showTab_attrtypes"
	puts "\ttmpfn $tmpfn"
    }

    set cell_high 26

    set w ".shext$extName"
    catch "destroy $w"
    toplevel $w
    wm title $w "Show Table <$extName>"
    wm geometry $w +200+0
    wm minsize $w 200 100
    

    # Use call by reference
    extsh_calc $showTab_attrtypes $nattrs 20 scrlx scrly showTab_attrwd;
    frame $w.top
    ptkTable $w.top.tab $showTab_attrwd $cell_high $showTab_attrtypes \
	    $tmpfn
    pack $w.top.tab -expand yes -fill both

    button $w.ok -text Dismiss -bd 2 -relief raised -command "destroy $w";
    pack append $w $w.top {top expand fill} \
	    $w.ok {expand fillx padx 3m pady 3m}

    # show title
    shtitle $showTab_attrnames $w.top.tab;

    # title is row #1, so tuples start with 2
    set row_num 2
    
    set tuple_val [PDsql_cursorfetch]

    while {$tuple_val != "" && $row_num <= 300} {
	ptkTable::insertRow $w.top.tab $row_num $tuple_val 
	set tuple_val [PDsql_cursorfetch]
	incr row_num
    }
    if {$tuple_val != ""} {
	tk_dialog .showTabWarning {Warning} {Table too big, only \
		show the first 300 rows!} warning 0 OK
    }

    ptkTable::insertDone $w.top.tab [expr $row_num - 1]
    
    # end processing, set cursor back
    ptkPDWait OFF

    # make this a modal dialog
    set old_grab [grab current]
    grab set $w
    tkwait window $w
    if [is_fname_tmp $tmpfn] {
	# PDsql_cursorclose
	PDsql_cursorrun "drop table $tmpfn"
    }
    if {$old_grab != ""} {grab set $old_grab}
}	

#
# getAttrNamesFromDesc: return a list of attribute names for the given
# extent description 
#
proc getAttrNamesFromDesc desc {
    global debugMode

    if $debugMode {
	puts "getAttrNamesFromDesc $desc"
    }
    set attrnames {}
    foreach i $desc {
	# attrn in forms of attrname_tablename
	# transform into tablename.attrname
	set attrn [lindex $i 0]
	set idx [string last "~" $attrn]
	if {$idx != -1} {
	    set an [string range $attrn 0 [expr $idx - 1]]
	    set tn [string range $attrn [expr $idx + 1] end]
	} else {
	    set an $attrn
	    set tn {}
	}

	# this is complicated, table name can have more than
	# one $ in it, and we can chop off the remaining ones
	set idx [string first {$} $tn]
	if { $idx != -1 } {
 	    incr idx -1
	    set tn [string range $tn 0 $idx]
	}

	if {$tn != {}} {
	    # clean the table name (get rid of %[0-9]+)
	    regsub -all {%([0-9])+} $tn {} tn
	    set attrn "$tn.$an"
	} else {
	    set attrn $an
	}
	lappend attrnames $attrn
    }
    return $attrnames
}

#
# getAttrTypesFromDesc: return a list of attribute types for the given extent
# Since attrinfo gets its types from PD metadata infterface, they are
# type ids.  Here, we want them to be in terms of string, so a conversion
# is done.
proc getAttrTypesFromDesc desc {
    set attrtypes {}
    foreach i $desc {
	lappend attrtypes [ptkTypeid2name [lindex $i 1]]
    }
    return $attrtypes
}

proc getTmpfnFromDesc desc {
    return [lindex [lindex $desc 0] 2]
}

#
# invoked to show the title of each field
#
proc shtitle {attrnames tabwin} {
    ptkTable::insertRow $tabwin 1 $attrnames -fill red
}

#
# Invoked to show a tuple in the table
# Takes two arguments: a list of tuple values and the window
# path name.
#
proc showTuple {tval showTab_attrwd w} {
    global tuple_num xgap ygap
    global debugMode

    set tn "tp$tuple_num";
    incr tuple_num 1;

    # some local variables
    set bgcolor "white";
    set bdwidth 2;

    # create a unique frame out of current tuple number
    frame $w.$tn -bd 1 -relief raised;
    pack $w.$tn -side top -fill x;
    set j 0;
    foreach fval $tval {
	label $w.$tn.$j -text $fval -anchor w -width \
		[lindex $showTab_attrwd $j] -bg $bgcolor -bd $bdwidth \
		-padx $xgap -pady $ygap -relief raised;
	pack $w.$tn.$j -side left;
	incr j 1;
    }
}

#
# invoked to calculate the scroll region in x dimension
# an y dimension.  We maintain an array of ptk_typewd.
# eventually, we should put it in Paradise extent.
# This proc is call by reference, so there is no return
# value.
#
proc extsh_calc {attrtypes nattrs ntuple scrlx scrly attrwd} {
    upvar $scrlx sx $scrly sy $attrwd attr_wd
    global pixperchar ptk_typewd;

    # estimate 20 pixel overhead for each label in x dimension
    # estimate 10 pixel overhead for each label in y dimention
    # try to overestimate so that scroll region will be bigger
    # than the actual region
    set xest 20;
    set yest 10;
    
    set sx 0;
    foreach i $attrtypes {
	set sx [expr $sx + $ptk_typewd($i)];
	lappend attr_wd $ptk_typewd($i);
    }
    set sx [expr $sx * $pixperchar + $xest * $nattrs];
    set sy [expr $pixperchar * ( 1 + $yest ) * $ntuple];
}

