# <std-header style=TCL orig-src=PTK>
#
#            Copyright 1998, NCR Corporation.  All Rights Reserved.
#
#                                NCR Confidential.
#
#
#  $Id: table.tcl,v 1.12 1998/05/19 03:22:34 kupsch Exp $
#
#  This file was originally part of ptk, a front end based on TCL/TK to the
#  Paradise database system developed by the University of Wisconsin-Madison.
#  The above copyright applies to changes made to this file by NCR Corporation.
#  The original copyright which may appear in the source below applies to the
#  original file as obtained by NCR Corporation and unchanged parts of this file.
#
#
#   -- do not edit anything above this line --   </std-header>

# This file implements a simple table widget to tabulate the contents
# of an extent.
#
# Usage: call "ptkTable path cell_wide cell_high" to create a
# table widget with the given path.  Then we can use method:
# ptkTable::insertRow(path, row, rowVal) to insert a row into
# this table.  The string is subject to truncation to fit the width
# of the cell.
# Call ptkTable::insertDone(path) to draw all the vertical lines.
#
# The importance of this widget is that you don't have to know exactly
# how many rows you will have, just keep adding rows to the table will
# be fine.

# width is measured in units of characters, height is measured in pixels
# ptkTable_cell_widt_pix is the array of width in pixels just to make
# the code run faster.  Each character is assumed to be 14 pixels wide
set ptkTable_cell_wide {}
set ptkTable_cell_wide_pix {}
set ptkTable_cell_high 32
set ptkTable_top_margin 5
set ptkTable_xmax 0
set ptkTable_cell_intv_pix 7
set ptkTable_attr_types {}
set ptkTable_tmpfn ""
set ptkTable_font "7x13bold";

proc ptkTable {w cell_wide cell_high attr_types tmpfn} {
    # set ptkTable internal variables to initial state
    global ptkTable_cell_wide ptkTable_cell_wide_pix ptkTable_cell_high
    global ptkTable_xmax ptkTable_cell_intv_pix
    global ptkTable_attr_types ptkTable_tmpfn
    global debugMode
    if {$debugMode == 1} {
	puts "field width: <$cell_wide>"
    }
    set ptkTable_cell_wide $cell_wide
    set ptkTable_cell_wide_pix {}
    set ptkTable_xmax 0
    set intv $ptkTable_cell_intv_pix
    set ptkTable_tmpfn $tmpfn
    set ptkTable_attr_types $attr_types
    set px 7; set py 32
    foreach i $ptkTable_cell_wide {
	lappend ptkTable_cell_wide_pix [expr $i * $px + $intv]
	incr ptkTable_xmax [expr $i * $px + $intv]
    }
    assert {[llength $ptkTable_cell_wide_pix] == [llength $cell_wide]}
    if [lsearch -exact $attr_types "Video"] {
	set ptkTable_cell_high 42
    } elseif [lsearch -exact $attr_types "Text"] {
        set ptkTable_cell_high 42
    } elseif [lsearch -exact $attr_types "Gif"] {
        set ptkTable_cell_high 42
    } else {
	set ptkTable_cell_high $cell_high
    }
    
    frame $w
    pack $w -expand yes -fill both

    frame $w.bot
    frame $w.top
    pack $w.top $w.bot -side top -expand yes -fill both

    scrollbar $w.top.sy -command "$w.top.c yview"
    scrollbar $w.top.sx -orient horiz -command "$w.top.c xview"
    canvas $w.top.c -bd 2 -relief sunken -height 700 -width 600 
    pack append $w.top  $w.top.sx {expand bottom fillx} $w.top.sy \
	    {expand right filly}
    pack $w.top.c -expand yes -fill both

}

# Note: the first is number 1, not 0!
proc ptkTable::insertRow {w row rowVal args} {
    global ptkTable_cell_high ptkTable_cell_wide ptkTable_xmax
    global ptkTable_cell_wide_pix ptkTable_cell_intv_pix
    global ptkTable_top_margin
    global ptkTable_attr_types ptkTable_tmpfn ptkTable_font
    global bitmap_dir
    
    set w "$w.top.c"
    set y [expr $row * $ptkTable_cell_high + $ptkTable_top_margin]
    $w create line 0 $y $ptkTable_xmax $y

    # display the row one col at a time
    set indx 0; set tx $ptkTable_cell_intv_pix; 
    set ty [expr $y - $ptkTable_cell_high / 2]
    set oidindx [expr [llength $rowVal] - 1]
    foreach val $rowVal {
	# cut the length if necessary
	set wide_char [lindex $ptkTable_cell_wide $indx]
	set len [string length $val]
	if { $len > $wide_char} {
	    set val [string range $val 0 [expr $wide_char - 3]]
	    set val "$val..."
	}

	# check if it is video object and not title row (1)
	if {[lindex $ptkTable_attr_types $indx] == "Video" &&\
		$row > 1} {
	    # if so, create a button instead
	    set btn [format "%s.b_%s_%s" $w $row $indx]
	    set oid [lindex $rowVal $oidindx]
	    button $btn -bitmap "$bitmap_dir/movie_camera.xbm" \
		    -fg blue -command \
		    "ptk_showvid $ptkTable_tmpfn $oid $indx" 
	    eval [concat $w create window $tx $ty -window $btn \
		    -anchor w]
        } elseif {[lindex $ptkTable_attr_types $indx] == "Text" &&\
		$row > 1} {
	    # if so, create a button instead
	    set btn [format "%s.b_%s_%s" $w $row $indx]
	    set oid [lindex $rowVal $oidindx]
	    button $btn -bitmap "$bitmap_dir/text_icon.xbm" \
		    -fg blue -command \
		    "ptk_showtext $ptkTable_tmpfn $oid $indx" 
	    eval [concat $w create window $tx $ty -window $btn \
		    -anchor w]
        } elseif {[lindex $ptkTable_attr_types $indx] == "Gif" &&\
		$row > 1} {
	    # if so, create a button instead
	    set btn [format "%s.b_%s_%s" $w $row $indx]
	    set oid [lindex $rowVal $oidindx]
	    button $btn -bitmap "$bitmap_dir/camera.xbm" \
		    -fg blue -command \
		    "ptk_showgif $ptkTable_tmpfn $oid $indx" 
	    eval [concat $w create window $tx $ty -window $btn \
		    -anchor w]
	} else {
	    eval [concat $w create text $tx $ty -text \"$val\" \
		    -anchor w $args -font $ptkTable_font]
	}
	incr tx [lindex $ptkTable_cell_wide_pix $indx]
	incr indx
    }
}

proc ptk_showtext {tmpfn oid indx} {
  global ParserOidAttributeName
  set q "select * from $tmpfn where $ParserOidAttributeName = \"$oid\""
  PDpop_text $q $indx 
}

proc ptk_showgif {tmpfn oid indx} {
  global ParserOidAttributeName
  set q "select * from $tmpfn where $ParserOidAttributeName = \"$oid\""
  PDpop_gif $q $indx
}



# Finally, we the number of rows in our table
proc ptkTable::insertDone {w nrows} {
    global ptkTable_cell_wide_pix ptkTable_cell_high
    global ptkTable_xmax
    set x 0; set ymax [expr $nrows * $ptkTable_cell_high + 20]
    foreach cw $ptkTable_cell_wide_pix {
	incr x $cw
	$w.top.c create line $x 0 $x $ymax
    }
    $w.top.c config -scrollregion "0 0 $ptkTable_xmax $ymax" \
	    -xscroll "$w.top.sx set" -yscroll "$w.top.sy set"
}

