# <std-header style=TCL orig-src=PTK>
#
#            Copyright 1998, NCR Corporation.  All Rights Reserved.
#
#                                NCR Confidential.
#
#
#  $Id: tupleDlg.tcl,v 1.17 1998/05/19 03:22:35 kupsch Exp $
#
#  This file was originally part of ptk, a front end based on TCL/TK to the
#  Paradise database system developed by the University of Wisconsin-Madison.
#  The above copyright applies to changes made to this file by NCR Corporation.
#  The original copyright which may appear in the source below applies to the
#  original file as obtained by NCR Corporation and unchanged parts of this file.
#
#
#   -- do not edit anything above this line --   </std-header>

# This implements the tuple dialog that pops up to show the
# tuple corresponds to the object clicked by user on map.
#
# Currently, this is not a mono dialog, so we can have many
# of such dialogs on screen.
#
# The argument is $tuple and $desc and $sid and $tags.
# Where $desc is a list of {attrname typeid extname} triples.
#
set ptk_tupleDlg_tcl(attrn_wd) 16
set ptk_tupleDlg_tcl(attrt_wd) 18
set ptk_tupleDlg_tcl(wcnt) 0

proc ptk_tpview {sid tuple desc tags canvwin query baseQuery updatable} {
    global ptk_typewd ptk_tupleDlg_tcl debugMode bitmap_dir attr_vals
    global numMod modIdx newTuple queries

    if $debugMode {
	puts "ptk_tpview"
    }
    set attrns [getAttrNamesFromDesc $desc]
    set type_strs [getAttrTypesFromDesc $desc]

    set attrwd {}
    foreach i $type_strs {
	lappend attrwd $ptk_typewd($i)
    }
	
    set nattrs [llength $attrns]
    set queries($ptk_tupleDlg_tcl(wcnt)) $query
    set w ".ptk_tpview$ptk_tupleDlg_tcl(wcnt)"
    catch {destroy $w}
    toplevel $w
    wm title $w "Tuple Dialog"
    incr ptk_tupleDlg_tcl(wcnt)

    set windex [expr $ptk_tupleDlg_tcl(wcnt)-1]
    set tpframe [frame $w.tpframe$sid -bd 0.8m -relief groove];
    set msgFrame [frame $w.msgframe$sid -bd 0.5m -relief raised];
    set msgwin [message $msgFrame.msg -bg gray80 -fg black -width 360];
    pack $msgwin -expand 1 -fill x;
    
    pack $tpframe -side top -padx 3 -pady 3
    set tablist {}
    for {set i 0} {$i < [llength $attrns]} {incr i} {
	set attr_name [lindex $attrns $i]
	set attr_type [lindex $type_strs $i]
	set attr_val  [lindex $tuple $i]
	set attr_vals($windex,$i) $attr_val
	set valwd $ptk_typewd($attr_type)

	set attrframe [frame $tpframe.attr$i]
	label $attrframe.an$i -width $ptk_tupleDlg_tcl(attrn_wd) \
		-anchor w -text "$attr_name" -bd 1 -relief raised
        if {$attr_type == "Text"} {
            button $attrframe.at$i -width $ptk_tupleDlg_tcl(attrt_wd) \
                    -anchor w -text "show text" \
                    -bd 0.5m -relief raised
            entry $attrframe.av$i -width $valwd \
                    -textvariable attr_vals($windex,$i) -bd 3 \
                        -relief sunken -fg blue -bg white

            lappend tablist $attrframe.av$i
        } elseif {$attr_type == "Video"} {
	    button $attrframe.at$i -width $ptk_tupleDlg_tcl(attrt_wd) \
		    -anchor w -text "play me" \
		    -bd 0.5m -relief raised
	    entry $attrframe.av$i -width $valwd \
		    -textvariable attr_vals($windex,$i) -bd 3 \
			-relief sunken -fg blue -bg white 
		    
	    lappend tablist $attrframe.av$i
	} else {
	    if [regexp {oid} $attr_name] {
		button $attrframe.at$i -width $ptk_tupleDlg_tcl(attrt_wd) \
			-anchor w -text "($attr_type)" \
			-bd 0.5m -relief raised -state disabled
	    	entry  $attrframe.av$i -width $valwd \
		    	-textvariable attr_vals($windex,$i) \
			-bd 3 -relief sunken \
		    	-fg blue -bg white -state disabled
	    } else {
	
	    	if [isAttrDispType [ptkTypename2id $attr_type]] {
		button $attrframe.at$i -width $ptk_tupleDlg_tcl(attrt_wd) \
			-anchor w -text "($attr_type)" \
			-bd 0.5m -relief raised
	    	} else {
		button $attrframe.at$i -width $ptk_tupleDlg_tcl(attrt_wd) \
			-anchor w -text "($attr_type)" \
			-bd 0.5m -relief raised -state disabled
	    	}
	    	if {[string length $attr_val] > $valwd} {
		set attr_val [string range $attr_val 0 \
			[expr $valwd - 6]]
		append attr_val " ... "
	    	}
	    	entry  $attrframe.av$i -width $valwd \
		    -textvariable attr_vals($windex,$i) \
			-bd 3 -relief sunken -fg blue -bg white 
	        lappend tablist $attrframe.av$i

	    }
	    lappend newtuple $attr_vals($windex,$i)
	}

	label $attrframe.sep$i -width 3 -text ":"
	pack $attrframe.an$i $attrframe.at$i $attrframe.sep$i \
		-side left 
	pack $attrframe.av$i -side left -expand 1 -fill x;
	pack $attrframe -side top -expand 1 -fill x;

	if {[isAttrDispType [ptkTypename2id $attr_type]]} {
	    bind $attrframe.at$i <Enter> \
		    "+$msgwin configure -text \"Click to configure it\"";
	    bind $attrframe.at$i <Leave> \
		    "+$msgwin configure -text \"   \"";
	    bind $attrframe.at$i <1> \
		    "+attrConfig $sid $i [list $tags] [list $type_strs] [list $attrns] $attrframe.at$i $canvwin"
	} else {
	    if  {$attr_type == "Video"} {
	    bind $attrframe.at$i <Enter> \
		    "+$msgwin configure -text \"Click to play video\"";
	    bind $attrframe.at$i <Leave> \
		    "+$msgwin configure -text \"   \"";
	    bind $attrframe.at$i <1> \
		    "+attrConfig $sid $i [list $tags] [list $type_strs] [list $attrns] $attrframe.at$i $canvwin"
	    } elseif {$attr_type == "Text"} { 
            bind $attrframe.at$i <Enter> \
                    "+$msgwin configure -text \"Click to show text\"";
            bind $attrframe.at$i <Leave> \
                    "+$msgwin configure -text \"   \"";
            bind $attrframe.at$i <1> \
                    "+attrConfig $sid $i [list $tags] [list $type_strs] [list $attrns] $attrframe.at$i $canvwin"
            }
	}
    }

    foreach entry $tablist {
	bind $entry <Tab> " tab {$tablist} "
    }
    set btframe [frame $w.btframe$sid]

    set numMod($windex) 0
    set newTuple($windex) {}
    set modIdx($windex) {}

    set udbtn [button $btframe.udbtn -width 9 \
	   -text "Update" -command " \
				 ptkPDWait ON; \
				 Update [llength $attrns] {$queries($windex)} \
				     $sid {$tuple} {$type_strs} {$baseQuery};\
				 destroy $w; \
				 ptkPDWait OFF;"]

    set okbtn [button $btframe.okbtn -width 9 \
	    -text "Dismiss" -command "destroy $w"]
 
    if {$updatable == "update"} {
	pack $udbtn $okbtn -side left -padx 3 -pady 0.5m -expand 1 -fill x
    } else {
    	pack $okbtn -side left -padx 3 -pady 0.5m -expand 1 -fill x
    }

    pack $btframe -side top -pady 1m;
    pack $msgFrame -side top -pady .3m -expand 1 -fill x;

    bind $okbtn <Enter> \
	    "+$msgwin configure -text \"Click to dismiss\"";
    bind $okbtn <Leave> \
	    "+$msgwin configure -text \"    \"";
    bind $okbtn <1> "+destroy $w";
    
}

proc tab { list } {
    set i [lsearch -exact $list [focus]]
    incr i
    if {$i >= [llength $list]} {
	set i 0
    }
    focus [lindex $list $i]
}

proc Update {length query sid oldtuple attrType baseQuery} {
    global session attr_vals numMod newTuple modIdx  
    global ptk_tupleDlg_tcl LAYER_BASETABLE

    set windex [expr $ptk_tupleDlg_tcl(wcnt)-1]
    for {set i 0} {$i < $length} {incr i} {
	set oldfield [lindex $oldtuple $i]
	if {![regexp "^$attr_vals($windex,$i)$" "$oldfield"]} {
	    lappend modIdx($windex) $i
	    incr numMod($windex)
	}	
	if { [string index $attr_vals($windex,$i) 0]=="(" } {
	    set ptlist [split [string trim $attr_vals($windex,$i) ()] ,]
	    for { set j 0 } {$j < [expr [llength $ptlist]/2]} { incr j } {
		append ptstring "[lindex $ptlist [expr $j*2]] "
		append ptstring "[lindex $ptlist [expr $j*2+1]],"
   	    }
 	    set ptstring [string trimright $ptstring ,]
	    set newfield "$ptstring"
	    lappend newTuple($windex) $newfield
	} else {
	    lappend newTuple($windex) $attr_vals($windex,$i)
	}
    }

    UpdateTuple $query $newTuple($windex) $modIdx($windex) $attrType $baseQuery
}

