
#include <all_global.h>
//#include <badd_error.h>
//#include <cplusplusApi.h>


//----------------------------------------------------------------------------------------
//  Global variables needed by smartPush and smartPull
//----------------------------------------------------------------------------------------
int PARADISE_PORT               = 9000;                          /* default paradise port        */
const char * PARADISE_PORT_FILE = ".paradise.port.number/";      /* paradise port file name      */
const char * IDS_DB_NAME        = "BADD_DB";                     /* The IDS db name              */
const char * CLIENT_DB_NAME     = "CLBADD_DB";                   /* The Client db name           */
const char * PULL_DATA_PATH     = "DATA/";                       /* smartPull DATA dir           */
const char * PUSH_DATA_PATH     = "DATA/";                       /* smartPush DATA directory     */
char * PUSH_BASE_PATH           = NULL;                          /* smartPush working directory  */
char * PULL_BASE_PATH           = NULL;                          /* smartPull working directory  */
char * CGI_DIR                  = NULL;                          /* smartPull working directory  */
char * TOR_USER                 = "_SYSTEM";                     /* The username                 */
char * TOR_PASSWORD             = "SECRET";                      /* The user password            */
baddErrorManager BEM;                                            /* Error handler for BADD       */
ClientErrorMap cem;                                              /* Paradise client err mngr.    */


//----------------------------------------------------------------------------------------
//  getTime() - Get the current system time in a timespec struct
//
//  ARGS:       curTime - a pass by ref timespec struct to set to the cur time
//  RETURN:     curTime - (by ref)
//----------------------------------------------------------------------------------------
void
getTime( timespec &curTime )
{
    timeval     t;
    gettimeofday(&t, NULL);
    curTime.tv_sec = t.tv_sec;
    curTime.tv_nsec = t.tv_usec * 1000;
}


//----------------------------------------------------------------------------------------
//  freeProfile() - deallocate strings of a profile
//
//  ARGS:       curTime - a pass by ref timespec struct to set to the cur time
//  RETURN:     curTime - (by ref)
//----------------------------------------------------------------------------------------
void
freeProfile( AProfile p )
{
    if(p.dbname) free(p.dbname);
    if(p.sqlString) free(p.sqlString);
    if(p.clientAddr) free(p.clientAddr);
}



//----------------------------------------------------------------------------------------
// split_string - This function takes a source char array, two string streams, and a 
//                delimiter The source string is split into two strings at the first 
//                occurance of the delimiter, and the two strings are stored in first and 
//                second, the two string streams.  If there is no split (no delimiter), 
//                false is returned
//
//  ARGS:         source - the source char array
//                first - a by ref ostrstream that will get the first part of the split
//                second - a by ref ostrstream that will get the second part of the split
//                delim - the char to split on
//
//  RETURN:       true if source was split, false if source had no delim
//----------------------------------------------------------------------------------------
bool 
split_string(char * source, ostrstream& first, ostrstream& second, char delim)
{
    
    second.freeze(0);   // unfreeze the string stream
    first.freeze(0);    // unfreeze the string stream
    
    first.seekp(0);
    second.seekp(0);
    
    int    count=0;
    char * ptr; 
    bool   b = false;
    
    // iterate throught the source char array and
    // copy from source into first string stream 
    // until we hit a delimiter or  '\0'
    for(ptr = source; *ptr; ptr++){
	if(*ptr==delim) {
	    b = true;   // set flag to indicate a split
	    break;
	}
	first << *ptr;
    }
    
    // iterate throught the rest of the source string and copy to
    // the second string stream, there may be delimiters
    // here, just copy them, they are used to further split the 'second'
    // string on subsequent calls to split
    first << ends;
    
    while(*(ptr++)) {
	second << *ptr;
    }
    second << ends;
    return b;
}


//----------------------------------------------------------------------------------------
// split_string - This function takes a source char array, two by ref char arrays, and a 
//                delimiter.  The source string is split into two strings at the first 
//                occurance of the delimiter, and the two strings are stored in first and 
//                second by ref char arrays.  If there is no split (no delimiter), 
//                false is returned
//
//  ARGS:         source - the source char array
//                first - a by ref char array that will get the first part of the split
//                second - a by ref char array that will get the second part of the split
//                delim - the char to split on
//
//  RETURN:       true if source was split, false if source had no delim
//----------------------------------------------------------------------------------------
bool 
split_string(const char* source, char*& first, char*& second, char delim)
{
    
    const char *  ptr; 
    bool          b = false;
    
    // iterate throught the source char array and
    // copy from source into first string stream 
    // until we hit a delimiter
    for(ptr = source; *ptr; ptr++){
	if(*ptr==delim) {
	    b = true;   // set flag to indicate a split
	    break;
	}
	*first = *ptr;
	first++;
    }
    
    // iterate throught the rest of the source string and copy to
    // the second string stream, there may be delimiters
    // here, just copy them, they are needed for next call to split
    *first = '\0';
    while(*(ptr++)) {
	*second = *ptr;
	second++;
    }
    *second = '\0';
    return b;
}



