////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  all_global.h
//
//  Contains definitions for variables and functions that are global to both 
//  smartPush and smartPull
//
//  Bruce Jackson
//  Professor David DeWitt
//
//  Jan 1999
//
////////////////////////////////////////////////////////////////////////////////////////////////////

#ifndef ALL_GLOBAL_H
#define ALL_GLOBAL_H

#include <iostream.h>
#include <strstream.h>
#include <stream.h>
#include <iostream.h>
#include <fstream.h>
#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/select.h>
#include <sys/time.h>
#include <time.h>
#include <errno.h>
#include <assert.h>
#include <iomanip.h>

#include <badd_error.h>
#include <cplusplusApi.h>


#define DEBUG
#define BLOCK_SIZE 1024                      /* unit of transfer over TCP sock               */
#define TS_INDEX 6                           /* the index of the TS field in profiles table  */
#define DEFAULT_LEN 2048                     /* Max length for a profile tanslation buffer   */
#define MAX_FILE_NAME 150                    /* Max buff length for filenames, tmp buffs etc */

extern int PARADISE_PORT;                    /* the default paradicse port              */
extern const char* PARADISE_PORT_FILE;       /* paradise port file                      */
extern const char* IDS_DB_NAME;              /* The IDS db name                         */
extern const char* CLIENT_DB_NAME;           /* The CLIENT db name                      */
extern const char* PULL_DATA_PATH;           /* smartPull working directory             */
extern const char* PUSH_DATA_PATH;           /* the data directory used by smart push   */
extern char* PUSH_BASE_PATH;                 /* The root directory for the ids          */
extern char* PULL_BASE_PATH;                 /* The root directory for pull client      */
extern char* CGI_DIR;                        /* the cgi directory where web client runs */
extern char* TOR_USER;                       /* The username                            */
extern char* TOR_PASSWORD;                   /* The user password                       */
extern baddErrorManager BEM;                 /* Error handler for BADD                  */
extern ClientErrorMap cem;                   /* Paradise client error manager           */


//--------------------------------------------------------------------------------
//   Definition of a profile structure.  This struct is used to store  
//   profiles in an in-memory timerTable (see below) used by smartPush 
//--------------------------------------------------------------------------------
typedef struct profile_tag {
    int      id;
    int      clientID;
    char*    clientAddr;
    char*    sqlString;
    char*    dbname;
    timespec ts_last;
    int      period; 
    int      port;	
    int      numBaseTables;
    bool     deleted;
}AProfile;


//--------------------------------------------------------------------------------
// These states are used to determine what SQL statements must be generated
// in the schema file when there are multiple loads stmts in one schema file
//--------------------------------------------------------------------------------
enum schemaState {
    GEN_OPEN,
    GEN_CLOSE,
    GEN_BOTH,
    GEN_NONE
};


//--------------------------------------------------------------------------------
//  Macros for printing messages.  Defined as null macros when debugging undefined
//--------------------------------------------------------------------------------
#ifdef DEBUG
#define INFO(MSG)       {                                                                  \
                             cerr << "INFO: " << __FUNCTION__ << "() - "                   \
                                  << __FILE__ << "(" << __LINE__                           \
                                  << "): " << MSG << ".\n";                                \
                        }                                                                  \

#define WARN(MSG)       {                                                                  \
                             cerr << "WARNING: " << __FUNCTION__ << "() - "                \
                                  << __FILE__ << "(" << __LINE__                           \
                                  << "): " << MSG << ".\n";                                \
                        }                                                                  \

#define ERROR(MSG)       {                                                                  \
                             cerr << "ERROR " << __FUNCTION__ << "() - "                \
                                  << __FILE__ << "(" << __LINE__                           \
                                  << "): " << MSG << ".\n";                                \
                        }                                                                  \

#else
#define INFO(MSG)
#define WARN(MSG)
#define ERROR(MSG)
#endif DEBUG

void getTime( timespec &curTime );
void freeProfile( AProfile p );
bool split_string(char* source, ostrstream& first, ostrstream& second, char delim);
bool split_string(const char* source, char*& first, char*& second, char delim);

#endif ALL_GLOBAL_H
