////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  mutex_multimap.h
//  
//  This is a wrapper around the STL multimap class. The main purpose of
//  this wrapper is to guard the data structure with a mutex. Sowmya defined
//  wrapper methods for only the select few interfaces needed for BADD.
//
//  Bruce Jackson,  modified from Sowmya Subramanian
//  Professor David DeWitt
//
//  Jan 1999
//
////////////////////////////////////////////////////////////////////////////////////////////////////

#ifndef MMULTIMAP_H
#define MMULTIMAP_H

#include <multimap.h>
#include <iostream.h>
#include "thread_mutex.h"
#include "thread_cond.h"


template <class Key, class T, class Compare = less<Key> >
class mutexMultimap : public multimap<Key, T, Compare> {
    public:
        void lock()   { _mutex.lock(); }
        void unlock() {	_mutex.unlock(); }
	void wait()   { _condv.wait(&_mutex.get_mutex()); }
        int timedwait(struct timespec *abstime) { 
	  return _condv.timedwait(&_mutex.get_mutex(),abstime);
	}
  	void signal()  { _condv.signal(); }

	iterator begin(bool hasLock=false) {
		if(!hasLock)
			_mutex.lock();
		iterator i = multimap<Key, T, Compare>::begin();
		if(!hasLock)
			_mutex.unlock();
		return i;
	}	
	
	iterator end(bool hasLock=false) {
		if(!hasLock)
			_mutex.lock();
		iterator i = multimap<Key, T, Compare>::end();
		if(!hasLock)
			_mutex.unlock();
		return i;
	}
	
        iterator insert(const value_type& x, bool hasLock=false) {
		if(!hasLock)
			_mutex.lock();
		iterator i = multimap<Key, T, Compare>::insert(x);
		if(!hasLock)
			_mutex.unlock();
		return i;	
	}
	
	void erase(iterator position, bool hasLock=false) {
	        if(!hasLock)
		   _mutex.lock();
		
		//--------------------------------------------------
		//  Deallocate dynamic fields of profile struct
		//--------------------------------------------------
		if( ((*position).second.sqlString) ){
		  free( (*position).second.sqlString );
		  (*position).second.sqlString=NULL;		  
		}
		if(  (*position).second.clientAddr ){
		  free( (*position).second.clientAddr);
		  (*position).second.clientAddr=NULL;		  
		}
		if ( (*position).second.dbname ){
		  free( (*position).second.dbname);
		  (*position).second.dbname=NULL;		  
		}
		(*position).second.deleted=true;		  
		multimap<Key, T, Compare>::erase(position);
		
		if(!hasLock)
		_mutex.unlock();
	}
  
	bool empty(bool hasLock=false) {
		if(!hasLock)
			_mutex.lock();
		bool b = multimap<Key, T, Compare>::empty();
		if(!hasLock)
			_mutex.unlock();
		return b;
	}
	
	size_type size(bool hasLock=false) {
		if(!hasLock)
			_mutex.lock();
		size_type s = multimap<Key, T, Compare>::size();
		if(!hasLock)
			_mutex.unlock();
		return s;
	}
	
	size_type count(const key_type& x,bool hasLock=false) {
		if(!hasLock)
			_mutex.lock();
		size_type s = multimap<Key, T, Compare>::count(x);
		if(!hasLock)
			_mutex.unlock();
		return s;	
	}


       void dump(int curtime) {
	       int i;

	       // get an iterator for the timerTable
	       mutexMultimap<timespec,AProfile,less<timespec> >::iterator ix;
	       bool emp;
	       int numProfiles = size(true);
      
	       cout << "\n\nAt Time: "<<curtime<<", Timer Table Contains:"<<endl;
	       cout <<"---------------------------------------------------------------------------------"<<endl;
	       cout <<"key   \t\tpid  \tcid  \taddr  \t\t\tport\tperiod\tts_last"<<endl;
	       cout <<"---------------------------------------------------------------------------------"<<endl;

	       emp = empty(true);
	       
	       if(emp){
		 cout << "(Empty)"<<endl;
	       }

	       else{
		 ix = begin(true);
		 for(i=0; i < numProfiles; i++){
		   cout <<(*ix).first.tv_sec <<"\t"
		   << (*ix).second.id <<"\t"
		   << (*ix).second.clientID <<"\t"
		   << (*ix).second.clientAddr <<"\t"
		   << (*ix).second.port <<"\t"
		   << (*ix).second.period <<"\t"
		   << (int)((*ix).second.ts_last.tv_sec) << endl;
		   
		   ix++;	
		 }
	       }
	       cout <<"---------------------------------------------------------------------------------"<<endl;
       }



       void empty_table() 
       {
	       // empty timer timer table and free allocated mem
	       int i;
	       
	       // get an iterator for the timerTable
	       mutexMultimap<timespec,AProfile,less<timespec> >::iterator ix;
	       bool emp;
	       int numProfiles = size(true);

	       emp = this.empty(true);
	       
	       if(emp){
		 return;
	       }
	       
	       else{
		 cout <<"num timer table elements: "<< numProfiles<<endl;
		 ix = begin(true);
		 for(i=0; i < numProfiles; i++){
		   
		   if ( (*ix).second.clientAddr != NULL ){
		     free( (*ix).second.clientAddr );
		     (*ix).second.clientAddr=NULL;
		   }
		   
		   if ( (*ix).second.dbname != NULL ){ 
		     free( (*ix).second.dbname );
		     (*ix).second.dbname=NULL;
		   }
		   if ( (*ix).second.sqlString != NULL ){ 
		     free( (*ix).second.sqlString );
		     (*ix).second.sqlString=NULL;
		   }
		   erase(ix, true);
		   ix++;	
		 }
	       }
       }
  
  
    private:
       P_Mutex _mutex;
       P_Cond _condv;
};

#endif


