
#include "pm_message.h"

pm_message::pm_message( int length )
{
     assert( length > 3*sizeof(int));
     data = NULL;
     data = new char[length];
     if(data){
	 bzero(data, length);
     }
}


pm_message::pm_message( msgTypes msgType, int cid, int pid, 
		  int sqllen, const char * sqlstring)
{
     //--------------------------------------------------
     //  Allocate memory and make some assertions
     //--------------------------------------------------
     int length = MIN_UPDATE_MSG_SIZE + sqllen;
     assert( UPDATE_MESSAGE_TYPE == msgType );
     assert( sqllen > 0 );
     data = NULL;
     data = new char[length];
     if(!data){
	 return;
     }

     //--------------------------------------------------
     // Set all fields of the update pm_message
     //--------------------------------------------------
     bzero( data, length);
     setMsgType( msgType );
     setCID( cid );
     setPID( pid );
     setMsgLength( length );
     setSQLLength( sqllen );
     setSQL( sqlstring );
}

pm_message::pm_message( msgTypes msgType, int cid, int pid, int port, 
		  int period, int numBase, int addlen, int sqllen, 
		  const char * addrstring, const char * sqlstring)
{

     //--------------------------------------------------
     //  Allocate memeory and make some assertions
     //--------------------------------------------------
     int length = MIN_ADD_MSG_SIZE + addlen + sqllen;
     assert( length > MIN_ADD_MSG_SIZE );

     data = NULL;
     data = new char[length];
     if(!data){
	 return;
     }

     //--------------------------------------------------
     // Set all fields of the add profile pm_message
     //--------------------------------------------------
     bzero( data, length);
     setMsgType( msgType );
     setCID( cid );
     setPID( pid );
     setMsgLength( length );
     setPort( port );
     setPeriod( period );
     setNumBase( numBase );
     setAddrLength( addlen );
     setSQLLength( sqllen );
     setAddr( addrstring );
     setSQL( sqlstring );
}

pm_message::pm_message( msgTypes msgType, int cid, int pid )
{
     //--------------------------------------------------
     //  Allocate memory and make some assertions
     //--------------------------------------------------
     assert( DELETE_MESSAGE_TYPE == msgType );
     data = NULL;
     data = new char[DEL_MSG_SIZE];
     if(!data){
	 return;
     }

     //--------------------------------------------------
     // Set all fields of the add profile pm_message
     //--------------------------------------------------
     bzero( data, DEL_MSG_SIZE);
     setMsgType( msgType );
     setCID( cid );
     setPID( pid );
     setMsgLength( DEL_MSG_SIZE );
}

pm_message::pm_message( const char * mes, int length )
{
     data = NULL;
     data = new char[length];
     if(data)
     {
	 bzero(data, length);      
	 memcpy(data, mes, length);
     }
}

pm_message::~pm_message( )
{
    if( data ){
	delete[] data;
    }
}

void    
pm_message::setMsgType( int msgType )
{
     u_long tmp = htonl( (u_long)msgType );
     memcpy( (void*)data, (void*)&tmp, sizeof(u_long) );
}

void    
pm_message::setCID( int cid )
{
     u_long tmp = htonl( (u_long)cid );
     memcpy( (void*)data+4, (void*)&tmp, sizeof(u_long) );     
}

void    
pm_message::setPID( int pid )
{
     ulong tmp = htonl( (u_long)pid );
     memcpy( (void*)data+8, (void*)&tmp, sizeof(u_long) );           
}

void    
pm_message::setMsgLength( int len )
{
     u_long tmp = htonl( (u_long)len );
     memcpy( (void*)data+12, (void*)&tmp, sizeof(u_long) );                 
}

void    
pm_message::setPort( int period )
{
     assert( getMsgType() == ADD_MESSAGE_TYPE );
     u_long tmp = htonl( (u_long)period );
     memcpy( (void*)data+16, (void*)&tmp, sizeof(u_long) );                 
}

void    
pm_message::setPeriod( int port )
{
     assert( getMsgType() == ADD_MESSAGE_TYPE );
     u_long tmp = htonl( (u_long)port );
     memcpy( (void*)data+20, (void*)&tmp, sizeof(u_long) );                 
}

void    
pm_message::setNumBase( int numBase )
{
     assert( getMsgType() == ADD_MESSAGE_TYPE );
     u_long tmp = htonl( (u_long)numBase );
     memcpy( (void*)data+24,(void*)&tmp, sizeof(u_long) );                      
}

void    
pm_message::setAddrLength( int len )
{
     assert( getMsgType() == ADD_MESSAGE_TYPE );
     u_long tmp = htonl( (u_long)len );
     memcpy( (void*)data+28, (void*)&tmp, sizeof(u_long) );                      
}

void    
pm_message::setSQLLength( int len)
{
     assert( getMsgType() == ADD_MESSAGE_TYPE || getMsgType() == UPDATE_MESSAGE_TYPE );
     int offset = MIN_ADD_MSG_SIZE-4;
     if (getMsgType() == UPDATE_MESSAGE_TYPE)
	 offset = MIN_UPDATE_MSG_SIZE-4;
     u_long tmp = htonl( (u_long)len );
     memcpy( (void*)data+offset, (void*)&tmp, sizeof(u_long) );                      
}

void    
pm_message::setSQL( const char * sql )
{
     int tp = getMsgType();
     assert( tp == ADD_MESSAGE_TYPE || tp == UPDATE_MESSAGE_TYPE);
     int offset = 0;

     if(tp == ADD_MESSAGE_TYPE){
	 offset = MIN_ADD_MSG_SIZE;
     }
     else{
	 offset = MIN_UPDATE_MSG_SIZE;
     }

     memcpy((void*)(data+offset), (void*)sql, getSQLLength());
}


void    
pm_message::setAddr( const char * addr )
{
     assert( getMsgType() == ADD_MESSAGE_TYPE);
     assert( getAddrLength() != 0);
     int offset =  MIN_ADD_MSG_SIZE + getSQLLength();
     memcpy( (void*)(data + offset), (void*)addr, getAddrLength());
}


int     
pm_message::getMsgType( )
{
     u_long tmp;
     memcpy((void*)&tmp, (void*)data, sizeof(u_long));
     return (int)ntohl(tmp);      
}

int     
pm_message::getCID( )
{
     u_long tmp;
     memcpy((void*)&tmp, (void*)data+4, sizeof(u_long));
     return (int)ntohl(tmp);      
}

int     
pm_message::getPID( )
{
     u_long tmp;
     memcpy((void*)&tmp, (void*)data+8, sizeof(u_long));
     return (int)ntohl(tmp);
}
int     
pm_message::getMsgLength( )
{
     u_long tmp;
     memcpy((void*)&tmp, (void*)data+12, sizeof(u_long));
     return (int)ntohl(tmp);
}


int     
pm_message::getPort( )
{
     assert ( getMsgType() == ADD_MESSAGE_TYPE );
     u_long tmp;
     memcpy((void*)&tmp, (void*)data+16, sizeof(u_long));
     return (int)ntohl(tmp);      
}

int     
pm_message::getPeriod( )
{
     assert ( getMsgType() == ADD_MESSAGE_TYPE );
     u_long tmp;
     memcpy((void*)&tmp, (void*)data+20, sizeof(u_long));
     return (int)ntohl(tmp);      
}

int     
pm_message::getNumBase( )
{
     assert (getMsgType() == ADD_MESSAGE_TYPE );
     u_long tmp;
     memcpy( (void*)&tmp, (void*)data+24, sizeof(u_long));
     return (int)ntohl(tmp);      
}


int     
pm_message::getAddrLength( )
{
     assert ( getMsgType() == ADD_MESSAGE_TYPE );
     u_long tmp;
     memcpy( (void*)&tmp, (void*)data+28, sizeof(u_long));
     return (int)ntohl(tmp);      
}


int     
pm_message::getSQLLength( )
{
     assert ( getMsgType() == ADD_MESSAGE_TYPE || getMsgType() == UPDATE_MESSAGE_TYPE);
     int offset = MIN_ADD_MSG_SIZE - sizeof(u_long);
     if( getMsgType() == UPDATE_MESSAGE_TYPE )
	 offset = MIN_UPDATE_MSG_SIZE - sizeof(u_long);
     u_long tmp;
     memcpy( (void*)&tmp, (void*)data+offset, sizeof(u_long));
     return (int)ntohl(tmp);      
}


char *  
pm_message::getAddr( )
{
     assert ( getMsgType() == ADD_MESSAGE_TYPE );
     char * buf = new char[ getAddrLength() ];
     memcpy((void*) buf, 
	    (void*)(data + MIN_ADD_MSG_SIZE+getSQLLength()), 
	    getAddrLength() );
     return buf;
}

char *  
pm_message::getSQL( )
{
     assert ( getMsgType() == ADD_MESSAGE_TYPE || 
	      getMsgType() == UPDATE_MESSAGE_TYPE );
     int offset = MIN_ADD_MSG_SIZE;
     if( getMsgType() == UPDATE_MESSAGE_TYPE )
	 offset = MIN_UPDATE_MSG_SIZE;
     char * buf = new char[ getSQLLength() ];
     memcpy( (void*)buf, (void*)data+offset, getSQLLength());
     return buf;
}

void
pm_message::getBytes( char* &buf )
{
     buf =  data;
}

ostream& 
operator<<(ostream& outs, pm_message * m)
{
 
     int msgType = m->getMsgType();
     char * ptr;

     outs << "\tPm_Message\n"
	  << "\t--------------------------------------------------------------------------\n"
	  << "\tpm_message type:   " << m->getMsgType();

     switch ( msgType )
     {
     case ADD_MESSAGE_TYPE:
	 outs << "\t(ADD_MESSAGE_TYPE)" << endl;
	 break;
     case DELETE_MESSAGE_TYPE:
	 outs << "\t(DELETE_MESSAGE_TYPE)" << endl;
	 break;
     case UPDATE_MESSAGE_TYPE:
	 outs << "\t(UPDATE_MESSAGE_TYPE)" << endl;
	 break;
     default:
	 outs << " (INVALID)";
     }
     outs << "\tcid:               " << m->getCID() << endl
	 << "\tpid:               " << m->getPID() << endl
	 << "\tmsg length:        " << m->getMsgLength() << endl;
     if( ADD_MESSAGE_TYPE == msgType ){ 
	 outs << "\t...........................................................................\n";
	 outs << "\tport:              " << m->getPort() << endl
	      << "\tperiod:            " << m->getPeriod() << endl
	      << "\tnum base:          " << m->getNumBase() << endl
	      << "\taddr len:          " << m->getAddrLength() << endl
	      << "\tsql len:           " << m->getSQLLength() << endl
	      << "\tsql:               ";

	 // output the SQL stmt, chop into pieces of length 70
	 int i = 0;
	 char * tmp = m->getSQL(); 
	 for(ptr = tmp; *ptr; ptr++){
	     outs << *ptr;
	     if(i%55 == 0  && i!= 0) outs << endl<<"\t\t\t    ";
	     i++;
	 }
	 outs << endl;
	 delete[] tmp;

	 outs << "\thost:              " << m->getAddr() <<endl;

     }else if ( UPDATE_MESSAGE_TYPE == msgType ){
	 outs << "\t...........................................................................\n";
	 outs << "\tsql len:           " << m->getSQLLength() << endl
	      << "\tsql:               ";
	 
	 // output the SQL stmt, chop into pieces of length 70
	 int i = 0;
	 char * tmp = m->getSQL();
	 for(ptr = tmp; *ptr; ptr++){
	     outs << *ptr;
	     if(i%55 == 0 && i != 0) outs << endl<<"\t\t\t";
	     i++;
	 }
	 outs << endl;
	 delete[] tmp;
     }
     outs << "\t--------------------------------------------------------------------------\n";
}






