////////////////////////////////////////////////////////////////////////////
//
//  profilt_thread.h
//
//  Bruce Jackson, August 1998    (modified from: Sowmya Subramanian, 1997?)
//  Prof. DeWitt
// 
//  This file contains the definition for the profileThread class. Profile
//  threads are created by the smartPush, and are responsible for executing
//  a given profile.  When a profile thread is created, it creates all the
//  temp tables it will need to determine what to disseminate.  These temp tables
//  have unique names based on the client id and profile id (a key for the 
//  profiles table).  The profile thread drops these temp tables when it finishes
//  executing the profile and disseminating data.
//
//  If a result is generated from the execution of a profile, that result is stored  
//  in files in a directory specific to that given profile/client pair 
//  <profile id, client id>. The directory is tarred, and the tar file is sent to 
//  the appropriate smartPull client. Finally, the profiles table and the in-memory 
//  timerTable are updated with the time last run for this profile, and the time to 
//  run it next. 
//
//////////////////////////////////////////////////////////////////////////////////


#ifndef PROFILET_H
#define PROFILET_H

/* General Includes */
#include <pthread.h>
#include <thread.h>
#include <push_global.h>
#include <all_global.h>
#include <iostream.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <string.h>
#include <errno.h>
#include <time.h>
#include <dirent.h>  // for working with dirs
#include <stl.h>

/* BADD Includes */
#include "sql_stuff.h"
#include "tcpSock.h"
#include "mutex_multimap.h"

typedef map<int, char*, less<int> >::value_type      queryMapValueType;
typedef map<int, char*, less<int> >::iterator        queryMapPtrType;

class profileThread : public P_Thread {
 public:
	profileThread(AProfile profile, bool & retcode);
	virtual ~profileThread();
	virtual int run();
 private:
	//char        (*queryStrings)[DEFAULT_LEN];
	map<int, char *, less<int> >       queryStringsMap;      // routing table
	int         numTempQueries;
	int         numFinalQueries;
	int         numTotalQueries;
	SQLQuery*   query;
	AProfile    _profile;
	int         createTempTables();
	int         dropTempTables();
	int         run_profile();
	int         send_profile();
	int         update_timerTable();
	void        clean_directory(const char* dirname);
	void        deleteQueryStrings();
};
  
#endif










