

#include <iostream.h>
#include <smartPull.h>
#include <sys/types.h>
#include <sys/stat.h>

void getEnvVars();

void
main(int argc, char **argv)
{
    
    int st, dataport;
    char * tmp = new char[100];
    
    //----------------------------------------
    //  Get environment variables
    //----------------------------------------
    getEnvVars();

    //------------------------------------------------------------
    //  Process the command line arguments
    //------------------------------------------------------------
    if(argc <= 3){
	cerr << "Usage: smartPull -P  -rport <port> -iport <port>"<<endl;
	cerr << "\n\t-P:   \tsmart pull clients paradise server port"<<endl;
	cerr << "\t-rport: \tsmartPull receiver port for prof mngmt requests"<<endl;
	cerr << "\t-iport: \tIDS receiver port for IDS profile registration etc"<<endl;
	cerr << "\t-dport: \tThe port that data will arrive to at smartPull from the IDS"
	     <<endl<<endl;
	exit(1);		
    }
    
    for(int argcount=1; argcount<argc; argcount++ ){
	
	// Get paradise port number from file
	if( strcmp(argv[argcount], "-P") == 0 ){
	    ifstream portfile(".paradise.port.number",ios::in);
	    if(!portfile){
		cerr << "unable to open .paradise.port.number"<<endl;
		exit(1);
	    }
	    portfile.gets(&tmp);
	    PARADISE_PORT = atoi((const char*)tmp);
	}
	
	// Set the receiver port
	else if( strcmp(argv[argcount], "-rport") == 0 ){
	    PULL_RECEIVER_PORT = atoi(argv[argcount+1]);    
	    argcount++;    
	}      
	
	// Set the ids receiver port
	else if( strcmp(argv[argcount], "-iport") == 0 ){
	    IDS_RECEIVER_PORT = atoi(argv[argcount+1]);
	    argcount++;
	}      
	
	// Set the data port
	else if( strcmp(argv[argcount], "-dport") == 0 ){
	    dataport = atoi(argv[argcount+1]);	    
	    argcount++;
	}      
    }
    
    // write the receiver thread port to a file so others know where to connect 	    
    ofstream pull_portfile(PULL_REC_PORT_FILE, ios::out);
    if(!pull_portfile){
	exit(1);
    }
    pull_portfile << PULL_RECEIVER_PORT << endl;
    pull_portfile.close();

    
    // Start the smartPull receiver
    smartPull spull( dataport );
    spull.run();	
}


void 
getEnvVars()
{
    char * tmp = NULL;
    char * buf = new char[100];
    bzero(buf, 100);

    //------------------------------
    //  Get the IDS_HOST 
    //------------------------------
    tmp = getenv( "IDS_HOST" );
    if(!tmp){
	cout << endl << endl 
	     <<"Set environment variable 'IDS_HOST' to the hostname of the IDS server.."
	     << endl << endl;
	exit(1);
    }
    IDS_HOSTNAME = strdup(tmp);

    //--------------------------------------------------
    //  Get the CGI_DIR and PULL_RECEIVER_PORT_FILE
    //--------------------------------------------------
    tmp = NULL;
    tmp = getenv( "CGI_DIR" );
    if(!tmp){
	cout << endl << endl 
	     <<"Set environment variable 'CGI_DIR' to path where cgi script badd.cgi resides.."
	     << endl << endl;
	exit(1);
    }
    CGI_DIR = strdup(tmp);
    memcpy(buf, CGI_DIR, strlen(CGI_DIR));
    PULL_REC_PORT_FILE = strcat(buf, "pull.receiver.port");

    //------------------------------
    //  Get the PULL_BASE_PATH 
    //------------------------------
    tmp = NULL;
    tmp = getenv( "PULL_BASE_PATH" );
    if(!tmp){
	cout << endl << endl 
	     <<"Set environment variable 'PULL_BASE_PATH' to bin dir for this SmartPull client.."
	     << endl << endl;
	exit(1);
    }
    PULL_BASE_PATH = strdup(tmp);

    cout << endl<<endl;
    cout << "SmartPull environment var info:"<<endl;
    cout << "---------------------------------------------------------------"<<endl;
    cout << "IDS_HOSTNAME:............. "<<IDS_HOSTNAME<<endl;
    cout << "CGI_DIR:.................. "<<CGI_DIR<<endl;
    cout << "PULL_RECEIVER_PORT_FILE:.. "<<PULL_REC_PORT_FILE<<endl;
    cout << "PULL_BASE_PATH:........... "<<PULL_BASE_PATH<<endl;
    cout << "---------------------------------------------------------------"<<endl<<endl;
}
    
    












