////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  push_global.h 
//
//  definitions of variables that are global to smartPush
//
//  Bruce Jackson
//  Professor David DeWitt
//
//  Jan 1999
//
////////////////////////////////////////////////////////////////////////////////////////////////////

#ifndef PUSH_GLOBAL_H
#define PUSH_GLOBAL_H

#include <thread_cond.h>
#include <thread_mutex.h>
#include <mutex_multimap.h>
#include <readWriteLock.h>
#include <badd_error.h>
#include <all_global.h>
#include <cplusplusApi.h>
//#include "profile_thread.h"
#include "queueObj.h"

//------------------------------------------------------------
//   Push global functions implemented in push_global.cpp
//------------------------------------------------------------
bool operator==(const timespec& x, const timespec& y);
bool operator<(const timespec& x, const timespec& y);

//------------------------------------------------------------
//   Push global variables instantiated in push_global.cpp
//------------------------------------------------------------
extern int IDS_RECEIVER_PORT ;         /* IDS port that listens for profile updates */
extern baddErrorManager BEM;           /* Error handler for BADD */
extern readWriteLock RWL;              /* Readers/Writers Lock */
extern ClientErrorMap cem;             /* client error manager provided by the Paradise API */
extern const char* IDS_RECEIVER_LOG;   /* Name of file where receiver thread logs transactions */
extern const char* IDS_REC_PORT_FILE;  /* IDS receiver port number is written to this file  */  


//extern int PARADISE_PORT;             /* paradise server port */
//extern int MY_RECEIVER_PORT ;          /* MY reciever port, for each clients rec thread */
//extern const char* CGI_DIR;            /* CGI Directory, various data files are written here */
//extern const char* SENDER_LOG;         /* Name of file where profile sender logs transaction */
//extern const char* MY_REC_PORT_FILE;   /* MY Receiver port number is written to this file  */  


//----------------------------------------------------------------------------------------
//  The following is the in memory data structure used to store profiles used by smartPush.
//  It is basically an in-mem copy of the profiles table stored in the DB mapped by the 
//  time each profile should be next run.  The typdefs are used to simplify the syntax of
//  STL.  The operators are needed by STL for use in the multimap
//----------------------------------------------------------------------------------------
extern mutexMultimap<struct timespec, AProfile, less<struct timespec> > timerTable;
typedef mutexMultimap<timespec,AProfile,less<timespec> >::iterator      timerTablePtrType;
typedef mutexMultimap<timespec,AProfile,less<timespec> >::value_type    timerTableValueType;

ostream& operator<<(ostream& x,const timespec& y);
bool operator==(const AProfile x, const AProfile y);
ostream& operator<<(ostream& x, const AProfile& y);

//----------------------------------------------------------------------------------------
//  The following is a global lockable queue used to store pointers to thread objects
//  so the main smartPush thread can free memory allocated to thread objects after the
//  the profile thread finishes.  Profile threads are detached, and insert themselves to   
//  this global queue when the finish
//----------------------------------------------------------------------------------------
extern queueObj * threadQueue; 


#endif




