
#include "queueObj.h"

//----------------------------------------------------------------------------
// queueObj() - Constructor
// 	INS : None
//	OUTS: None
//	DESC: Initializes q mutex
//----------------------------------------------------------------------------
queueObj::queueObj()
{
    qMutex = new pthread_mutex_t;
    assert(qMutex);
    pthread_mutex_init(qMutex, NULL);
}

//----------------------------------------------------------------------------
// ~queueObj() - Destructor
// 	INS : None
//	OUTS: None
//	DESC: Deletes q mutex
//----------------------------------------------------------------------------
queueObj::~queueObj()
{
    deque<dataType>::iterator i;

    pthread_mutex_destroy(qMutex);
    delete qMutex;
}


//----------------------------------------------------------------------------
// Put() - Insert element into the queue
// 	INS : data element
//	OUTS: None
//	DESC: Inserts the element into the queue
//----------------------------------------------------------------------------
void 
queueObj::Put( dataType element )
{
    q.push_back(element);
}

//----------------------------------------------------------------------------
// Lock() - Lock the queue
// 	INS : None
//	OUTS: None
//----------------------------------------------------------------------------
void 
queueObj::Lock()
{
    pthread_mutex_lock(qMutex);
}


//----------------------------------------------------------------------------
// Unlock() - Unlock the queue
// 	INS : None
//	OUTS: None
//----------------------------------------------------------------------------
void 
queueObj::Unlock()
{
    pthread_mutex_unlock(qMutex);
}


//----------------------------------------------------------------------------
// Get() - Pop element out of the queue and return it
// 	INS : None
//	OUTS: element (by ref)
//----------------------------------------------------------------------------
void 
queueObj::Get( dataType & element )
{
    assert(q.size() > 0);
    element = q.front();
    q.pop_front();
}


//----------------------------------------------------------------------------
// Size() - Get the current number of elements in the queue
// 	INS : None
//	OUTS: int size
//	DESC: lock, return size, unlock
//----------------------------------------------------------------------------
int 
queueObj::Size()
{
    return q.size();
}


//----------------------------------------------------------------------------
// dump() - Prints the queue
// 	INS : None
//	OUTS: None
//	DESC: Prints the queue
//----------------------------------------------------------------------------
void 
queueObj::Dump()
{
    deque<dataType>::iterator i;
    int ctr;
    
    // Do something with the dataType here
    for(i=q.begin(), ctr=0; i != q.end(); i++,ctr++) {
	//(*i)->Dump();
    }
}








