////////////////////////////////////////////////////////////////////////////////
//
//  smartPull.cpp
//  Bruce Jackson
//  Prof. DeWitt
//
//  This is the implementation of the smartPull receiver class.  smartPull listens
//  for incoming messages from the IDS.  When a message arrives, smartPull gets
//  the message, untarrs the message, and executes the SQL statements comtained
//  in the schema file sent with the data.  These statements will create the 
//  tables on the client side if need be, and then load tose tables with the data 
//  that just arrived.  After executing these SQL statements, smartPull listens
//  for the next message to arrive.
//
////////////////////////////////////////////////////////////////////////////////

#include <smartPull.h>


smartPull::smartPull(int port)
{
    bool con_st;
    _dataport = port;
    _socket = new TCPSock(_dataport, con_st);
    if(con_st == false){
	INFO("DATAPORT: "<<_dataport);
	perror("Smart Pull could not create TCP Socket (port)");
	exit(1);
    }
}


smartPull::~smartPull()
{
    _socket->close();
    _pushSocket.close();
    delete _socket;
}


void 
smartPull::run()
{
    
    timeval select_timer;
    int status;
    int i=1;
    
    set_timer(select_timer);
    
    ////////////////////////////////////////////////////////////////////
    //
    //  start the reciever thread.  This thread will handle profile
    //  adds/deletes (updates are done by add new, delete old)
    //  
    //  It will RECIEVE requests on our MY_RECEIVER_PORT and it
    //  will talk to IDS receiver on IDS_RECIEVER_PORT (both stored in files
    //  here, IDS_RECEIVER_PORT will be sent with every broadcast and
    //  checked/updated each time.  PARADISE_PORT is this client's paradise
    //  server port
    //
    ////////////////////////////////////////////////////////////////////
    pullReceiver rt;
    rt.start();
    
    
    updaterThread updThread;
    updThread.start();
    
    
    cout <<endl<<endl<<endl<<endl;
    cout <<"||||||||||||||||||||||||||||||||||||||||||||||||||||||||||" << endl;
    cout <<"||                                                      ||" << endl;
    cout <<"|| SMART PULL: information receiver                     ||" << endl;
    cout <<"||                                                      ||" << endl;
    cout <<"||||||||||||||||||||||||||||||||||||||||||||||||||||||||||" << endl;
    cout << endl<<endl<<"Listening on port "<<_dataport<<" for data from IDS site " << endl<<endl;
    
    for(;;) {
	
	status = do_select(select_timer);
	if(status == false){  /* There is no request yet .. continue */
	    continue;
	}
	
	/* There is a request -- accept it */
	status = _socket->accept(_pushSocket);
	if(status == false)
	    perror("Error while accepting message");
	
	cout <<endl<<endl;
	cout <<"||||||||||||||||||||||||||||||||||||||" << endl;
	cout <<"||                                  ||" << endl;
	cout <<"||  PULL: Message "<<i<<" arrival...      ||" << endl;
	cout <<"||                                  ||" << endl;
	cout <<"||||||||||||||||||||||||||||||||||||||" << endl;
	
	/* Get the message */
	status = get_msg();
	if(status == false)
	    perror("Error while getting message get");
	
	/* Process the message */
	status = process_msg();
	if(status == false)
	    perror("Error while processing message");
	
	cout <<"||||||||||||||||||||||||||||||||||||||||||||||||||" << endl;
	cout <<"||                                              ||" << endl;
	cout <<"||  PULL: Message "<<i<<" Processed, listening again..||" << endl;
	cout <<"||                                              ||" << endl;
	cout <<"||||||||||||||||||||||||||||||||||||||||||||||||||" << endl<<endl;
	
	i++;
    }
}



int 
smartPull::do_select(timeval timer)
{
    fd_set readfds;
    int count;
    
    FD_ZERO( &readfds);
    FD_SET (_socket->get_socket(), &readfds);
    
    count = select(FD_SETSIZE, (fd_set *) &readfds, (fd_set *) 0,
		   (fd_set *) 0, (struct timeval *)&timer);
    if(count < 0){
	if(errno == EINTR){
	    printf(" Nothing to panic \n");
	    return false;
	}
	else{
	    perror("Error: Select returned error %d \n");
	}
    }
    
    if(count > 0){
	if(FD_ISSET(_socket->get_socket(),&readfds)){
	    cout << "ConnectionSock found in FD_SET" << endl;
	    return true;
	}
	else{
	    perror("ConnectionSock not in FD_SET\n");
	}
    }
    return false;
}




//////////////////////////////////////////////////////////////////////////////
//
//  This method receives an incoming message.  It does the following
// 
//  1. Receive the profile ID
//  2. Receive the client ID
//  3. Receive the file size
//  4. Receive the tar file  (in chunks of BLOCK_SIZE)
//
//////////////////////////////////////////////////////////////////////////////
int 
smartPull::get_msg()
{
    int status;
    int size;
    int profileid;
    
    status = _pushSocket.get( _profileid );          /* Get the profile ID */
    if(status == false)
	perror("Error in getting the profile_id");
    status = _pushSocket.get(_myid); 	         /* Get my client ID */
    if(status == false)
	perror("Error in getting the client_id");
    
    status = _pushSocket.get(size); 	        /* Get the file size */
    if(status == false)
	perror("Error in getting the file size");
    
    
    /* Get the tarred file */
    int nblocks, nbytes;
    char *buf;
    buf = new char[BLOCK_SIZE];
    
    /* Dump the tarred file into file named: <_myid>.<profileid>.dir.tar */
    
    ofstream ifile;
    char dirname[50];
    
    sprintf(dirname,"%s%d.%d.dir.tar", PULL_DATA_PATH, _myid, _profileid);
    ifile.open(dirname,ios::out);
    
    nblocks = (int)size/(int)BLOCK_SIZE;             /* number of blocks */
    nbytes = (int)size%(int)BLOCK_SIZE;               /* bytes remaining */
    
    while(nblocks > 0) {
	memset(buf,0,BLOCK_SIZE);
	status = _pushSocket.get_bytes(buf,BLOCK_SIZE);
	if(status != BLOCK_SIZE){
	    perror(" Error in reading file ");
	    return false;
	}
	ifile.write(buf,BLOCK_SIZE);
	nblocks--;
    }
    memset(buf,0,BLOCK_SIZE);
    status = _pushSocket.get_bytes(buf,nbytes);
    if(status != nbytes){
	perror(" Error in reading file ");
	return false;
    }
    ifile.write(buf,nbytes);
    
    /* Close the file and socket descriptors */
    delete [] buf;
    ifile.close();
    _pushSocket.close();	
    return true;
}


/////////////////////////////////////////////////////////////////////////////////////////
//
//
//  
//
//
/////////////////////////////////////////////////////////////////////////////////////////
int 
smartPull::process_msg()
{
    int status, numTuples;
    char tar_name[100];
    
    // Un-tar the data file
#ifndef PUR_TEST
    sprintf(tar_name,"%s%d.%d.dir.tar", PULL_DATA_PATH, 
	    _myid, _profileid);
    ostrstream os;
    os << "tar -xvf " << tar_name << ends;
    status = system(os.str());
    status = status >> 8;
    os.freeze(0);
#endif
    
    /* execute query */
	char queryString[DEFAULT_LEN];
	char fname[MAX_FILE_NAME];
	
	/* construct the filename the schema information is stored in */
#ifdef PUR_TEST
	sprintf(fname,"/fs1/brucej/badd_stuff/DATA/JUNKDIR/100.1.schema");
#else
	sprintf(fname,"%s%d.%d.dir/%d.%d.schema", PULL_DATA_PATH,
		_myid,_profileid,_myid,_profileid);
#endif
	
	SQLQuery query(CLIENT_DB_NAME);   // query object to execute queries
	ifstream pf;
        pf.open(fname); 
	
	
	// get the number of base tables, stored on line 1 of the schema file
	pf.getline(queryString, DEFAULT_LEN, '\n');
	int numTables=atoi(queryString);
	
	// get and execute the first SQL stmt, "open db" stmt
	pf.getline(queryString, DEFAULT_LEN, '\n');
	status = query.execute(queryString, numTuples);
	
	// for each table, get the table name and see if it 
	// exists.  If not, execute the create table line that follows
	// otherwise scip executing the "create table" stmt
	// in either case, execute the "load table stmt"
	for( int i=0; i<numTables; i++){
	    pf.getline(queryString, DEFAULT_LEN, '\n');          // Table name
	    if ( (query.table_exists(queryString) ) == EXISTS)   // See if it exists
		pf.getline(queryString, DEFAULT_LEN, '\n');        // If so, skip the "create table" stmt
	    else{
		pf.getline(queryString, DEFAULT_LEN, '\n');        // create the table 
		cout << "PULL EXECUTING: "<<queryString<<endl;
		status = query.execute(queryString, numTuples);
	    }	
	    
	    
	    // Load the tuple by executing the load stmt
	    pf.getline(queryString, DEFAULT_LEN, '\n');
	    status = query.execute(queryString, numTuples);
	    cout << "PULL EXECUTING: "<<queryString<<endl;
	    cout << "Tuples loaded: "<<numTuples<<endl;
	}
	
	// execute the close stmt;
	pf.getline(queryString, DEFAULT_LEN, '\n');
	if( (status = query.execute(queryString, numTuples)) != OK)
	    BEM.display(status);
	
	pf.close();
	return true;
} 


void 
smartPull::set_timer(timeval &t){
    t.tv_sec = 5;
    t.tv_usec = 0;
}










