//////////////////////////////////////////////////////////////////////
//
//  SmartPush - IDS
// 
//  Bruce Jackson
//  Prof. DeWitt
//
//  1998-1999
//
//  Command line arguments:
//
//     Optional:
//     -----------------------------------------------------------------
//       -P                         Get paradise port from a file
//       -pport  <port number>      Specify paradise port
//       -rport  <port number>      Specify receiver port
//
//////////////////////////////////////////////////////////////////////

#include <iostream.h>
#include <all_global.h>
#include <push_global.h>
#include <profile_thread.h>
#include <ids_receiver.h>
#include <pthread.h>
#include <signal.h>
#include <cplusplusApi.h>


//-------------------------------
//  functions called by main
//-------------------------------
int   initTimerTable();
int   runProfiles();
void  getEnvVars();

//-------------------------------
//  main
//-------------------------------
int 
main(int argc, char **argv)
{
    int    argcount;
    bool   portset = false, dbset = false;	
    char * tmp = NULL;
    
    //----------------------------------------
    //  Get all Environment variables
    //----------------------------------------
    getEnvVars();
    
    //--------------------------------------------------
    //   Process the command line arguments
    //--------------------------------------------------
    for(argcount=1; argcount<argc; argcount++ ){
	
	// Get paradise port number from file
	if( strcmp(argv[argcount], "-P") == 0 ){
	    if(!portset){
		ifstream portfile(".paradise.port.number",ios::in);
		if(!portfile){
		    cerr << "\n\n-P option used but no file '.paradise.port.file' found\n\n";
		    exit(1);
		}
		char *tmp;
		portfile.gets(&tmp);
		PARADISE_PORT = atoi((const char*)tmp);
		portset = true;
	    }
	    else{ 
		cout << "\nInvalid argument: use either -P or -pport <portnum> "
		     << "to set the paradise port (not both)"<<endl;
		exit(1);
	    } 
	}
	
	// Paradise port number was passed in as an argument
	else if( strcmp(argv[argcount], "-pport") == 0 ){
	    if(!portset){
		PARADISE_PORT = atoi(argv[argcount+1]);
		portset=true;
		argcount++;
	    }
	    else{ 
		cout << "\nInvalid argument: use -P or -pport <portnum> "
		     << "to set the paradise port"<<endl;
		exit(1);
	    } 
	}
	
	// Set the receiver port
	else if( strcmp(argv[argcount], "-rport") == 0 ){
	    IDS_RECEIVER_PORT = atoi(argv[argcount+1]);
	    argcount++;
	}
    }
    
    //tmp = strdup(IDS_DB_NAME);
    
    // write the receiver thread port to a file so others know where to connect 
    ofstream rec_portfile(IDS_REC_PORT_FILE, ios::out);
    rec_portfile << IDS_RECEIVER_PORT << endl;
    rec_portfile.close();    
    
    
    //----------------------------------------
    //  Start the IDS Server 
    //----------------------------------------
    
    // Initialize the cursor
    ParaCursor::ParaAppInit( false ); 	
    
    // Initialize the timerTable
    if( initTimerTable() != OK ){
	perror("Failed to Initialize the timer table");
	exit(1);
    }
    
    //------------------------------------------------------------------
    //  Start reciever thread.  This thread listens for
    //  messages from smartPull clients and processes them.  It
    //  recieves profile and parameter add, update and delete requests.
    //------------------------------------------------------------------
    idsReceiver rt;
    rt.start();
    
    // Run profiles
    runProfiles();
    
    rt.end();
    
    // Close the cursor
    ParaCursor::ParaAppClose() ; 
    
    return 0;
}


///////////////////////////////////////////////////////////////////
//
// runProfiles
//
// Infinite loop that spawns profile threads on a timed basis
// (will run only 9 profiles in purify version and then exit)
//
///////////////////////////////////////////////////////////////////
int 
runProfiles()
{
    timerTablePtrType  ix;
    int                status, i;
    profileThread *    profile_t = NULL;    
    bool               done = false;
    bool               retcode;
    timespec           tmp;
    int                count = 0;
    int                numDeadThreads;

    threadQueue = NULL;
    threadQueue = new queueObj();
    if(!threadQueue){
	ERROR("Memory allocation failed");
	exit(1);
    }
    
    while(!done) {
	
	// Code to ensure ctrl-C and ctrl-D get caught 
	signal(SIGINT, SIG_DFL);      // catch ctrl-C 
	signal(SIGQUIT, SIG_DFL);     // catch ctrl-D      
	

	//---------------------------------------------------------
	// Delete any profile threads that have completed running
	//---------------------------------------------------------
	threadQueue->Lock();
	numDeadThreads = threadQueue->Size();
	//INFO("there are: "<<numDeadThreads << " dead threads to cleanup");
	for(i = 0; i < numDeadThreads; i++){
	    threadQueue->Get(profile_t);
	    if(profile_t){
		//INFO("deleting thread: "<<profile_t->get_tid());
		delete profile_t;
		profile_t = NULL;
	    }
	}
	threadQueue->Unlock();	


	timerTable.lock();            // lock the timerTable for this Reader
	// check for empty timerTable if empty, release locks, continue      
	if( timerTable.empty(true) ) {
	    timerTable.unlock();
	    sleep(3);
	    continue;
	}	  
	
	
	//--------------------------------------------------
	//   get the first element of timerTable and 
	//   set a timed wait accordingly
	//--------------------------------------------------
	ix = timerTable.begin(true);
	tmp.tv_sec = (*ix).first.tv_sec;
	tmp.tv_nsec = (*ix).first.tv_nsec;
	status = timerTable.timedwait(&tmp);
	
	//------------------------------------------------------------
	//   check if an updater deleted us while we were timed out
	//   and we woke up before being signaled
	//   also make sure we are still first on the list.  
	//   If not continue
	//------------------------------------------------------------
	if( ( (*ix).second.deleted == true)){
	    timerTable.unlock();
	    continue;
	}
	
	RWL.readerLock();            // GET: main thread reader lock
	
	//----------------------------------------------------------------------
	// We have resumed execution for two possible reasons
	// 1) Normal case:  we timed out
	//    start the next profile thread, it is ready to run.
	// 2) We were signaled by another thread updating the timerTable.
	//    continue on and get first element of timerTable in next loop iteration
	//----------------------------------------------------------------------
	if( (status == ETIMEDOUT) &&  (ix == timerTable.begin(true)) ) {
	    
	    //INFO("Starting profile: "<<(*ix).second.id<<" for client "<<(*ix).second.clientID );
	    
#ifdef PUR_TEST
	    if(count == 9){   // if this is a purify test
		timerTable.unlock();
		RWL.readerUnlock();
		return OK;
	    }
#endif
	    // Create a 'detached' profile thread
	    profile_t = new profileThread( (*ix).second, retcode );
	    
	    if( retcode == true ){
	      //freeProfile( (*ix).second );
		timerTable.erase( ix, true );  // remove entry ( deleted flag of profile is now TRUE )
		RWL.readerLock();              // get profile threads RW lock (reader)
		profile_t->start();            // start this profile thread
		if( profile_t->detach()){
		    WARN("Failed to detach thread: "<<profile_t->get_tid());
		}
		INFO("**** Started Profile Thread Number: " << ++count 
		      <<" with tid: "<<profile_t->get_tid());
	    }
	}
	timerTable.unlock();
	RWL.readerUnlock();      // RELEASE: main thread reader lock
    }
    
    return OK;
}




int 
initTimerTable()
{  
    //Set up a currsor object and authenticate with TOR_USER, TOR_PASSWORD
    int err;
    char *          hostname = NULL;
    ParaCursor *    cursor = NULL;
    ParaConnect *   connection = NULL;
    hostname =      getenv("PDHOST");
    if(!hostname){
	hostname = "localhost";
    }
    connection = new ParaConnect( hostname, PARADISE_PORT );
    if(! connection ){
	ERROR("Failed to create ParaConnection, aborting");
	exit(1);
    }
    
    cursor = new ParaCursor( *connection );
    if(! cursor){
	ERROR("Failed to create ParaCursor, aborting");
	delete connection;
	exit(1);
    }
    err = cursor->authenticate( TOR_USER, TOR_PASSWORD );
    
    if(err != clNOERROR){ 
	INFO("Authentication failed with err: "<<cem[err].message<<" for user: '" <<TOR_USER<<"', password: '"<<TOR_PASSWORD<<"' aborting");
	cem[err].message;
	delete cursor;
	delete connection;
	exit(1);
    }
    
    char queryStr[100];
    ClientBaseAdt** result;
    BaseAdtInfo::BaseAttrType attrType;
    int ret;
    AProfile pf;
    timespec t;
    timeval tval;
    
    // Open the database
    sprintf(queryStr, "OPEN DB %s", IDS_DB_NAME);
    cursor->declare(queryStr);
    ret = cursor->exec();
    INFO("ret val for: "<< queryStr<<" is: " <<ret); 
    
    
    // Select all profiles
    sprintf(queryStr, "SELECT * FROM PROFILES");
    cursor->declare(queryStr);
    cursor->open();
    ret = cursor->fetch();
    INFO("ret val for: "<< queryStr<<" is: " <<ret); 
    
    int attrCount;
    cursor->numAttrs(attrCount);
    
    // Get each profile from the "profiles" table 
    // in the main BADD db and insert it into the timerTable
    while(!ret) {
	
	cursor->getTuple(result);
	
	// set deleted flag to false
	pf.deleted = false;
	
	// retrieve each profile's fields and build a struct. 
	// Insert this struct into the in-memory timerTable
	for (int i=0; i<attrCount; i++)
	{
	    ClientBaseAdt *obj;
	    
	    cursor->getObj(obj,attrType,i);
	    switch (i) {
		
	    case 0:                       // profile id
		pf.id = 
		    (int)((ClientInt4Adt*)obj)->getIntVal();
		break;
	    case 1:                       // clientID
		pf.clientID = 
		    (int)((ClientInt4Adt*)obj)->getIntVal();
		break;
	    case 2:                       // the clientAddr
		pf.clientAddr = 
		    strdup((const char*)((ClientStringAdt*)obj)->getStrVal());
		break;
	    case 3:                       // the sqlString (profile)
		pf.sqlString = 
		    strdup((const char*)((ClientStringAdt*)obj)->getStrVal());
		break;
	    case 4:
		pf.dbname =                 // the main BADD dbname
		    strdup((const char*)((ClientStringAdt*)obj)->getStrVal());
		break;
		
	    case 5:                       // period for running this profile
		pf.period = (int)((ClientInt4Adt*)obj)->getIntVal();
		break;
		
	    case 6:                       // Time this profile was last run	       
		t.tv_sec = (int)((ClientInt4Adt*)obj)->getIntVal() + pf.period;
		t.tv_nsec = 0;
		
		pf.ts_last.tv_sec = (int)((ClientInt4Adt*)obj)->getIntVal();
		pf.ts_last.tv_nsec=0;
		break;
		
	    case 7:                          // client smartPull port
		pf.port = (int)((ClientInt4Adt*)obj)->getIntVal();
		break;
	    case 8:                          // the number of temp tables used to ex. this prof.
		pf.numBaseTables = (int)((ClientInt4Adt*)obj)->getIntVal();
		break;
	    }
	}
	
	// add this profile to the timerTable, with time as the key
	timerTable.insert( timerTableValueType( t,pf ) );
	ret = cursor->fetch();
	
#ifdef DEBUG      
	// debugging info
	cout <<"\n--------------------------------------\n";
	cout << "ProfileID: \t"         << pf.id         << endl;
	cout << "clientID: \t"          << pf.clientID   <<endl;
	cout << "clientAddr: \t"        << pf.clientAddr << endl;
	cout << "dbname: \t"            << pf.dbname     << endl;
	cout << "period: \t"            << pf.period     << endl;
	cout << "port num: \t"          << pf.port       << endl;
	cout << "num_temp_tables: \t"   << pf.numBaseTables   << endl;
	cout << "TS_Last Run: =\t\t"    << pf.ts_last.tv_sec  << endl;
	cout << "Time to run NEXT: \t"  << t.tv_sec           << endl;
	cout << "sqlString: \t"         << pf.sqlString  << endl;
	cout <<"----------------------------------------\n";
#endif    
    }
    
    if ( ret != clEND_TUPLES ) {
	cerr << "Cursor Error: "<< ret << "at: " << __FILE__ <<" "<< __LINE__ <<" :" 
	     << cem[ret].message << endl;
	cursor->close();      
	cerr << "Exit with error.\n";
	delete cursor;
	delete connection;
	return (-1);
    }
    
    cursor->close();

    // Close the database
    sprintf(queryStr, "CLOSE DB %s", IDS_DB_NAME);
    cursor->declare(queryStr);
    ret = cursor->exec();
    INFO("ret val for: "<< queryStr<<" is: " <<ret); 

    delete cursor;
    delete connection;
    return 0;      
}

void 
getEnvVars()
{
    char * tmp = NULL;

    //------------------------------
    //  Get the PUSH_BASE_PATH 
    //------------------------------
    tmp = getenv( "PUSH_BASE_PATH" );
    if(!tmp){
	cout << endl << endl 
	     <<"Set environment variable 'PUSH_BASE_PATH' to bin dir for IDS .."
	     << endl << endl;
	exit(1);
    }
    PUSH_BASE_PATH = strdup(tmp);

    //------------------------------
    //  Get the CGI_DIR 
    //------------------------------
    tmp = NULL;
    tmp = getenv( "CGI_DIR" );
    if(!tmp){
	cout << endl << endl 
	     <<"Set environment variable 'CGI_DIR' to path where cgi script badd.cgi resides.."
	     << endl << endl;
	exit(1);
    }
    CGI_DIR = strdup(tmp);

    cout << endl<<endl;
    cout << "SmartPush environment var info:"<<endl;
    cout << "---------------------------------------------------------------"<<endl;
    cout << "PUSH_BASE_PATH:..... "<<PUSH_BASE_PATH<<endl;
    cout << "CGI_DIR:............ "<<CGI_DIR<<endl;
    cout << "---------------------------------------------------------------"<<endl<<endl;
}




