////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  sql_stuff.h
//
//  This file contains all the methods for the SQLQuery class. These methods are
//  basic cursor operations that are used as interfaces to the paradise server.
//  The different methods include, query execute, schema dump, update query,
//  and query execute and dump.//  
//
//  Bruce Jackson
//  Professor David DeWitt
//
//  Jan 1999
//
////////////////////////////////////////////////////////////////////////////////////////////////////

#ifndef _SQL_STUFF_H
#define _SQL_STUFF_H

#include <pthread.h>
#include <all_global.h>
#include <cplusplusApi.h>


//********************* MACROS FOR USE IN CURSOR CLASS **************************

#define CURSOR_OPEN_CALL(x) \
  { \
      int ret = (x);                                                               \
      if( ret && ret != clEND_TUPLES)                                              \
       	{                                                                          \
      	   cerr << "** Cursor OPEN error: " << __FILE__ <<" "<< __LINE__<<" ";     \
       	   cerr << cem[ret].message <<endl;                                        \
          _cursor->close();                                                        \
          return CURSOR_ERROR;                                                     \
       	}                                                                          \
      if(ret && ret == clEND_TUPLES)                                               \
          return NO_RESULT;                                                        \
  }

//
// Execute a cursor call and just print message if call fails.  
//
#define CURSOR_CALL(x)                                                             \
 {                                                                                 \
      int ret = (x);                                                               \
      if( ret != clNOERROR )                                                       \
       	 {                                                                         \
       	   cerr << "Cursor Error: "<< cem[ret].message <<" ("<<ret<<") at: ";      \
           cerr << __FILE__ <<" "<< __LINE__<<" "<<__FUNCTION__<<endl;             \
       	 }                                                                         \
 }


//
// Execute a cursor call and return with no code if call fails.  
// If y == true, the db is open and should be closed before returning
//
#define CURSOR_CALL1(x, y)                                                         \
 {                                                                                 \
      int ret = (x);                                                               \
      if(ret && ret != clEND_TUPLES)                                               \
       	 {                                                                         \
       	   cerr << "Cursor Error: "<< cem[ret].message <<" ("<<ret<<") at: ";      \
           cerr << __FILE__ <<" "<< __LINE__<<" "<<__FUNCTION__<<endl;             \
           _cursor->close();                                                       \
           if(y) close_db();                                                       \
           return;                                                                 \
       	 }                                                                         \
 }

//
// Execute a cursor call and returns error code if call fails.  
// Return y if y not 0, else return the paradise err code.
// If z == true, the db is open and should be closed before returning
//
#define CURSOR_CALL2(x, y, z)                                                      \
  {                                                                                \
      int ret = (x);                                                               \
      if(ret && ret != clEND_TUPLES)                                               \
       	 {                                                                         \
       	     cerr << "Cursor Error: "<< cem[ret].message <<" ("<<ret<<") at: ";    \
             cerr << __FILE__ <<" "<< __LINE__<<" "<< __FUNCTION__<<endl;          \
             _cursor->close();                                                     \
             if( z ) close_db();                                                   \
             if( y != 0 ) { return y; }                                            \
             else { return CURSOR_ERROR; }                                         \
       	 }                                                                         \
  }

#define CLEANUP {                                                                  \
	          _cursor->close();                                                \
                  close_db();                                                      \
                }

//******************************************************************************




class SQLQuery {

    public:
	SQLQuery();
	SQLQuery( const char *dbname, int port = PARADISE_PORT );
	~SQLQuery();

	int execute_and_dump( char* query, char* prof_dir, int numTables, 
			     int curNum, int cid, int pid );
	int execute( char *, int &numTuple );
	int close_db( void );
	int open_db( void );
	int dump_schema( char* schemaFile, char* tableName, char* dataFile, int attrCount, 
			  int numTables, schemaState state );
	//int run_update( char *query, timespec& ttime );
	int run_update( int cid, int pid , int runtime );
	int table_exists( char* table_name );
	int html_table_dump( char* queryStr, char* filename );
	
	//int export_badd_schema();
	
    protected:
	char *        _hostname;
	char          _queryStr[DEFAULT_LEN];
	char *        _dbname;
	int           _port;
	ParaConnect * _connection;
	ParaCursor *  _cursor;
};

#endif


















