////////////////////////////////////////////////////////////////////////////////
//
//  tcpSock.cpp
//
//  Bruce Jackson:  from  Sowmya Subramanian
//  Prof. David DeWitt, BADD-Project
//
//  This file contains the definition of a generic TCPSock class.
//
//////////////////////////////////////////////////////////////////////////////////

#include <tcpSock.h>

TCPSock::TCPSock()
{
	int status;
	
	status = assign();
	if(status == false){
		perror("Cannot open stream socket");
		exit(0);
	}
}

TCPSock::TCPSock(int port, bool& con_stat)                                 /* Listen on Port */
{
	int status;

	status = listen(port);
	if(status == false) {
		perror("Cannot listen on port");
		con_stat=false;
	}
	else{
	        con_stat=true;
	}
}

TCPSock::TCPSock(char * host, int port, bool& con_stat)        /* Connect to host/port */
{
  //char * host_l = strdup(host);
	int status;
	char msg[100];
	sprintf(msg,"Could not connect to host '%s', port '%d'", host, port); 
	status = connect(host,port);
	if(status == false){
	    perror(msg);
	}
	//con_stat = false;
	//}
	//else{
	con_stat = status;
	//}
}

TCPSock::~TCPSock()
{ 
    close();
}

int 
TCPSock::assign()
{
    if ((_sockfd = socket(AF_INET, SOCK_STREAM, 0)) < 0) return false;
    return true;
}

int 
TCPSock::bind(int port)
{
    sockaddr_in		addr;
    int status;
    
    status = assign();                                /* Assign a socket */
    
    if(status == false) return status;   
    
    bzero(&addr,sizeof(sockaddr_in));
    addr.sin_family = AF_INET;
    addr.sin_addr.s_addr = htonl(INADDR_ANY);
    addr.sin_port = htons((u_short)port);
    
    if(::bind(_sockfd, (sockaddr *)&addr, sizeof(sockaddr_in)) < 0)
	return false;
    
    return true;
}

int 
TCPSock::listen()
{
    if(::listen(_sockfd, 5) < 0) return false;
    return true;
}

int 
TCPSock::accept(TCPSock& clientsock)
{
    int clientsockfd;
    sockaddr_in addr;
    int addr_size;
    
    addr_size = sizeof(addr);
    if((clientsockfd = ::accept(_sockfd, (sockaddr *)&addr, 
				&addr_size)) < 0)
	return false;
    clientsock._sockfd = clientsockfd;
    return true;
}

int 
TCPSock::accept(TCPSock *clientsock)
{
    if(!clientsock) return false;
    return accept(*clientsock);
}

TCPSock * 
TCPSock::accept()
{
    TCPSock *clientsock;
    int clientsockfd;
    sockaddr_in addr;
    int addr_size;
    
    if(!(clientsock = new TCPSock())) return (TCPSock *)0;
    
    addr_size = sizeof(addr);
    if((clientsockfd = ::accept(_sockfd, (sockaddr *)&addr, 
				&addr_size)) < 0)
	return (TCPSock *)0;
    
    clientsock->_sockfd = clientsockfd;
    return clientsock;
}

int 
TCPSock::close()
{
    if(::close(_sockfd) < 0) return false;
    return true;
}

int TCPSock::connect(const char *host, int port)
{
    return do_connect(host,port);
}

int TCPSock::get_port()
{
    sockaddr_in addr;
    int addr_len;
    
    addr_len = sizeof(sockaddr_in);
    
    if (getsockname(_sockfd, (sockaddr *)&addr, &addr_len) < 0) return -1;
    
    return (int) ntohs(addr.sin_port);
}

int 
TCPSock::do_connect(const char *host, int port)
{
    sockaddr_in  sin;
    hostent *    hostp;
    int          status;
    
    if (!host || port < 0) return false;
    
    status = bind();
    
    if(status == false) return false;
    
    bzero((char *)&sin,sizeof(sockaddr_in));
    sin.sin_family = AF_INET;
    sin.sin_port = htons((u_short)port);
    
    /* if dotted notation fails, try host database	*/
    if ((hostp = gethostbyname(host)) == (hostent *)0) return false;
    
    memcpy(&sin.sin_addr, hostp->h_addr, hostp->h_length);
    
    
    if (::connect(_sockfd, (sockaddr *)&sin, sizeof(sockaddr_in)) < 0)
	return false;
    
    return true;	
}


int 
TCPSock::get_bytes(void *data, int size)
{
    int nleft, nread;
    
//	memset((char*)data,0,size);                       /* Initialize data */
    nleft = size;
    while(nleft > 0) {
	nread = read(_sockfd,data,nleft);
	
	if(nread < 0)
	    return false;
	if(nread == 0)
	    break;
	nleft -= nread;
	data += nread;
    }
    return(size - nleft);
}

int 
TCPSock::put_bytes(const void *data, int size)
{
    int nleft, nwritten;
    
    nleft = size;
    while(nleft > 0){
	nwritten = write(_sockfd, data, nleft);
	if(nwritten <= 0)
	    return false;
	nleft -= nwritten;
	data += nwritten;
    }
    return (size - nleft);
}  


int 
TCPSock::get(int &i)
{
    int status=0;
    int tmp=0;
    
    status = get_bytes(&tmp, sizeof(int));
    if(status != sizeof(int))
	return false;
    i = ntohl(tmp);
    return true;
}

int 
TCPSock::put(int i)
{
    int tmp=0;
    
    tmp = htonl(i);
    if(put_bytes(&tmp,sizeof(int)) != sizeof(int))
	return false;
    return true;
}
