
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//  Define these two according  to your env var settings,
//  PORT_FILE = CGI_DIR + "pull.receiver.port"
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

#define PORT_FILE   "/u/b/r/brucej/public/cgi-bin/badd/pull.receiver.port"
#define IDS_HOST    "e-fitz.cs.wisc.edu"
#define BLOCK_SIZE 1024

///////////////////////////////////////////////////////////////////
// sendProfile.cpp
// 
// Bruce Jackson, July 1998
//
//  Profile sender takes all profile managment information as command 
//  line arguments, creates a TCP connection to the receiver thread 
//  and sends the profile management request.  The receiver thread 
//  processes the request and sends back an error code based on the 
//  success or failure of processing the request.  These are the 
//  possible profile management requests:
//
//   1:  add_prof       Add a new profile
//   2:  del_prof       Delete a profile
//   3:  (unimplemented)
//   4:  up_param       Update a parameter command
//   5:  view_param     View current parameters
//
//  For adds and updates, the current version requires that all profile
//  information must be sent, including location and words of interest.
//  Deletes require only the clientId and profileId, which uniquely
//  identifies a profile.  The profile management transaction is 
//  logged to the file "profsend.log" for debugging.  Error codes
//  are sent back to the CGI script that executes this program.
//
//  There are two classes of errors that can be sent back to the CGI script:
//
//  A: Errors generated in profsend  
//
//     1:  Failed to open profile log file/
//     3:  No code flag was given (must know the type of request, add/delete/update)
//     4:  Invalid command line argument
//     5:  Wrong arguments used for an add or update request
//     6:  Wrong arguments used for a delete request
//     7:  Invalid code used with -code flag
//     20: Error in sending an integer to the receiver thread
//     21: Error in sending a variable length buffer to the receiver thread
//     25: Failed to receive an error code from the receiver thread
//
//  B: Errors generated in receiver thread:
//
//     7:  Failed parsing the query
//     8:  Typechecking query failed
//     9:  Translation of query failed
//     11: Failed to add profile
//     12: Trying to add a profile that exists already
//     13: Failed to delete profile
//     14: Trying to delete a profile that does not exist
//     26: Paradise cursor call error (close db)
//     27: Another cursor error
//     28: Paradise call failed
//
//  
//  Flags used for command line arguments
//
//     -cid       [client id]
//     -pid       [profile id]
//     -period    [period]
//     -addr      [client ip address]
//     -port      [client port] 
//     -sql       [sql statement to be translated]
//     -param_sql [sql string for param update]
//     -code      [ add_prof,2,3]
//     
//////////////////////////////////////////////////////////////////


#include "tcpSock.h"
#include <string.h>
#include <fstream.h>
#include <time.h>

void finish(ofstream &outf, TCPSock* sock, int exitcode);

void
main(int argc, char** argv)
{
  
    // set the REMOTE PARADISE SERVER port
    ifstream portfile( PORT_FILE, ios::in);
    if(!portfile){
	exit(400);
    }
    
    char * tmp = new char[20];
    portfile.gets(&tmp);
    portfile.close();
    int receiverPort = atoi((const char*)tmp);	
    
    // open the profsend log file
    ofstream out("sender.log", ios::app);
    if(!out){
	portfile.close();
	exit(1);
    }
    
    int status;
    bool st;
    time_t my_time = time(NULL);
    int count=1;
    
    // command line arg vars get stored in these vars
    int code=0, return_code;
    int port=-1, cid=-1, pid=-1, period=-1;
    char * addr = NULL;
    char * sqlstring = NULL;
    char * paramstring = NULL;
    
    int addlen=0, sqllen=0, paramlen=0;
    int nblocks, nbytes;
    char * buf = new char[1024];
    int len, i;
    int rport=-1;
    TCPSock * sock=NULL;
    
    
    out << "Begin profile log transaction:"<<endl;
    out << "-------------------------------:"<<endl;
    out << "transaction start time\t" << ctime(&my_time);
    
    out <<"receiver port: \t\t"<<receiverPort<<endl;
    
    // Process the command line args
    for(count=1; count<argc-1; count+=2 ){
	if( strcmp(argv[count], "-cid") == 0 ){
	    cid = atoi(argv[count+1]);
	    out << "cid: \t\t\t"<<cid<<endl; 
	}
	else if( strcmp(argv[count], "-pid") == 0 ){
	    pid = atoi(argv[count+1]);
	    out << "pid: \t\t\t"<<pid<<endl; 
	}
	else if( strcmp(argv[count], "-code") == 0 ){	    
	    out << "code: \t\t\t"<< argv[count+1]<<endl;
	    if (strcmp(argv[count+1], "add_prof") == 0) 
		code = 1;
	    else if (strcmp(argv[count+1], "del_prof") == 0) 
		code = 2;
	    else if (strcmp(argv[count+1], "up_param") == 0) 
		code = 4;
	    else if (strcmp(argv[count+1], "view_param") == 0) 
		code = 5;
	    else{
		out << "\t**Error: Invalid argument for -code flag: usage -code [add_prof/del_prof/up_prof/up_param/view_param]\n";
		finish(out, sock, 7);
	    }
	}
	else if( strcmp(argv[count], "-port") == 0){
	    port = atoi(argv[count+1]);
	    out << "port: \t\t\t"<<port<<endl; 
	}
	else if( strcmp(argv[count], "-period") == 0){
	    period = atoi(argv[count+1]);
	    out << "period: \t\t"<<period<<endl; 
	}
	else if( strcmp(argv[count], "-addr") == 0){
	    len = strlen(argv[count+1]);
	    addr = new char[len+2];
	    addr[len+1]='\0';
	    memcpy (addr, argv[count+1], len);
	    out <<"addr: \t"<<argv[count+1]<<", length: "<< len <<endl;
	    addlen = strlen(addr);
	}
	else if( strcmp(argv[count], "-sql") == 0){
	    len = strlen(argv[count+1]);
	    sqlstring = new char[len+1];
	    sqlstring[len]='\0';
	    memcpy (sqlstring, argv[count+1], len);
	    out <<"SQL: "<<argv[count+1]<<", length: "<< len <<endl;
	    sqllen = strlen(sqlstring);
	}
	else if( strcmp(argv[count], "-param_sql") == 0){
	    len = strlen(argv[count+1]);
	    paramstring = new char[len+1];
	    paramstring[len]='\0';
	    memcpy (paramstring, argv[count+1], len);
	    out <<"Words of Interest:\t"<<argv[count+1]<<", length of word string: "<< len <<endl;
	    paramlen = strlen(paramstring);
	}
	else if( strcmp(argv[count], "-help") == 0){
	    out << "\tUsage: profsend -[flag1] flag1arg -[flag2] flag2arg ......."<<endl
		<< "\t\tflags   Arguments for flags"<<endl
		<< "\t\t-----   -------------------"<<endl
		<< "\t\t-help      display this screen"<<endl
		<< "\t\t-cid       [client id]"<<endl
		<< "\t\t-pid       [profile id]"<<endl
		<< "\t\t-period    [period]"<<endl
		<< "\t\t-addr      [client ip address]"<<endl
		<< "\t\t-port      [client port] "<<endl
		<< "\t\t-sql       [sql statement to be translated]"<<endl
		<< "\t\t-param_sql [param update sql string]"<<endl
		<< "\t\t-code      [1,2,4,5]"<<endl<<endl;
	    finish(out, sock, 4);
	}
	else{
	    out << "\t**Error: Invalid arg: "<<argv[count]<<endl;
	    out << "\tUsage: profsend -[flag1] flag1arg -[flag2] flag2arg ......."<<endl
		<< "\t\tflags   Arguments for flags"<<endl
		<< "\t\t-----   -------------------"<<endl
		<< "\t\t-help      display this screen"<<endl
		<< "\t\t-cid       [client id]"<<endl
		<< "\t\t-pid       [profile id]"<<endl
		<< "\t\t-period    [period]"<<endl
		<< "\t\t-addr      [client ip address]"<<endl
		<< "\t\t-port      [client port] "<<endl
		<< "\t\t-sql       [sql statement to be translated]"<<endl
		<< "\t\t-param_sql [param update sql string]"<<endl
		<< "\t\t-code      [1,2,4,5]"<<endl<<endl;
	    finish(out, sock, 4);
	}
    }
    
    // do some errror checking before trying to send information over sock
    if( code == 0 ){
	out << "\t**Error: A message type code was not given: "<<endl
	    << "\t\t(codes: 1=add_prof, 2=del_prof, 3=up_prof, 4=up_param,"
	    << "\n\t\t        5=view_param)"<<endl;
	finish(out, sock, 3);
    }
    if( pid == -1 || cid == -1 ){
	out << "\n\t**Error: no pid or cid was sent"
	    << endl;
	out << "\t\t   useage: -code [a code] -cid <cid> -pid <pid>.... "
	    <<endl;
	finish(out, sock, 6);
    }
    if( code == 1 ){  // see that add or update has right args
	if( pid == -1 || cid == -1 ||
	    port == -1 || period == -1 ||
	    addr == NULL || sqlstring == NULL ){
	    out << "\n\t**Error: Wrong arguments for add or update message"
		<< endl;
	    out << "\t\t   useage: -code [add_prof] -cid <cid> -pid <pid> -period <period>\n"
		<< "\t\t           -port <port> -addr <address> -sql <sql-string>"<<endl;
	    finish(out, sock, 5);
	}
    }
    if( code == 4 || code == 5){  
	// see that keyword add, update or delete has the right args
	if(paramstring==NULL){
	    out << "\n\t**Error: Wrong arguments for add, del, or update words"
		<< endl;	    
	    finish(out, sock, 6);
	}
    }
    
    // Create a Socket to the reciever port
    sock = new TCPSock( IDS_HOST, receiverPort, st);
    if( ! st ){
	finish(out, sock, 1);
    }
    
    
    //////////////////////////////////////////////////////////////////////////
    //
    //  Send the profile management request.  For all messages, we start by 
    //  sending the code, pid then cid.  Various other fields send based on code
    //
    //////////////////////////////////////////////////////////////////////////
    if(! ( sock->put( code ) )) {          // message type code
	out <<"\t**Error: Could not send Code"<<endl;
	finish(out, sock, 20);
    }
    if( !( sock->put( pid ) )) {           // profile id
	out <<"\t**Error: Could not send profile ID"<<endl;
	finish(out, sock, 20);
    }
    if( !( sock->put( cid ) )) {           // client id
	out <<"\t**Error: Could not send client ID"<<endl;
	finish(out, sock, 20);
    }
    
    // send what is needed depending on the message type
    switch (code){
    case 1:          /* add a new profile */
	buf = new char[1024];
	if (addlen>1024 || sqllen >1024){
	    out << "\t**Error: sql string or addr too long"<<endl;
	    finish(out, sock, 2);
	}
	if( !(sock->put( port ) )) {            // cli port
	    out <<"\t**Error: Could not send the port"<<endl;
	    finish(out, sock, 20);
	}
	if( ! (sock->put( period ) )) {         // clint period
	    out <<"\t**Error: Could not send the period"<<endl;
	    finish(out, sock, 20);
	}
	if( ! (sock->put( addlen ) ))  {        // length of address field
	    out <<"\t**Error: Could not send the length of the address"<<endl;
	    finish(out, sock, 20);
	}
	if( ! (sock->put( sqllen ) ))  {        // length of query field 
	    out <<"\t**Error: Could not send the length of the sql string"<<endl;		 
	    finish(out, sock, 20);	
	}
	
	// Send the address
	memcpy (buf, addr, addlen);
	if( sock->put_bytes( ((void*)buf) , addlen) != addlen) {
	    out <<"\t**Error: Could not send the address"<<endl;
	    finish(out, sock, 21);
	}
	
	// Send the query
	memcpy(buf, sqlstring, sqllen);
	if( sock->put_bytes( ((void*)buf), sqllen) != sqllen ){
	    out <<"\t**Error: while sending query "<<endl;
	    delete [] buf;
	    finish(out, sock, 21);
	}
	
	delete [] buf;
	delete [] addr;
	delete [] sqlstring;
	break;
	
    case 2:
	// delete profile, all info has been sent (pid cid code)
	break;
	
    case 4: case 5:      /* add, update or delete words */
	// send the length of the word string
	if( ! (sock->put( paramlen ) ))  {        
	    // length of words string field 
	    out <<"\t**Error: Could not send length of paramstring"
		<<endl;	       
	    finish(out, sock, 20);	
	}
	
	// Send the words of interest
	memcpy(buf, paramstring, paramlen);
	if( sock->put_bytes( ((void*)buf), paramlen) != paramlen ){
	    out <<"\t**Error: while sending words of interest "<<endl;
	    delete [] buf;
	    finish(out, sock, 21);
	}	       
	break;
	
    default:        /* error checking above insures we won't get here */
	break;	  
    }
    
    // get the return code from the receiver thread
    if( ! (sock->get( return_code ) )) {        // client period
	out <<"\t**Error: Could not get the return code"<<endl;
	finish(out, sock, 25);
    }
    
    finish(out, sock, return_code);
}

void 
finish(ofstream &outf, TCPSock* sock, int exitcode)
{
    outf <<"exitcode: "<<exitcode<<endl; 
    outf <<"-------------------------------:"<<endl;
    outf <<"End profile log transaction:"<<endl<<endl<<endl;
    sock->close();
    outf.close();
    exit (exitcode);  
}	








